@@ -1,7 +1,7 @@
 #!/bin/sh
 
 # 
-#   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+#   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 # 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -181,6 +181,10 @@ do
         aclocalinclude="$ACLOCAL_FLAGS"
      fi
 
+     if test -d cygnal; then
+        aclocalinclude="-I cygnal ${aclocalinclude}"
+     fi
+
      if test -d libltdl/m4; then
         aclocalinclude="-I libltdl/m4 -I macros $ACLOCAL_FLAGS"
      fi
