@@ -1,5 +1,5 @@
 
-//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -15,30 +15,33 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-/* $Id: aqua.cpp,v 1.32 2008/03/21 04:59:51 nihilus Exp $ */
+/* $Id: aqua.cpp,v 1.33 2008/05/26 22:13:33 ann Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "gnashconfig.h"
 #endif
 
 extern "C"{
-#include <unistd.h>
-#ifdef HAVE_GETOPT_H
-	#include <getopt.h>
-#endif
-//	extern int getopt(int, char *const *, const char *);
+# ifdef HAVE_GETOPT_H
+#  include <getopt.h>
+# endif
+# ifndef __GNUC__
+  extern int getopt(int, char *const *, const char *);
+# endif
 }
 
 #include "gnash.h"
-#include "gui.h"
 #include "aquasup.h"
 #include "log.h"
 #include "movie_root.h"
+#include "RunResources.h"
 
-#include "render_handler.h"
+#include "Renderer.h"
 
 #include <Carbon/Carbon.h>
 
+using namespace std;
+
 namespace gnash {
 
 /* Main window widget */
@@ -54,14 +57,13 @@ pascal OSStatus DoWindowClose (EventHandlerCallRef  nextHandler,
 
 void DoAdvanceMovie ( EventLoopTimerRef inTimer, void* data)
 {
-   GNASH_REPORT_FUNCTION;
    AquaGui* gui = static_cast<AquaGui*>(data);
    Gui::advance_movie(gui);
 }
   
 	
-AquaGui::AquaGui(unsigned long xid, float scale, bool loop, unsigned int depth)
-	: Gui(xid, scale, loop, depth),
+AquaGui::AquaGui(unsigned long xid, float scale, bool loop, RunResources& r)
+	: Gui(xid, scale, loop, r),
           _advance_timer(NULL)
 {
 }
@@ -76,7 +78,6 @@ AquaGui::~AquaGui()
 
 bool AquaGui::run()
 {
-  GNASH_REPORT_FUNCTION;
     double interval = _interval / 1000.0;
 
 
@@ -96,7 +97,6 @@ bool AquaGui::run()
 
 void AquaGui::renderBuffer()
 {
-    GNASH_REPORT_FUNCTION;
     _glue.render();
 
       Rect rectPort;
@@ -111,8 +111,6 @@ bool AquaGui::init(int argc, char **argv[]) /* Self-explainatory */
 	long response;
 	Str255 text = " OS X version lower than 10.4 is not supported!", tmp = "";
   
-	GNASH_REPORT_FUNCTION;
-	
 	/* Version check */
 	err = Gestalt(gestaltSystemVersion, &response);
 	Boolean ok = ((err == noErr) && (response >= 0x00001040));
@@ -125,10 +123,10 @@ bool AquaGui::init(int argc, char **argv[]) /* Self-explainatory */
       	
   	_glue.init(argc, argv);
 
-    _renderer = _glue.createRenderHandler();
+    _renderer.reset(_glue.createRenderHandler());
     if(!_renderer)return false;
 
-    set_render_handler(_renderer);
+	_runResources.setRenderer(boost::shared_ptr<Renderer>(_renderer));
     return true;
 
 }
@@ -144,7 +142,6 @@ void AquaGui::key_event(int key, bool down)
 
 void AquaGui::setCursor(gnash_cursor_type newcursor)
 {
-	  GNASH_REPORT_FUNCTION;
 
 	  switch(newcursor) {
 	  	case gnash::CURSOR_HAND:	  		
@@ -168,7 +165,8 @@ bool AquaGui::createWindow(const char* title, int width, int height)
 	EventTypeSpec     eventType;                 // Specifier for event type
 	EventHandlerUPP   handlerUPP;                // Pointer to event handler routine
  
-	GNASH_REPORT_FUNCTION;
+        _width = width;
+        _height = height;
 
 	SetRect(&theBounds, 0, 0, width, height);
 	OSStatus status = CreateNewWindow ( kDocumentWindowClass,
@@ -199,7 +197,6 @@ bool AquaGui::createMenu()
 	MenuRef rApplicationMenu;
 	MenuItemIndex outIndex[1];	  
 	
-	GNASH_REPORT_FUNCTION;
 	
 	/* Enable 'Prefereces...' */
 	EnableMenuCommand(NULL, kHICommandPreferences);
@@ -207,14 +204,13 @@ bool AquaGui::createMenu()
 	GetIndMenuItemWithCommandID(NULL, kHICommandPreferences, 1, &rApplicationMenu, outIndex);
 
 	/* Enable 'About' */
-	InsertMenuItemTextWithCFString(rApplicationMenu, CFSTR("About gnash"), (short) 0, 0, kHICommandAbout);
+	InsertMenuItemTextWithCFString(rApplicationMenu, CFSTR("About Gnash"), (short) 0, 0, kHICommandAbout);
 
 	return true;
 }
 
 bool AquaGui::setupEvents()
 {	
-	GNASH_REPORT_FUNCTION;
 
 	return true;
 }
