@@ -19,9 +19,8 @@
 #ifndef AQUA_OGL_GLUE_H
 #define AQUA_OGL_GLUE_H
 
+#include "Renderer_ogl.h"
 #include "aqua_glue.h"
-#include <AGL/agl.h>
-
 
 namespace gnash
 {
@@ -32,8 +31,8 @@ class AquaOglGlue : public AquaGlue
     AquaOglGlue();
     virtual ~AquaOglGlue();
 
-    bool init(int argc, char ***argv);
-    render_handler* createRenderHandler();
+    bool init(int argc, char **argv[]);
+    Renderer* createRenderHandler();
     bool prepDrawingArea(int width, int height, AGLDrawable drawable);
     void setInvalidatedRegions(const InvalidatedRanges& /* ranges */);
     void render();
