@@ -26,7 +26,8 @@
 #include "rc.h" // for use of rcfile
 #include "debugger.h"
 #include "arg_parser.h"
-#include "utility.h" // for clamp
+#include "GnashNumeric.h" // for clamp
+#include "bzrversion.h"
 
 #ifdef HAVE_FFMPEG_AVCODEC_H
 extern "C" {
@@ -47,6 +48,7 @@ extern "C" {
 
 #include <string>
 #include <iostream>
+#include <ios>
 
 #ifdef ENABLE_NLS
 # include <locale>
@@ -171,7 +173,8 @@ build_options()
         << _(" - Media handler: ") << MEDIA_CONFIG << endl
         
         << _("   Configured with: ") << CONFIG_CONFIG << endl
-        << _("   CXXFLAGS: ") << CXXFLAGS << endl;
+        << _("   CXXFLAGS: ") << CXXFLAGS << endl
+        << "   Version: "  << BRANCH_NICK << ":" << BRANCH_REVNO << endl;
 
 #ifdef USE_FFMPEG
     cout << _("Built against ffmpeg version: ") << LIBAVCODEC_IDENT << endl;
@@ -282,7 +285,7 @@ parseCommandLine(int argc, char* argv[], gnash::Player& player)
                     player.hideMenu(true);
                     break;
                 case 's':
-                    player.setScale(gnash::utility::clamp<float>(
+                    player.setScale(gnash::clamp<float>(
                                     parser.argument<float>(i),
                                     0.01f, 100.f));
                     break;
@@ -444,6 +447,8 @@ int
 gnash_main(int argc, char *argv[])
 {
 
+    std::ios::sync_with_stdio(false);
+
     gnash::Player player;
 
     // Enable native language support, i.e. internationalization
