@@ -1,5 +1,5 @@
 //
-//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -21,11 +21,17 @@
 #include "gnash.h"
 
 #include <gtk/gtk.h>
+#if !defined(_WIN32) && !defined(__MACH__)
+#include <gdk/gdkx.h>
+#else
 #include <gdk/gdk.h>
+#endif
 
 namespace gnash
 {
 
+class Renderer;
+
 class GtkGlue
 {
   public:
@@ -34,14 +40,20 @@ class GtkGlue
     virtual bool init(int argc, char **argv[]) = 0;
 
     virtual void prepDrawingArea(GtkWidget *drawing_area) = 0;
-    virtual render_handler* createRenderHandler() = 0;
-    virtual void setRenderHandlerSize(int /*width*/, int /*height*/) { };
+    virtual Renderer* createRenderHandler() = 0;
+    virtual void setRenderHandlerSize(int /*width*/, int /*height*/) {}
     virtual void render() = 0;
+    
     virtual void render(int /*minx*/, int /*miny*/, int /*maxx*/, int /*maxy*/)
-			{ render();	};
+    {
+        render();	
+    }
+
     virtual void configure(GtkWidget *const widget,
-                           GdkEventConfigure *const event) = 0;
-    virtual void beforeRendering() { /* nop */ };
+            GdkEventConfigure *const event) = 0;
+    
+    virtual void beforeRendering() {};
+
   protected:
     GtkWidget *_drawing_area;
 };
