@@ -1,6 +1,6 @@
 // gtk_glue_gtkglext.cpp:  Gnome ToolKit glue of some sort, for Gnash.
 //
-//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -56,7 +56,7 @@ GtkGlExtGlue::init(int argc, char** argv[])
 #ifdef FIX_I810_LOD_BIAS
     int c = getopt (argc, *argv, "m:");
     if (c == 'm') {
-      _tex_lod_bias = (float) atof(optarg);
+      _tex_lod_bias = (float) strtof(optarg, NULL);
     }
 #endif
 
@@ -102,7 +102,7 @@ GtkGlExtGlue::prepDrawingArea(GtkWidget *drawing_area)
                                  NULL, TRUE, GDK_GL_RGBA_TYPE);
 }
 
-render_handler*
+Renderer*
 GtkGlExtGlue::createRenderHandler()
 {
 //    GNASH_REPORT_FUNCTION;
@@ -112,7 +112,7 @@ GtkGlExtGlue::createRenderHandler()
     // Attach our OpenGL context to the drawing_area.
     gdk_gl_drawable_make_current(gldrawable, glcontext);
 
-    render_handler* renderer = create_render_handler_ogl();
+    Renderer* renderer = create_Renderer_ogl();
 
 #ifdef FIX_I810_LOD_BIAS
     glTexEnvf(GL_TEXTURE_FILTER_CONTROL_EXT, GL_TEXTURE_LOD_BIAS_EXT, _tex_lod_bias);
