@@ -1,6 +1,6 @@
 // gui.cpp:  Top level GUI for SWF player, for Gnash.
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -21,14 +21,13 @@
 #include "gnashconfig.h"
 #endif
 
-#include "sprite_instance.h"
+#include "MovieClip.h"
 #include "gui.h"
-#include "render.h"  // debug
-#include "render_handler.h"
+#include "Renderer.h"
 #include "sound_handler.h"
-#include "gnash.h" // for get_sound_handler
 #include "movie_root.h"
 #include "VM.h"
+#include "DisplayObject.h"
 
 #ifdef GNASH_FPS_DEBUG
 #include "ClockTime.h"
@@ -39,10 +38,6 @@
 #include <cstring>
 #include <algorithm> // std::max, std::min
 
-#ifdef SKIP_RENDERING_IF_LATE
-#include "WallClockTimer.h"
-#endif
-
 /// Define this to make sure each frame is fully rendered from ground up
 /// even if no motion has been detected in the movie.
 //#define FORCE_REDRAW 1
@@ -70,17 +65,22 @@
 //
 //#define DEBUG_MOUSE_COORDINATES 1
 
+// Define the following macro if you want to skip rendering
+// when late on FPS time.
+// This is an experimental feature, so it's off by default
+//#define SKIP_RENDERING_IF_LATE 1
+
+
 
 namespace gnash {
 
-Gui::Gui() :
+Gui::Gui(RunResources& r) :
     _loop(true),
     _xid(0),
     _width(1),
     _height(1),
-    _depth(16),
+    _runResources(r),
     _interval(0),
-    _renderer(NULL),
     _redraw_flag(true),
     _fullscreen(false),
     _mouseShown(true),
@@ -95,15 +95,19 @@ Gui::Gui() :
     ,fps_counter_total(0)
     ,fps_timer(0)
     ,fps_timer_interval(0.0)
+    ,frames_dropped(0)
 #endif
-#ifdef SKIP_RENDERING_IF_LATE
-    ,estimatedDisplayTime(0) // milliseconds (will grow later..)
-#endif // SKIP_RENDERING_IF_LATE
     ,_movieDef(0)
     ,_stage(0)
     ,_stopped(false)
     ,_started(false)
     ,_showUpdatedRegions(false)
+
+    // NOTE: it's important that _systemClock is constructed
+    //       before and destroyed after _virtualClock !
+    ,_systemClock()
+    ,_virtualClock(_systemClock)
+
 #ifdef ENABLE_KEYBOARD_MOUSE_MOVEMENTS 
     ,_xpointer(0)
     ,_ypointer(0)
@@ -114,15 +118,14 @@ Gui::Gui() :
 
 }
 
-Gui::Gui(unsigned long xid, float scale, bool loop, unsigned int depth)
+Gui::Gui(unsigned long xid, float scale, bool loop, RunResources& r)
 	:
     _loop(loop),
     _xid(xid),
     _width(1),
     _height(1),
-    _depth(depth),
+    _runResources(r),
     _interval(0),
-    _renderer(NULL),
     _redraw_flag(true),
     _fullscreen(false),
     _mouseShown(true),
@@ -137,15 +140,19 @@ Gui::Gui(unsigned long xid, float scale, bool loop, unsigned int depth)
     ,fps_counter_total(0)    
     ,fps_timer(0)
     ,fps_timer_interval(0.0)
+    ,frames_dropped(0)
 #endif        
-#ifdef SKIP_RENDERING_IF_LATE
-    ,estimatedDisplayTime(0) // milliseconds (will grow later..)
-#endif // SKIP_RENDERING_IF_LATE
     ,_movieDef(0)
     ,_stage(0)
     ,_stopped(false)
     ,_started(false)
     ,_showUpdatedRegions(false)
+
+    // NOTE: it's important that _systemClock is constructed
+    //       before and destroyed after _virtualClock !
+    ,_systemClock()
+    ,_virtualClock(_systemClock)
+
 #ifdef ENABLE_KEYBOARD_MOUSE_MOVEMENTS 
     ,_xpointer(0)
     ,_ypointer(0)
@@ -159,9 +166,11 @@ Gui::~Gui()
 {
     if ( _movieDef.get() ) log_debug("~Gui - _movieDef refcount: %d", _movieDef->get_ref_count());
 
-    delete _renderer;
 #ifdef GNASH_FPS_DEBUG
-    std::cerr << "Total frame advances: " << fps_counter_total << std::endl;
+    if ( fps_timer_interval ) {
+        std::cerr << "Total frame advances/drops: "
+                  << fps_counter_total << "/" << frames_dropped << std::endl;
+    }
 #endif
 }
 
@@ -177,6 +186,12 @@ Gui::unsetFullscreen()
     log_unimpl(_("Fullscreen not yet supported in this GUI"));
 }
 
+void
+Gui::hideMenu()
+{
+    log_unimpl(_("Menu hiding not yet supported in this GUI"));
+}
+
 bool
 Gui::showMouse(bool /* show */)
 {
@@ -206,11 +221,12 @@ Gui::allowScale(bool allow)
 void
 Gui::toggleFullscreen()
 {
+    /// Sends request to Gnash core to change display state.
 	if (_fullscreen) {
-		unsetFullscreen();
+		_stage->setStageDisplayState(movie_root::DISPLAYSTATE_NORMAL);
 	}
 	else {
-		setFullscreen();
+		_stage->setStageDisplayState(movie_root::DISPLAYSTATE_FULLSCREEN);
 	} 
 }
 
@@ -366,16 +382,18 @@ Gui::updateStageMatrix()
 		}
 	}
 
-	//log_debug("updateStageMatrix: scaleMode:%d, valign:%d, halign:%d", scaleMode, valign, halign);
+	//log_debug("updateStageMatrix: scaleMode:%d, valign:%d, halign:%d",
+	//scaleMode, valign, halign);
 
 	// TODO: have a generic set_matrix ?
-	if ( _renderer ) {
+	if (_renderer.get()) {
 		_renderer->set_scale(_xscale, _yscale);
 		_renderer->set_translation(_xoffset, _yoffset);
 	}
 	else
 	{
-		//log_debug("updateStageMatrix: could not signal updated stage matrix to renderer (no renderer registered)");
+		//log_debug("updateStageMatrix: could not signal updated stage
+		//matrix to renderer (no renderer registered)");
 	}
 
 	// trigger redraw
@@ -387,7 +405,6 @@ Gui::updateStageMatrix()
 void
 Gui::resize_view(int width, int height)
 {
-	GNASH_REPORT_FUNCTION;
 
 	assert(width>0);
 	assert(height>0);
@@ -412,48 +429,18 @@ Gui::resize_view(int width, int height)
 }
 
 
-void
-Gui::menu_step_forward()
-{
-	movie_root& m = VM::get().getRoot();
-	m.goto_frame(m.get_current_frame()+1);
-}
-
-void
-Gui::menu_step_backward()
-{
-	movie_root& m = VM::get().getRoot();
-	m.goto_frame(m.get_current_frame()-1);
-}
-
-void
-Gui::menu_jump_forward()
-{
-	movie_root& m = VM::get().getRoot();
-	m.goto_frame(m.get_current_frame()+10);
-}
-
-void
-Gui::menu_jump_backward()
-{
-	movie_root& m = VM::get().getRoot();
-	m.goto_frame(m.get_current_frame()-10);
-}
-
 void
 Gui::toggleSound()
 {
+    assert (_stage);
+    // @todo since we registered the sound handler, shouldn't we know
+    //       already what it is ?!
+    sound::sound_handler* s = _stage->runResources().soundHandler();
 
-    media::sound_handler* s = get_sound_handler();
+    if (!s) return;
 
-    if (!s)
-       return;
-
-    if (s->is_muted()) {
-       s->unmute();
-    } else {
-       s->mute();
-    }
+    if (s->is_muted()) s->unmute();
+    else s->mute();
 }
 
 
@@ -483,7 +470,7 @@ Gui::notify_mouse_moved(int ux, int uy)
 		display(m);
 	}
     
-	character* activeEntity = m->getActiveEntityUnderPointer();
+	DisplayObject* activeEntity = m->getActiveEntityUnderPointer();
 	if ( activeEntity )
 	{
 		if ( activeEntity->isSelectableTextField() )
@@ -546,16 +533,13 @@ Gui::refreshView()
 void
 Gui::notify_key_event(gnash::key::code k, int modifier, bool pressed) 
 {
-	movie_root* m = _stage;
 
 	/* Handle GUI shortcuts */
 	if (pressed)
 	{
-		if (k == gnash::key::ESCAPE)
-		{
-			if (isFullscreen())
-			{
-				unsetFullscreen();
+		if (k == gnash::key::ESCAPE) {
+			if (isFullscreen()) {
+				_stage->setStageDisplayState(movie_root::DISPLAYSTATE_NORMAL);
 			}
 		}
 		
@@ -585,11 +569,9 @@ Gui::notify_key_event(gnash::key::code k, int modifier, bool pressed)
 				case gnash::key::F:
 					toggleFullscreen();
 					break;
-				case gnash::key::RIGHT_BRACKET:
-					menu_step_forward();
-					break;
-				case gnash::key::LEFT_BRACKET:
-					menu_step_backward();
+				case gnash::key::h:
+				case gnash::key::H:
+					showUpdatedRegions(!showUpdatedRegions());
 					break;
 				case gnash::key::MINUS:
 				{
@@ -597,7 +579,7 @@ Gui::notify_key_event(gnash::key::code k, int modifier, bool pressed)
 					unsigned int ni = std::min(_interval+2, 1000u);
 					setInterval(ni);
 					break;
-                                }
+                }
 				case gnash::key::PLUS:
 				{
 					// Min interval allowed: 1/100 second (100FPS)
@@ -609,7 +591,7 @@ Gui::notify_key_event(gnash::key::code k, int modifier, bool pressed)
 				{
 					if ( _stage )
 					{
-						float fps = _stage->get_movie_definition()->get_frame_rate();
+						float fps = _stage->frameRate();
 						// Min interval allowed: 1/100 second (100FPS)
 						unsigned int ni = 1000.0/fps;
 						setInterval(ni);
@@ -679,11 +661,11 @@ Gui::notify_key_event(gnash::key::code k, int modifier, bool pressed)
 
     if ( _stopped ) return;
 
-	if ( m->notify_key_event(k, pressed) )
+	if ( _stage->notify_key_event(k, pressed) )
 	{
 		// any action triggered by the
 		// event required screen refresh
-		display(m);
+		display(_stage);
 	}
 
 }
@@ -729,12 +711,12 @@ Gui::display(movie_root* m)
 		changed_ranges.growBy(40.0f / _xscale);
 		
 		// optimize ranges
-		changed_ranges.combine_ranges();
+		changed_ranges.combineRanges();
 		
 	}
-
-	if (redraw_flag)     // TODO: Remove this and want_redraw to avoid confusion!?
-	{
+    
+    // TODO: Remove this and want_redraw to avoid confusion!?
+	if (redraw_flag)  {
 		changed_ranges.setWorld();
 	}
 	
@@ -749,8 +731,18 @@ Gui::display(movie_root* m)
     // m_child_invalidated flag if at least one of it's childs has the
     // invalidated flag set.
     log_debug("DUMPING CHARACTER TREE"); 
-    m->dump_character_tree();
     
+    InfoTree tr;
+    InfoTree::iterator top = tr.begin();
+    _stage->getMovieInfo(tr, top);
+
+    for (InfoTree::iterator i = tr.begin(), e = tr.end();
+            i != e; ++i) {
+        std::cout << std::string(tr.depth(i) * 2, ' ') << i->first << ": " << 
+            i->second << std::endl;
+    }
+
+
     // less verbose, and often necessary: see the exact coordinates of the
     // invalidated bounds (mainly to see if it's NULL or something else).	
     std::cout << "Calculated changed ranges: " << changed_ranges << "\n";
@@ -789,7 +781,7 @@ Gui::display(movie_root* m)
 		// show invalidated region using a red rectangle
 		// (Flash debug style)
 		IF_DEBUG_REGION_UPDATES (
-		if ( ! changed_ranges.isWorld() )
+		if (_renderer.get() && !changed_ranges.isWorld())
 		{
 		
 			for (size_t rno = 0; rno < changed_ranges.size(); rno++) {
@@ -810,8 +802,8 @@ Gui::display(movie_root* m)
 				corners[2].y = ymax;
 				corners[3].x = xmin;
 				corners[3].y = ymax;
-				matrix no_transform;
-				gnash::render::draw_poly(corners, 4,
+				SWFMatrix no_transform;
+				_renderer->draw_poly(corners, 4,
 					rgba(0,0,0,0), rgba(255,0,0,255), no_transform, false);
 					
 			}
@@ -835,8 +827,14 @@ Gui::play()
     if ( ! _started ) start();
     else
     {
-        media::sound_handler* s = get_sound_handler();
+        assert (_stage);
+        // @todo since we registered the sound handler, shouldn't we know
+        //       already what it is ?!
+        sound::sound_handler* s = _stage->runResources().soundHandler();
         if ( s ) s->unpause();
+
+        log_debug("Starting virtual clock");
+        _virtualClock.resume();
     }
 
     playHook ();
@@ -845,14 +843,22 @@ Gui::play()
 void
 Gui::stop()
 {
+    // _stage must be registered before this is called.
+    assert(_stage);
+
     if ( _stopped ) return;
     if ( isFullscreen() ) unsetFullscreen();
 
     _stopped = true;
 
-    media::sound_handler* s = get_sound_handler();
+    // @todo since we registered the sound handler, shouldn't we know
+    //       already what it is ?!
+    sound::sound_handler* s = _stage->runResources().soundHandler();
     if ( s ) s->pause();
 
+    log_debug("Pausing virtual clock");
+    _virtualClock.pause();
+
     stopHook();
 }
 
@@ -865,9 +871,20 @@ Gui::pause()
     }
     else
     {
-    	media::sound_handler* s = get_sound_handler();
+        // TODO: call stop() instead ?
+        // The only thing I see is that ::stop exits full-screen,
+        // but I'm not sure that's intended behaviour
+
+        // @todo since we registered the sound handler, shouldn't we know
+        //       already what it is ?!
+    	sound::sound_handler* s = _stage->runResources().soundHandler();
     	if ( s ) s->pause();
         _stopped = true;
+
+        log_debug("Pausing virtual clock");
+        _virtualClock.pause();
+
+        stopHook();
     }
 }
 
@@ -881,18 +898,26 @@ Gui::start()
         return;
     }
 
-    std::auto_ptr<movie_instance> mr ( _movieDef->create_movie_instance() );
+    std::auto_ptr<Movie> mr ( _movieDef->createMovie() );
     mr->setVariables(_flashVars);
 
     _stage->setRootMovie( mr.release() ); // will construct the instance
-    resize_view(_width, _height); // to properly update stageMatrix if scaling is given 
 
     bool background = true; // ??
     _stage->set_background_alpha(background ? 1.0f : 0.05f);
 
-    media::sound_handler* s = get_sound_handler();
+    // @todo since we registered the sound handler, shouldn't we know
+    //       already what it is ?!
+    sound::sound_handler* s = _stage->runResources().soundHandler();
     if ( s ) s->unpause();
     _started = true;
+    
+    // to properly update stageMatrix if scaling is given  
+    resize_view(_width, _height); 
+
+    log_debug("Starting virtual clock");
+    _virtualClock.resume();
+
 }
 
 bool
@@ -905,87 +930,66 @@ Gui::advanceMovie()
   
 
 
-#ifdef SKIP_RENDERING_IF_LATE
-	WallClockTimer advanceTimer;
-#endif // SKIP_RENDERING_IF_LATE
-
 	gnash::movie_root* m = _stage;
 	
-#ifdef GNASH_FPS_DEBUG
-	fpsCounterTick(); // will be a no-op if fps_timer_interval is zero
-#endif
-
 // Define REVIEW_ALL_FRAMES to have *all* frames
 // consequencially displaied. Useful for debugging.
 //#define REVIEW_ALL_FRAMES 1
 
 #ifndef REVIEW_ALL_FRAMES
 	// Advance movie by one frame
-	m->advance();
+	bool advanced = m->advance();
 #else
 	size_t cur_frame = m->getRootMovie()->get_current_frame();
 	size_t tot_frames = m->getRootMovie()->get_frame_count();
-	m->advance();
-	m->get_movie_definition()->ensure_frame_loaded(tot_frames);
+	bool advanced = m->advance();
+	m->getRootMovie.ensureFrameLoaded(tot_frames);
 	m->goto_frame(cur_frame+1);
-    	m->set_play_state(gnash::sprite_instance::PLAY);
+    m->set_play_state(gnash::MovieClip::PLAYSTATE_PLAY);
 	log_debug(_("Frame %d"), m->get_current_frame());
 #endif
 
+#ifdef GNASH_FPS_DEBUG
+	if ( advanced ) fpsCounterTick(); // will be a no-op if fps_timer_interval is zero
+#endif
+
+
+
+    // TODO: ask stage about doDisplay ?
+    // - if it didn't advance might need to check updateAfterEvent
+    bool doDisplay = true;
 
 #ifdef SKIP_RENDERING_IF_LATE
-
-	boost::uint32_t advanceTime = advanceTimer.elapsed(); // in milliseconds !
-
-	boost::uint32_t timeSlot = _interval; // milliseconds between advance calls 
-
-	if ( advanceTime+gui->estimatedDisplayTime < timeSlot )
-	{
-		advanceTimer.restart();
-		display(m);
-		boost::uint32_t displayTime = advanceTimer.elapsed();
-
-		if ( displayTime > estimatedDisplayTime)
-		{
-			//log_debug("Display took %6.6g seconds over %6.6g available for each frame", displayTime, timeSlot);
-
-			// Don't update estimatedDisplayTime if it's bigger then timeSlot*0.8
-			if (  displayTime < timeSlot*0.8 )
-			{
-				// TODO: check for absurdly high values, like we can't set
-				//       estimatedDisplayTime to a value higher then FPS, or
-				//       we'll simply never display...
-				estimatedDisplayTime = displayTime;
-			}
-		}
-	}
-	else
-	{
-		log_debug("We're unable to keep up with FPS speed: "
-			"advanceTime was %u + estimatedDisplayTime (%u) "
-			"== %u, over a timeSlot of %u",
-			advanceTime, estimatedDisplayTime,
-			advanceTime+estimatedDisplayTime, timeSlot);
-		// TODO: increment a counter, we don't want to skip too many frames
-	}
-#else // ndef SKIP_RENDERING_IF_LATE
-
-	display(m);
-
+    // We want to skip rendering IFF it's time to advance again.
+    // We'll ask the stage about it
+    if ( _stage->timeToNextFrame() <= 0 )
+    {
+        if ( doDisplay ) // or should it be if advanced ?
+        {
+            // TODO: take note of a frame drop (count them)
+            //log_debug("Frame rendering dropped due to being late");
+#ifdef GNASH_FPS_DEBUG
+            ++frames_dropped;
+#endif
+        }
+        doDisplay = false;
+    }
 #endif // ndef SKIP_RENDERING_IF_LATE
-	
+
+	if (doDisplay) display(m);
+
 	if ( ! loops() )
 	{
 		size_t curframe = m->get_current_frame(); // can be 0 on malformed SWF
-		gnash::sprite_instance* si = m->getRootMovie();
-		if (curframe + 1 >= si->get_frame_count())
+		const gnash::MovieClip& si = m->getRootMovie();
+		if (curframe + 1 >= si.get_frame_count())
 		{
 			quit(); 
 		}
 	}
 
     /// Quit if we've reached the advance limit.
-    if (_maxAdvances && (_advances++ > _maxAdvances))
+    if (_maxAdvances && advanced && (_advances++ > _maxAdvances))
     {
         quit();
     }
@@ -1048,7 +1052,7 @@ Gui::getMovieInfo() const
     InfoTree::iterator topIter = tr->begin();
     InfoTree::iterator firstLevelIter;
 
-    VM& vm = VM::get();
+    VM& vm = _stage->getVM();
 
     std::ostringstream os;
 
@@ -1058,6 +1062,16 @@ Gui::getMovieInfo() const
     os << "SWF " << vm.getSWFVersion();
     topIter = tr->insert(topIter, StringPair("VM version", os.str()));
 
+    // This short-cut is to avoid a bug in movie_root's getMovieInfo,
+    // which relies on the availability of a _rootMovie for doing
+    // it's work, while we don't set it if we didn't start..
+    // 
+    if ( ! _started )
+    {
+        topIter = tr->insert(topIter, StringPair("Stage properties", 
+                    "not constructed yet"));
+        return tr;
+    }
 
     movie_root& stage = vm.getRoot();
     stage.getMovieInfo(*tr, topIter);
@@ -1067,7 +1081,7 @@ Gui::getMovieInfo() const
     //
     topIter = tr->insert(topIter, StringPair("Mouse Entities", ""));
 
-    const character* ch;
+    const DisplayObject* ch;
     ch = stage.getActiveEntityUnderPointer();
     if ( ch )
     {
@@ -1185,9 +1199,11 @@ Gui::fpsCounterTick()
     //log_debug("Effective frame rate: %0.2f fps", (float)(fps_counter/secs));
     std::cerr << boost::format("Effective frame rate: %0.2f fps "
                                "(min %0.2f, avg %0.2f, max %0.2f, "
-                               "%u frames in %0.1f secs total)") % rate %
+                               "%u frames in %0.1f secs total, "
+                               "dropped %u)") % rate %
                                fps_rate_min % avg % fps_rate_max %
-                               fps_counter_total % secs_total << std::endl;
+                               fps_counter_total % secs_total %
+                               frames_dropped << std::endl;
       
     fps_counter = 0;
     fps_timer = current_timer;
@@ -1219,7 +1235,6 @@ Gui::setStage(movie_root* stage)
     assert(stage);
     assert(!_stage);
     _stage = stage;
-    _stage->setGui(this);
 }
 
 bool
@@ -1229,5 +1244,27 @@ Gui::yesno(const std::string& question)
     return true;
 }
 
+void
+Gui::setQuality(Quality q)
+{
+	if ( ! _stage )
+	{
+		log_error("Gui::setQuality called before a movie_root was available");
+		return;
+	}
+    _stage->setQuality(q);
+}
+
+Quality
+Gui::getQuality() const
+{
+	if ( ! _stage )
+	{
+		log_error("Gui::getQuality called before a movie_root was available");
+		return QUALITY_HIGH; // just a guess..
+	}
+    return _stage->getQuality();
+}
+
 // end of namespace
 }
