@@ -25,7 +25,11 @@
 #include "rect.h"  // for composition
 #include "snappingrange.h"  // for InvalidatedRanges
 #include "GnashKey.h" // for gnash::key::code type
+#include "Renderer.h" // for gnash::key::code type
 #include "smart_ptr.h"
+#include "VirtualClock.h"
+#include "SystemClock.h"
+#include "gnash.h" // for Quality
 
 #ifdef USE_SWFTREE
 #include "tree.hh" // for tree
@@ -39,12 +43,6 @@
 // gnashconfig.h
 //#define GNASH_FPS_DEBUG
 
-// Define the following macro if you want to skip rendering
-// when late on FPS time.
-// This is an experimental feature, so it's off by default
-//#define SKIP_RENDERING_IF_LATE
-
-
 /// Define this to disable region updates debugging altogether. If undefined,
 /// debugging will be a runtime option. The flag and flag-setting functions
 /// will not be disabled (too ugly).
@@ -64,7 +62,7 @@
 // Forward declarations
 namespace gnash
 {
-    class render_handler;
+    class RunResources;
     class movie_root;
     class movie_definition;
 }
@@ -101,6 +99,13 @@ public:
       _interval = interval;
     }
 
+    /// Return the clock provided by this Gui.
+    //
+    /// The Gui clock will be paused when the gui is put
+    /// in pause mode and resumed when gui playback is resumed.
+    ///
+    VirtualClock& getClock() { return _virtualClock; }
+
     /// Set the time in milliseconds after which the programme should exit.
     virtual void setTimeout(unsigned int timeout) = 0;
 
@@ -319,6 +324,12 @@ public:
     /// state.
     void restart();
 
+    /// Set rendering quality, if not locked by RC file..
+    void setQuality(Quality q);
+
+    /// Get current rendering quality
+    Quality getQuality() const;
+
     /// Toggle sound state between muted and unmuted. If
     /// there is no active sound handler this does nothing.
     void toggleSound();
@@ -380,7 +391,7 @@ public:
 protected:
 
     /// Default constructor. Initialises members to safe defaults.
-    Gui();
+    Gui(RunResources& r);
 
     /** \brief
      * Expanded constructor for more control over member values.
@@ -396,7 +407,7 @@ protected:
      *
      * @param depth Colour depth to be used in the client area of our window.
      */
-    Gui(unsigned long xid, float scale, bool loop, unsigned int depth);
+    Gui(unsigned long xid, float scale, bool loop, RunResources& r);
 
     /// Determines if playback should restart after the movie ends.
     bool            _loop;
@@ -417,13 +428,13 @@ protected:
     int             _height;
 
     /// Desired colour depth in bits.
-    int             _depth;
+    RunResources& _runResources;
 
     /// Main loop interval: the time between successive advance_movie calls.
     unsigned int    _interval;
 
     /// The handler which is called to update the client area of our window.
-    render_handler* _renderer;
+    boost::shared_ptr<Renderer> _renderer;
 
     /// Signals that the next frame must be re-rendered completely because the
     /// window size did change.
@@ -466,6 +477,7 @@ private:
     
 #ifdef GNASH_FPS_DEBUG
     unsigned int fps_counter;
+
     float fps_rate_min, fps_rate_max;   
 
     // Number of calls to fpsCounterTick, which is also
@@ -480,6 +492,9 @@ private:
     ///
     float fps_timer_interval;
     
+    /// Number of frames rendering of which was dropped
+    unsigned int frames_dropped;
+
     /// \brief
     /// Should be called on every frame advance (including inter-frames caused
     /// by mouse events).
@@ -490,12 +505,6 @@ private:
 
 #endif // def GNASH_FPS_DEBUG
 
-#ifdef SKIP_RENDERING_IF_LATE
-    /// Estimated max number of milliseconds required for a call to ::display
-    /// This should be incremented everytime we take more
-    boost::uint32_t estimatedDisplayTime;
-#endif // SKIP_RENDERING_IF_LATE
-
     VariableMap _flashVars;
 
     boost::intrusive_ptr<movie_definition> _movieDef;
@@ -511,6 +520,9 @@ private:
 
     /// If true, updated regions (invalidated ranges) are visibly outlined.
     bool _showUpdatedRegions;
+
+    SystemClock _systemClock;
+    InterruptableVirtualClock _virtualClock;
     
 #ifdef ENABLE_KEYBOARD_MOUSE_MOVEMENTS 
 	int _xpointer;
@@ -522,15 +534,16 @@ private:
 };
 
 /// Named constructors
-std::auto_ptr<Gui> createGTKGui(unsigned long xid, float scale, bool loop, unsigned int depth);
-std::auto_ptr<Gui> createKDEGui(unsigned long xid, float scale, bool loop, unsigned int depth);
-std::auto_ptr<Gui> createKDE4Gui(unsigned long xid, float scale, bool loop, unsigned int depth);
-std::auto_ptr<Gui> createSDLGui(unsigned long xid, float scale, bool loop, unsigned int depth);
-std::auto_ptr<Gui> createFLTKGui(unsigned long xid, float scale, bool loop, unsigned int depth);
-std::auto_ptr<Gui> createFBGui(unsigned long xid, float scale, bool loop, unsigned int depth);
-std::auto_ptr<Gui> createAQUAGui(unsigned long xid, float scale, bool loop, unsigned int depth);
-std::auto_ptr<Gui> createRISCOSGui(unsigned long xid, float scale, bool loop, unsigned int depth);
-std::auto_ptr<Gui> createDumpGui(unsigned long xid, float scale, bool loop, unsigned int depth);
+std::auto_ptr<Gui> createGTKGui(unsigned long xid, float scale, bool loop, RunResources& r);
+std::auto_ptr<Gui> createKDEGui(unsigned long xid, float scale, bool loop, RunResources& r);
+std::auto_ptr<Gui> createKDE4Gui(unsigned long xid, float scale, bool loop, RunResources& r);
+std::auto_ptr<Gui> createSDLGui(unsigned long xid, float scale, bool loop, RunResources& r);
+std::auto_ptr<Gui> createFLTKGui(unsigned long xid, float scale, bool loop, RunResources& r);
+std::auto_ptr<Gui> createFBGui(unsigned long xid, float scale, bool loop, RunResources& r);
+std::auto_ptr<Gui> createAQUAGui(unsigned long xid, float scale, bool loop, RunResources& r);
+std::auto_ptr<Gui> createRISCOSGui(unsigned long xid, float scale, bool loop, RunResources& r);
+std::auto_ptr<Gui> createAOS4Gui(unsigned long xid, float scale, bool loop, RunResources& r);
+std::auto_ptr<Gui> createDumpGui(unsigned long xid, float scale, bool loop, RunResources& r);
 
  
 } // end of gnash namespace
