@@ -1,6 +1,6 @@
 // gui_gtk.cpp: Wrapper for Gnome ToolKit graphical user interface, for Gnash.
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -31,12 +31,12 @@
 namespace gnash {
 
 #ifdef GUI_DUMP
-std::auto_ptr<Gui> createDumpGui(unsigned long windowid, float scale, bool do_loop, unsigned int bit_depth)
+std::auto_ptr<Gui> createDumpGui(unsigned long windowid, float scale, bool do_loop, RunResources& r)
 {
-	return std::auto_ptr<Gui>(new DumpGui(windowid, scale, do_loop, bit_depth));
+	return std::auto_ptr<Gui>(new DumpGui(windowid, scale, do_loop, r));
 }
 #else // ! GUI_DUMP
-std::auto_ptr<Gui> createDumpGui(unsigned long , float , bool , unsigned int )
+std::auto_ptr<Gui> createDumpGui(unsigned long , float, bool, RunResourcesfloat , bool , unsigned int )
 {
 	throw GnashException("Support for DUMP gui was not compiled in");
 }
