@@ -17,6 +17,7 @@
 
 #include <qapplication.h>
 #include "kde_glue.h"
+#include "Renderer_ogl.h"
 #include <qgl.h>
 
 
@@ -31,7 +32,7 @@ class KdeOpenGLGlue : public KdeGlue
     
     bool init(int argc, char **argv[]);
     void prepDrawingArea(QWidget *drawing_area);
-    render_handler* createRenderHandler();
+    Renderer* createRenderHandler();
     void render();
   private:
 #ifdef FIX_I810_LOD_BIAS
