@@ -1,9 +1,9 @@
 // 
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
 // 
 // This program is distributed in the hope that it will be useful,
@@ -15,19 +15,14 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-// 
-//
-
-/* $Id: sdlsup.h,v 1.21 2007/05/28 15:41:00 ann Exp $ */
 
 #ifndef __SDLSUP_H__
 #define __SDLSUP_H__
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include "gnashconfig.h"
 #endif
 
-#include "tu_config.h"
 #include "gui.h"
 
 #include "SDL.h"
@@ -49,10 +44,10 @@
 namespace gnash
 {
 
-class DSOEXPORT SDLGui : public Gui
+class SDLGui : public Gui
 {
 public:
-    SDLGui(unsigned long xid, float scale, bool loop, unsigned int depth);
+    SDLGui(unsigned long xid, float scale, bool loop, RunResources& r);
     virtual ~SDLGui();
     virtual bool init(int argc, char **argv[]);
     virtual bool createWindow(const char *title, int width, int height);
@@ -64,11 +59,14 @@ public:
     virtual void disableCoreTrap();
     virtual void setTimeout(unsigned int timeout);
 
-		void key_event(SDLKey key, bool down);
+    void key_event(SDL_KeyboardEvent * key, bool down);
+
+    // See gui.h for documentation
+    void setInvalidatedRegions(const InvalidatedRanges& ranges);
 
 private:
-    unsigned int	_timeout;
-    bool 		_core_trap;
+    unsigned int _timeout;
+    bool         _core_trap;
 
     /// Handle VIDEORESIZE event
     void resize_event();
@@ -76,21 +74,24 @@ private:
     /// Handle VIDEOEXPOSE event
     void expose_event();
 
+    static key::code sdl_to_gnash_key(SDL_KeyboardEvent* key);
+    static int sdl_to_gnash_modifier(int state);
+
 #ifdef RENDERER_AGG
     SdlAggGlue          _glue;
 #elif defined(RENDERER_CAIRO)
-    SdlCairoGlue	_glue;
+    SdlCairoGlue    _glue;
 #elif defined(RENDERER_OPENGL)
-    SdlOglGlue		_glue;
+    SdlOglGlue      _glue;
 # ifdef FIX_I810_LOD_BIAS
-    float 		_tex_lod_bias;
+    float       _tex_lod_bias;
 # endif
 #endif
 
 };
  
 // void xt_event_handler(Widget xtwidget, gpointer instance,
-// 		 XEvent *xevent, Boolean *b);
+//       XEvent *xevent, Boolean *b);
 
 // end of namespace gnash 
 }
