@@ -40,28 +40,41 @@
 
 // Macro to prevent repeated logging calls for the same
 // event
-#define LOG_ONCE(x) { static bool warned = false; if (!warned) { warned = true; x; } }
+#define LOG_ONCE(x) { \
+    static bool warned = false; \
+    if (!warned) { warned = true; x; } \
+}
 
-// Define to switch between printf-style log formatting
-// and boost::format
 # include <boost/preprocessor/arithmetic/inc.hpp>
 # include <boost/preprocessor/repetition/enum_params.hpp>
 # include <boost/preprocessor/repetition/repeat.hpp>
 # include <boost/preprocessor/repetition/repeat_from_to.hpp>
 # include <boost/preprocessor/seq/for_each.hpp>
 
+// Mingw32 (win32 console) doesn't use the standard GCC defines that
+// Gnash used for debug messages, so make it so...
+#ifndef __FUNCDNAME__
+#define __FUNCDNAME__ __FUNCTION__
+#endif
+
 namespace gnash {
 
-#define GNASH_DEBUG_LEVEL 2
-
 // This is a basic file logging class
-class DSOEXPORT LogFile {
+class DSOEXPORT LogFile
+{
 public:
 
     static LogFile& getDefaultInstance();
 
     ~LogFile();
 
+    enum LogLevel {
+        LOG_SILENT,
+        LOG_NORMAL,
+        LOG_DEBUG,
+        LOG_EXTRA
+    };
+
     enum FileState {
         CLOSED,
         OPEN,
@@ -107,31 +120,39 @@ public:
     void setLogFilename(const std::string& fname);
 
     // accessors for the verbose level
-    void setVerbosity () {
-        _verbose++;
+    void setVerbosity() {
+        ++_verbose;
     }
 
-    void setVerbosity (int x) {
+    void setVerbosity(int x) {
         _verbose = x;
     }
 
-    int getVerbosity () {
+    int getVerbosity() const {
         return _verbose;
     }
     
-    void setActionDump (int x) {
+    void setActionDump(int x) {
         _actiondump = x;
     }
 
-    int getActionDump () {
+    void setNetwork(int x) {
+        _network = x;
+    }
+
+    int getActionDump() const {
         return _actiondump;
     }
     
+    int getNetwork() const {
+        return _network;
+    }
+    
     void setParserDump (int x) {
         _parserdump = x;
     }
 
-    int getParserDump () {
+    int getParserDump() const {
         return _parserdump;
     }
     
@@ -139,14 +160,14 @@ public:
         _stamp = b;
     }
 
-    bool getStamp () {
+    bool getStamp() const {
         return _stamp;
     }
 
     /// Set whether to write logs to file
-    void setWriteDisk (bool b);
+    void setWriteDisk(bool b);
 
-    bool getWriteDisk () {
+    bool getWriteDisk() const {
         return _write;
     }
     
@@ -183,24 +204,27 @@ private:
     boost::mutex _ioMutex;
 
     /// Stream to write to stdout.
-    std::ofstream	 _outstream;
+    std::ofstream _outstream;
 
     /// How much output is required: 2 or more gives debug output.
-    int		 _verbose;
+    int _verbose;
 
     /// Whether to dump all SWF actions
-    bool		 _actiondump;
+    bool _actiondump;
+
+    /// Whether to dump all networking actions
+    bool _network;
 
     /// Whether to dump parser output
-    bool		 _parserdump;
+    bool _parserdump;
 
     /// The state of the log file.
     FileState _state;
 
-    bool		 _stamp;
+    bool _stamp;
 
     /// Whether to write the log file to disk.
-    bool		 _write;
+    bool _write;
 
     std::string _filespec;
 
@@ -210,6 +234,19 @@ private:
 
 };
 
+DSOEXPORT void processLog_network(const boost::format& fmt);
+DSOEXPORT void processLog_error(const boost::format& fmt);
+DSOEXPORT void processLog_unimpl(const boost::format& fmt);
+DSOEXPORT void processLog_trace(const boost::format& fmt);
+DSOEXPORT void processLog_debug(const boost::format& fmt);
+DSOEXPORT void processLog_action(const boost::format& fmt);
+DSOEXPORT void processLog_parse(const boost::format& fmt);
+DSOEXPORT void processLog_security(const boost::format& fmt);
+DSOEXPORT void processLog_swferror(const boost::format& fmt);
+DSOEXPORT void processLog_amferror(const boost::format& fmt);
+DSOEXPORT void processLog_aserror(const boost::format& fmt);
+DSOEXPORT void processLog_abc(const boost::format& fmt);
+
 /// This heap of steaming preprocessor code magically converts
 /// printf-style statements into boost::format messages using templates.
 //
@@ -225,44 +262,47 @@ private:
 /// This is a sequence of different log message types to be used in
 /// the code. Append the name to log_ to call the function, e.g. 
 /// log_error, log_unimpl.
-#define LOG_TYPES (error) (debug) (unimpl) (aserror) (swferror) (amferror) (security) (action) (parse) (trace)
+#define LOG_TYPES (error) (debug) (unimpl) (aserror) (swferror) \
+    (amferror) (security) (action) (parse) (trace) (abc) (network)
 
 /// This actually creates the template functions using the TOKENIZE
 /// functions above. The templates look like this:
 //
-/// template< typename T0 , typename T1 , typename T2 , typename T3 > 
-/// void
-/// log_security (const T0& t0, const T1& t1, const T2& t2, const T3& t3)
+/// template<typename T0 , typename T1 , typename T2>
+/// void log_error(const T0& t0 , const T1& t1 , const T2& t2)
 /// {
-///     if (_verbosity == 0) return;
-///     processLog_security(myFormat(t0) % t1 % t2 % t3);
+///     if (LogFile::getDefaultInstance().getVerbosity() == 0) return;
+///     boost::format f(t0);
+///     using namespace boost::io;
+///     f.exceptions(all_error_bits ^ (too_many_args_bit |
+///                                    too_few_args_bit |
+///                                    bad_format_string_bit));
+///     processLog_error(f % t1 % t2);
 /// }
-//
+///
 /// Only not as nicely indented.
 ///
-/// Use "g++ -E log.h" or "gcc log.h" to check.
+/// Use "g++ -E log.h" or "cpp log.h" to check.
 #define LOG_TEMPLATES(z, n, data)\
-    template< \
-         BOOST_PP_ENUM_PARAMS(\
-         BOOST_PP_INC(n), typename T)\
-     >\
-    void log_##data (\
-        BOOST_PP_REPEAT(\
-        BOOST_PP_INC(n), \
-        TOKENIZE_ARGS, t)\
-    ) { \
+template<BOOST_PP_ENUM_PARAMS(BOOST_PP_INC(n), typename T)>\
+void log_##data(BOOST_PP_REPEAT(BOOST_PP_INC(n), TOKENIZE_ARGS, t)) \
+{\
     if (LogFile::getDefaultInstance().getVerbosity() == 0) return; \
-    processLog_##data(logFormat(t0) \
-    BOOST_PP_REPEAT_FROM_TO(1, \
-        BOOST_PP_INC(n), \
-        TOKENIZE_FORMAT, t));\
-    }\
+    boost::format f(t0); \
+    using namespace boost::io; \
+    f.exceptions(all_error_bits ^ (too_many_args_bit | \
+                                   too_few_args_bit | \
+                                   bad_format_string_bit)); \
+    processLog_##data(f BOOST_PP_REPEAT_FROM_TO(1, \
+            BOOST_PP_INC(n), \
+            TOKENIZE_FORMAT, t));\
+}
 
 /// Defines the maximum number of template arguments
 //
 /// The preprocessor generates templates with 1..ARG_NUMBER
 /// arguments.
-#define ARG_NUMBER 16
+#define ARG_NUMBER 10
 
 /// Calls the macro LOG_TEMPLATES an ARG_NUMBER number
 /// of times, each time adding an extra typename argument to the
@@ -279,24 +319,6 @@ BOOST_PP_SEQ_FOR_EACH(GENERATE_LOG_TYPES, _, LOG_TYPES)
 #undef LOG_TEMPLATES
 #undef ARG_NUMBER
 
-DSOEXPORT void processLog_error(const boost::format& fmt);
-DSOEXPORT void processLog_unimpl(const boost::format& fmt);
-DSOEXPORT void processLog_trace(const boost::format& fmt);
-DSOEXPORT void processLog_debug(const boost::format& fmt);
-DSOEXPORT void processLog_action(const boost::format& fmt);
-DSOEXPORT void processLog_parse(const boost::format& fmt);
-DSOEXPORT void processLog_security(const boost::format& fmt);
-DSOEXPORT void processLog_swferror(const boost::format& fmt);
-DSOEXPORT void processLog_amferror(const boost::format& fmt);
-DSOEXPORT void processLog_aserror(const boost::format& fmt);
-
-/// A fault-tolerant boost::format object for logging
-//
-/// Generally to be used in the LogFile macro BF(), which will also
-/// be recognized by gettext for internationalization and is less
-/// effort to type.
-DSOEXPORT boost::format logFormat(const std::string &str);
-
 /// Convert a sequence of bytes to hex or ascii format.
 //
 /// @param bytes    the array of bytes to process
@@ -304,7 +326,8 @@ DSOEXPORT boost::format logFormat(const std::string &str);
 ///                 for checking that length does not exceed the array size.
 /// @param ascii    whether to return in ascii or space-separated hex format.
 /// @return         a string representation of the byte sequence.
-DSOEXPORT std::string hexify(const unsigned char *bytes, size_t length, bool ascii);
+DSOEXPORT std::string hexify(const unsigned char *bytes, size_t length,
+        bool ascii);
 
 // Define to 0 to completely remove parse debugging at compile-time
 #ifndef VERBOSE_PARSE
@@ -331,6 +354,10 @@ DSOEXPORT std::string hexify(const unsigned char *bytes, size_t length, bool asc
 #define VERBOSE_MALFORMED_AMF 1
 #endif
 
+// Define to 0 this to remove Networking verbosity at compile-time
+#ifndef VERBOSE_NETWORKING
+#define VERBOSE_NETWORKING 1
+#endif
 
 #if VERBOSE_PARSE
 #define IF_VERBOSE_PARSE(x) do { if ( LogFile::getDefaultInstance().getParserDump() ) { x; } } while (0);
@@ -344,6 +371,12 @@ DSOEXPORT std::string hexify(const unsigned char *bytes, size_t length, bool asc
 #define IF_VERBOSE_ACTION(x)
 #endif
 
+#if VERBOSE_ACTION
+#define IF_VERBOSE_NETWORK(x) do { if ( LogFile::getDefaultInstance().getNetwork() ) { x; } } while (0);
+#else
+#define IF_VERBOSE_NETWORK(x)
+#endif
+
 #if VERBOSE_ASCODING_ERRORS
 // TODO: check if it's worth to check verbosity level too...
 #define IF_VERBOSE_ASCODING_ERRORS(x) { if ( gnash::RcInitFile::getDefaultInstance().showASCodingErrors() ) { x; } }
@@ -386,7 +419,7 @@ public:
     }
 
     ~__Host_Function_Report__(void) {
-	if (LogFile::getDefaultInstance().getVerbosity() >= GNASH_DEBUG_LEVEL + 1) {
+	if (LogFile::getDefaultInstance().getVerbosity() > LogFile::LOG_DEBUG) {
 	    log_debug("%s returning", func);
 	}
     }
