@@ -33,21 +33,33 @@
 #include <cstdlib>
 
 #if defined(WIN32) || defined(_WIN32)
-#define LIBLTDL_DLL_IMPORT 1
+# define LIBLTDL_DLL_IMPORT 1
 #endif
-#include <ltdl.h>
 #ifdef HAVE_DLFCN_H
-	#include <dlfcn.h>
+# include <dlfcn.h>
 #endif
 #ifdef HAVE_LIBGEN_H
-	#include <libgen.h>
+# include <libgen.h>
 #endif
+
+#include "ltdl.h"
 #include <boost/thread/mutex.hpp>
 
-#if defined(_WIN32) || defined(WIN32)
-#	define PLUGINSDIR "./"
+#if defined(WIN32) || defined(_WIN32)
+int        lt_dlsetsearchpath   (const char *search_path);
+int        lt_dlinit           (void);
+void *     lt_dlsym            (lt_dlhandle handle, const char *name);
+const char *lt_dlerror         (void);
+int        lt_dlclose          (lt_dlhandle handle);
+int        lt_dlmakeresident   (lt_dlhandle handle);
+lt_dlhandle lt_dlopenext       (const char *filename);
 #endif
 
+#if defined(_WIN32) || defined(WIN32)
+# define PLUGINSDIR "./"
+#endif
+
+using namespace std;
 
 namespace gnash {
 
@@ -62,7 +74,12 @@ SharedLib::SharedLib()
 
 SharedLib::SharedLib(const std::string& filespec)
 {
-//    GNASH_REPORT_FUNCTION;
+    GNASH_REPORT_FUNCTION;
+}
+
+SharedLib::SharedLib(const std::string &filespec, const std::string &envvar)
+{
+    GNASH_REPORT_FUNCTION;
 #ifdef LT_DLMUTEX
 //     return lt_dlmutex_register (gnash_mutex_lock, gnash_mutex_unlock,
 //                                 gnash_mutex_seterror, gnash_mutex_geterror);
@@ -77,13 +94,16 @@ SharedLib::SharedLib(const std::string& filespec)
 //     } else {
 //         log_debug ("Initialized ltdl");
     }
-    std::string pluginsdir = PLUGINSDIR;
     
-    char *env = std::getenv ("GNASH_PLUGINS");
-    if (env) pluginsdir = env;
-   
-
-    lt_dlsetsearchpath(pluginsdir.c_str());
+    string pluginsdir;
+    char *env = std::getenv (envvar.c_str());
+    if (env) {
+        pluginsdir = env;
+    } else {
+        pluginsdir = PLUGINSDIR;
+    }
+    
+    // lt_dladdsearchdir(pluginsdir.c_str());
 }
 
 SharedLib::~SharedLib()
@@ -148,7 +168,7 @@ SharedLib::openLib (const std::string& filespec)
 SharedLib::initentry *
 SharedLib::getInitEntry (const std::string& symbol)
 {
-//    GNASH_REPORT_FUNCTION;
+    GNASH_REPORT_FUNCTION;
     lt_ptr run = NULL;
     
     scoped_lock lock(_libMutex);
