@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -18,28 +18,28 @@
 #ifndef GNASH_PROPERTYLIST_H
 #define GNASH_PROPERTYLIST_H
 
-#include "dsodefs.h" // DSOEXPORT
-
 #include "Property.h" // for templated functions
-#include "as_prop_flags.h" // for templated functions
+#include "PropFlags.h" // for templated functions
 #include "as_value.h" // for templated functions
 #include "string_table.h"
 
 #include <map> 
 #include <string> // for use within map 
 #include <cassert> // for inlines
-#include <cctype> // for toupper
 #include <utility> // for std::pair
-#include <set> // for propNameSet
+#include <set>
+#include <boost/cstdint.hpp> 
 #include <boost/multi_index_container.hpp>
 #include <boost/multi_index/ordered_index.hpp>
 #include <boost/multi_index/key_extractors.hpp>
 
+
 // Forward declaration
 namespace gnash {
 	class as_object;
 	class as_environment;
 	class as_function;
+	class ObjectURI;
 }
 
 namespace gnash {
@@ -53,14 +53,19 @@ namespace gnash {
 class PropertyList
 {
 public:
-	/// A tag type for multi-index
-	struct oType {/**/};
+
+    typedef std::pair<std::string, std::string> KeyValuePair;
+    typedef std::vector<KeyValuePair> SortedPropertyList;
+    
+
+    /// A tag type for multi-index
+	struct oType {};
 
 	/// The actual container
 	/// index 0 is the fully indexed name/namespace pairs, which are unique
-	/// Because of the way searching works, this index can also be used to search
-	/// for the names alone (composite keys are sorted lexographically, beginning
-	/// with the first element specified)
+	/// Because of the way searching works, this index can also be
+    /// used to search for the names alone (composite keys are sorted
+    /// lexographically, beginning with the first element specified)
 	///
 	/// index 1 is an ordered sequence, and it is used for the AS3 style
 	/// enumeration (which requires an order number for each property),
@@ -83,11 +88,17 @@ public:
 	> container;
 
 	/// Construct the PropertyList 
-	PropertyList()
+    //
+    /// The constructor takes a VM reference because PropertyList
+    /// conceptually needs access to Virtual Machine resources
+    /// (string_table) but not to the Stage.
+	PropertyList(VM& vm)
 		:
 		_props(),
-		mDefaultOrder(0) // correct ?
-	{/**/}
+		mDefaultOrder(0),
+        _vm(vm)
+	{
+    }
 
 	/// Copy constructor
 	PropertyList(const PropertyList& pl);
@@ -104,7 +115,7 @@ public:
 	/// @param visitor
 	///	The visitor function. Must take a string_table::key 
 	///	reference as first argument and a const as_value reference
-	///	as second argument.
+	///	as second argument. Scan by enumeration order.
 	///
 	/// @param this_ptr
 	///	The object reference used to extract values from properties.
@@ -112,8 +123,14 @@ public:
 	template <class V>
 	void visitValues(V& visitor, const as_object& this_ptr) const
 	{
-		for (container::const_iterator it = _props.begin(),
-			itEnd = _props.end(); it != itEnd; ++it)
+        typedef container::nth_index<1>::type ContainerByOrder;
+        
+        // The template keyword is not required by the Standard here, but the
+        // OpenBSD compiler needs it. Use of the template keyword where it is
+        // not necessary is not an error.
+        for (ContainerByOrder::const_reverse_iterator
+                it = _props.template get<1>().rbegin(),
+                ie = _props.template get<1>().rend(); it != ie; ++it)
 		{
 			as_value val = it->getValue(this_ptr);
 			visitor.accept(it->mName, val);
@@ -124,7 +141,7 @@ public:
 	//
 	/// The method will invoke the given visitor method
 	/// passing it two arguments: name of the property and
-	/// value of it.
+	/// value of it. Scan by enumeration order.
 	///
 	/// @param visitor
 	///	The visitor function. Must take a string_table::key 
@@ -137,8 +154,14 @@ public:
 	template <class V>
 	void visitNonHiddenValues(V& visitor, const as_object& this_ptr) const
 	{
-		for (container::const_iterator it = _props.begin(),
-			itEnd = _props.end(); it != itEnd; ++it)
+        typedef container::nth_index<1>::type ContainerByOrder;
+        
+        // The template keyword is not required by the Standard here, but the
+        // OpenBSD compiler needs it. Use of the template keyword where it is
+        // not necessary is not an error.
+        for (ContainerByOrder::const_reverse_iterator it =
+                _props.template get<1>().rbegin(),
+                ie = _props.template get<1>().rend(); it != ie; ++it)
 		{
 	        if (it->getFlags().get_dont_enum()) continue;
 			as_value val = it->getValue(this_ptr);
@@ -146,6 +169,19 @@ public:
 		}
 	}
 
+    /// Is any non-hidden property in this list ?
+    bool hasNonHiddenProperties() const
+    {
+        typedef container::nth_index<1>::type ContainerByOrder;
+        for (ContainerByOrder::const_reverse_iterator it = 
+                _props.get<1>().rbegin(), ie=_props.get<1>().rend();
+                it != ie; ++it)
+		{
+	        if (! it->getFlags().get_dont_enum()) return true;
+        }
+        return false;
+    }
+
 
 	/// Get the as_value value of a named property
 	//
@@ -233,7 +269,7 @@ public:
 	///
 	bool setValue(string_table::key key, const as_value& value,
 			as_object& this_ptr, string_table::key namespaceId = 0,
-			const as_prop_flags& flagsIfMissing=0);
+			const PropFlags& flagsIfMissing=0);
 
 	/// Reserves a slot number for a property
 	///
@@ -248,8 +284,7 @@ public:
 	/// The namespace in which the property should be found.
 	///
 	/// @return true if the slot did not previously exist.
-	bool reserveSlot(unsigned short slotId, string_table::key key,
-		string_table::key nsId = 0);
+	bool reserveSlot(const ObjectURI& uri, boost::uint16_t slotId);
 
 	/// Get a property, if existing
 	//
@@ -319,7 +354,7 @@ public:
 	///
 	bool addGetterSetter(string_table::key key, as_function& getter,
 		as_function* setter, const as_value& cacheVal,
-		const as_prop_flags& flagsIfMissing=0, string_table::key ns=0);
+		const PropFlags& flagsIfMissing=0, string_table::key ns=0);
 
 	/// \brief
 	/// Add a getter/setter property, if not already existing
@@ -340,7 +375,7 @@ public:
 	///         otherwise (property already existent?)
 	///
 	bool addGetterSetter(string_table::key key, as_c_function_ptr getter,
-		as_c_function_ptr setter, const as_prop_flags& flagsIfMissing,
+		as_c_function_ptr setter, const PropFlags& flagsIfMissing,
 		string_table::key ns = 0);
 
 	/// \brief
@@ -360,7 +395,7 @@ public:
 	///
 	bool addDestructiveGetter(string_table::key key,
 		as_function& getter, string_table::key ns = 0,
-		const as_prop_flags& flagsIfMissing=0);
+		const PropFlags& flagsIfMissing=0);
 
 	/// \brief
 	/// Add a destructive getter property, if not already existant.
@@ -379,7 +414,7 @@ public:
 	///
 	bool addDestructiveGetter(string_table::key key,
 		as_c_function_ptr getter, string_table::key ns = 0,
-		const as_prop_flags& flagsIfMissing=0);
+		const PropFlags& flagsIfMissing=0);
 
 	/// Set the flags of a property.
 	//
@@ -431,8 +466,7 @@ public:
 	///	    of keys in the props argument not found in
 	///	    this properties set.
 	///
-	std::pair<size_t,size_t> setFlagsAll(
-			const PropertyList& props,
+	std::pair<size_t,size_t> setFlagsAll( const PropertyList& props,
 			int setTrue, int setFalse);
 
 	/// \brief
@@ -453,6 +487,7 @@ public:
 	/// \brief
 	/// Enumerate all non-hidden properties pushing
 	/// their keys to the given as_environment.
+    /// Follows enumeration order.
 	///
 	/// @param donelist
 	/// Don't enumerate those in donelist. Add those done to donelist.
@@ -461,14 +496,15 @@ public:
 
 	/// \brief
 	/// Enumerate all non-hidden properties inserting
-	/// their name/value pair to the given map
+	/// their name/value pair to the given SortedPropertyList.
+    /// Follows enumeration order.
 	///
 	/// @param this_ptr
 	/// 	The as_object used to set the 'this' pointer
 	/// 	for calling getter/setter function (GetterSetterProperty);
 	/// 	it will be unused when getting or setting SimpleProperty
 	/// 	properties.
-	void enumerateKeyValue(const as_object& this_ptr, std::map<std::string, std::string>& to) const;
+	void enumerateKeyValue(const as_object& this_ptr, SortedPropertyList& to) const;
 
 	/// Remove all entries in the container
 	void clear();
@@ -492,6 +528,9 @@ public:
 	///	Note that the PropertyList itself might be changed
 	///	from this call, accessed trough the 'this' pointer,
 	///	so this method too is non-const.
+    ///
+    /// This does not reflect the normal enumeration order. It is sorted
+    /// lexicographically by property.
 	///
 	void dump(as_object& this_ptr);
 
@@ -508,16 +547,24 @@ public:
 	///	Note that the PropertyList itself might be changed
 	///	from this call, accessed trough the 'this' pointer,
 	///	so this method too is non-const.
+    ///
+    /// This does not reflect the normal enumeration order. It is sorted
+    /// lexicographically by property.
 	///
 	void dump(as_object& this_ptr, std::map<std::string, as_value>& to);
 
 	/// Mark all simple properties, getters and setters
 	/// as being reachable (for the GC)
-	void DSOEXPORT setReachable() const;
+	void setReachable() const;
 
 private:
-	container _props;
-	unsigned short mDefaultOrder;
+
+    container _props;
+
+    unsigned short mDefaultOrder;
+    
+    VM& _vm;
+
 };
 
 
