@@ -18,28 +18,32 @@
 #ifndef GNASH_TEXTFIELD_H
 #define GNASH_TEXTFIELD_H
 
-#include "character.h" // for inheritance
+#include "InteractiveObject.h" // for inheritance
 #include "styles.h" // for line_style
-#include "fill_style.h"
 #include "Range2d.h"
 #include "rect.h" // for inlines
 #include "Font.h" // for visibility of font add_ref/drop_ref
 
+#include <vector>
+
 // Forward declarations
 namespace gnash {
     namespace SWF {
         class DefineEditTextTag;
         class TextRecord;
     }
+	class TextFormat_as;
 }
 
 namespace gnash {
 
 /// An instance of a DefineEditTextTag 
-class TextField : public character
+class TextField : public InteractiveObject
 {
 
 public:
+    
+    typedef std::vector<size_t> LineStarts;
 
     /// Text alignment values
 	enum TextAlignment
@@ -50,6 +54,13 @@ public:
 		ALIGN_JUSTIFY
 	};
 
+	/// Text format display values
+	enum TextFormatDisplay
+	{
+		TEXTFORMAT_BLOCK = 0,
+		TEXTFORMAT_INLINE = 1
+	};
+	
 	/// Possible autoSize values
 	enum AutoSizeValue {
 
@@ -80,26 +91,54 @@ public:
 		typeInput
 	};
 
+	static void htmlptag() {
+
+	}
+	
+	typedef void dotag ();
+    typedef std::map<std::string,dotag*> HTMLTags;
+	
+	HTMLTags _htmltags;
+
+	void maptags()
+	{
+		_htmltags["P"] = htmlptag;
+		//_htmltags.insert(std::make_pair("P",ptag*()));
+	}
+	
+
     /// Constructs a TextField as specified in a DefineEditText tag.
-	TextField(character* parent, const SWF::DefineEditTextTag& def, int id);
+	TextField(DisplayObject* parent, const SWF::DefineEditTextTag& def, int id);
 
     /// Constructs a TextField with default values and the specified bounds.
     //
     /// Notably, the default textHeight is 12pt (240 twips).
-    TextField(character* parent, const rect& bounds);
+	/// @param parent A pointer to the DisplayObject parent of this TextField
+	/// @param bounds A rect specifying the bounds of this TextField
+    TextField(DisplayObject* parent, const rect& bounds);
 
 	~TextField();
 
 	// TODO: should this return isSelectable() ?
-	bool can_handle_mouse_event() const { return true; }
+	/// Returns true for now, TextField is always "Mouse-Enabled"
+	bool mouseEnabled() const { return true; }
 
-	character* get_topmost_mouse_entity(boost::int32_t x, boost::int32_t y);
+	/// Returns a pointer to the topmost InteractiveObject at (x,y)
+	//
+	/// @param x x-coordinate
+	/// @param y y-coordinate
+	InteractiveObject* topmostMouseEntity(boost::int32_t x,
+            boost::int32_t y);
 
-	bool wantsInstanceName() const
+	// Text fields need to handle cxform specially 
+	virtual cxform get_world_cxform() const;
+	
+    bool wantsInstanceName() const
 	{
 		return true; // text fields can be referenced 
 	}	
-		
+
+	/// This function is called as a user-input handler
 	bool on_event(const event_id& id);	
 
 	const std::string& getVariableName() const
@@ -123,9 +162,14 @@ public:
     /// updated.
 	void updateText(const std::string& s);
 
+	void updateHtmlText(const std::string& s);
+
  	/// Return value of our text.
 	std::string get_text_value() const;
 
+	/// Return value of our htmlText.
+	std::string get_htmltext_value() const;
+
 	/// Return true if text is defined
 	bool getTextDefined() const { return _textDefined; }
 
@@ -133,14 +177,17 @@ public:
         return m_cursor;
     }
 
+	/// /brief get a std::pair of size_t with start/end of selection
     const std::pair<size_t, size_t>& getSelection() const {
         return _selection;
     }
 
-    /// Replace the current selection with the new text.
+    /// /brief Replace the current selection with the new text.
+	//
+	/// @param replace String to replace the current selection
     void replaceSelection(const std::string& replace);
 
-    /// Set the current selection
+    /// /brief Set the current selection
     //
     /// @param start    The index of the beginning of the selection.
     /// @param end      The index of the end of the selection.
@@ -157,55 +204,65 @@ public:
 		string_table::key nsname = 0);
 
 	/// Draw the dynamic string.
-	void	display();
+	void display(Renderer& renderer);
 
 	void add_invalidated_bounds(InvalidatedRanges& ranges, bool force);
 
-	rect getBounds() const
+	/// \brief Get bounding rect of this TextField
+	virtual rect getBounds() const
 	{
 		return _bounds;
 	}
 
-	// See dox in character.h
+	// See dox in DisplayObject.h
 	bool pointInShape(boost::int32_t x, boost::int32_t y) const;
 
 	/// Return true if the 'background' should be drawn
 	bool getDrawBackground() const;
 
-	/// Specify wheter to draw the background
+	/// \brief Specify wheter to draw the background
+	//
+	/// @param draw If true the background of this TextField will be drawn
 	void setDrawBackground(bool draw);
 
-	/// Return color of the background
+	/// \brief Return color of the background
 	rgba getBackgroundColor() const;
 
-	/// Set color of the background
+	/// \brief Set color of the background
 	//
 	/// Use setDrawBackground to actually use this value.
-	///
+	/// @param col RGBA Object with color information. TextField
+	/// 	background will be drawn in this color
 	void setBackgroundColor(const rgba& col);
 
-	/// Return true if this TextField should have it's border visible
+	/// \brief Return true if this TextField should have it's border visible
 	bool getDrawBorder() const;
 
-	/// Specify wheter to draw the border
+	/// \brief Specify whether to draw the border
+	//
+	/// @param draw If true the border of this TextField will be drawn
 	void setDrawBorder(bool draw);
 
-	/// Return color of the border
+	/// \brief Return color of the border
 	rgba getBorderColor() const;
 
-	/// Set color of the border
+	/// \brief Set color of the border
 	//
 	/// Use setDrawBorder to actually use this value.
-	///
+	/// @param col RGBA Object with color information. TextField border
+	/// 	will be drawn in this color.
 	void setBorderColor(const rgba& col);
 
-	/// Return color of the text
+	/// \brief Return color of the text
 	const rgba& getTextColor() const 
 	{
 		return _textColor;
 	}
 
-	/// Set color of the text
+	/// \brief Set color of the text
+	//
+	/// @param col RGBA Object with color information. Text in this TextField
+	/// 	will be displayed in this color.
 	void setTextColor(const rgba& col);
 
 	/// \brief
@@ -215,32 +272,41 @@ public:
 		return _embedFonts;
 	}
 
-    /// Get the current maxChars setting of the TextField
+    /// \brief Get the current maxChars setting of the TextField
     boost::int32_t maxChars() const {
         return _maxChars;
     }
 
-    /// Set the current maxChars setting of the TextField
+    /// \brief Set the current maxChars setting of the TextField
+	//
+	/// @param max The maximum number of characters that can be
+	/// 	input by the user (Does not restrict Scripts)
     void maxChars(boost::int32_t max) {
         _maxChars = max;
     }
 
-    /// Get the current multiline setting of the TextField
+    /// \brief Get the current multiline setting of the TextField
     bool multiline() const {
         return _multiline;
     }
 
-    /// Set the current multiline setting of the TextField
+    /// \brief Set the current multiline setting of the TextField
+	//
+	/// @param b If true "Enter" key will be recognized (Does not
+	/// 	restrict Scripts)
     void multiline(bool b) {
         _multiline = b;
     }
-
-    /// Get the current password setting of the TextField
+	
+    /// \brief Get the current password setting of the TextField
     bool password() const {
         return _password;
     }
 
-    /// Set the current password setting of the TextField
+    /// \brief Set the current password setting of the TextField
+	//
+	/// @param b If true characters in the TextField will be displayed
+	/// 	as (*)
     void password(bool b) {
         _password = b;
     }
@@ -251,23 +317,23 @@ public:
 	/// @param use
 	void setEmbedFonts(bool use);
 
-	/// Get autoSize value 
+	/// \brief Get autoSize value 
 	AutoSizeValue getAutoSize() const
 	{
 		return _autoSize;
 	}
 
-	/// Return text TextAlignment
+	/// \brief Return text TextAlignment
     TextAlignment getTextAlignment();
 
-	/// Set autoSize value 
+	/// \brief Set autoSize value 
 	//
 	/// @param val
 	/// 	The AutoSizeValue to use
 	///
 	void setAutoSize(AutoSizeValue val);
 
-	/// Parse autoSize string value
+	/// \brief Parse autoSize string value
 	//
 	/// @param val
 	/// 	Auto size value as a string (one of none, left, center, right)
@@ -276,7 +342,7 @@ public:
 	///
 	static AutoSizeValue parseAutoSizeValue(const std::string& val);
 
-	/// Return autoSize value as a string
+	/// \brief Return autoSize value as a string
 	//
 	/// @param val
 	/// 	Auto size value 
@@ -286,20 +352,20 @@ public:
 	///
 	static const char* autoSizeValueName(AutoSizeValue val);
 
-	/// Set type (input or dynamic)
+	/// \brief Set type (input or dynamic)
 	//
 	/// @param val
 	/// 	The TypeValue to use, no-op if typeInvalid.
 	///
 	void setType(TypeValue val) { if (val != typeInvalid) _type=val; }
 
-	/// Get type (input, dynamic or invalid)
+	/// \brief Get type (input, dynamic or invalid)
 	TypeValue getType() const
 	{
 		return _type;
 	}
 
-	/// Return true if this TextField is read-only
+	/// \brief Return true if this TextField is read-only
 	bool isReadOnly() const { return _type != typeInput; }
 
 	/// Parse type string value
@@ -329,7 +395,7 @@ public:
 		return _wordWrap;
 	}
 
-	/// Set wordWrap parameter 
+	/// \brief Set wordWrap parameter 
 	//
 	/// @param on
 	///	If true text hitting bounding box limits will continue
@@ -346,7 +412,7 @@ public:
 		return _html;
 	}
 
-	/// Set html parameter
+	/// \brief Set html parameter
 	//
 	/// @param on
 	///	If true HTML tags in the text will be parsed and rendered
@@ -354,19 +420,23 @@ public:
 		_html = on;
 	}
 
-	/// Return true if the TextField text is selectable
+	/// \brief Return true if the TextField text is selectable
 	bool isSelectable() const
 	{
 		return _selectable;
 	}
 
 	/// Set 'selectable' parameter
+	//
+	/// @param v
+	///	If true text in this TextField will be selectable
 	void setSelectable(bool v) 
 	{
 		_selectable = v;
 	}
 
-	// See character::isActiveTextField
+	// See DisplayObject::isActiveTextField
+	/// Return true if the TextField text is selectable
 	virtual bool isSelectableTextField() const
 	{
 		return isSelectable();
@@ -392,6 +462,7 @@ public:
 
 	const Font* getFont() { return _font.get(); }
 
+	
 	boost::uint16_t getFontHeight() const
 	{
 		return _fontHeight;
@@ -434,19 +505,101 @@ public:
 
 	void setAlignment(TextAlignment h);
 
-	boost::uint16_t getLeading() const
+	boost::int16_t getLeading() const
 	{
 		return _leading;
 	}
 
-	void setLeading(boost::uint16_t h);
+	void setLeading(boost::int16_t h);
 
 	bool getUnderlined() const
 	{
 		return _underlined;
 	}
+	
+	TextFormatDisplay getDisplay() const
+	{ 
+		return _display;
+	}
+	
+	bool getBullet() const
+	{
+		return _bullet;
+	}
+	
+	const std::vector<int>& getTabStops() const
+	{
+		return _tabStops;
+	}
+
+	bool isRestrict() const
+	{
+		return _restrictDefined;
+	}
+	
+	const std::string& getRestrict() const
+	{
+		return _restrict;
+	}
+
+	size_t getScroll() const
+	{
+		return _scroll;
+	}
+
+	size_t getMaxScroll() const
+	{
+		return _maxScroll;
+	}
+
+	size_t getHScroll() const
+	{
+		return _hScroll;
+	}
+
+	size_t getMaxHScroll() const
+	{
+		return _maxHScroll;
+	}
+
+	size_t getBottomScroll() const
+	{
+		return _bottomScroll;
+	}
 
 	void setUnderlined(bool v);
+	void setTabStops(const std::vector<int>& tabStops);
+	void setBullet(bool b);
+	void setURL(std::string url);
+	void setTarget(std::string target);
+	void setRestrict(const std::string& restrict);
+	void setDisplay(TextFormatDisplay display);
+	void setScroll(size_t scroll) {
+		_scroll = scroll;
+		format_text();
+	}
+	void setMaxScroll(size_t maxScroll) {
+		_maxScroll = maxScroll;
+		format_text();
+	}
+	void setHScroll(size_t hScroll) {
+		_hScroll = hScroll;
+		format_text();
+	}
+	void setMaxHScroll(size_t maxHScroll) {
+		_maxHScroll = maxHScroll;
+		format_text();
+	}
+	void setbottomScroll(size_t bottomScroll) {
+		_bottomScroll = bottomScroll;
+		format_text();
+	}
+
+	/// Returns the number of the record that the cursor is in
+	//
+	size_t cursorRecord();
+
+	void setTextFormat(TextFormat_as& tf);
 
 	const rect& getTextBoundingBox() const
 	{
@@ -459,16 +612,25 @@ public:
 	///
 	void setTextValue(const std::wstring& wstr);
 
+	/// Set our htmlText to the given string.
+	//
+	void setHtmlTextValue(const std::wstring& wstr);
+
+protected:
+
+	/// Mark reachable reosurces (for GC)
+	//
+	/// Reachable resources are:
+	///  - The font being used (m_font) 
+	///  - Our definition
+	///  - Common DisplayObject resources
+	///
+	void markReachableResources() const;
 
 private:
 
     void init();
 
-    /// The immutable definition of our TextField
-    //
-    /// This is NULL for dynamic TextFields.
-    boost::intrusive_ptr<const SWF::DefineEditTextTag> _tag;
-
 	/// \brief Set our text to the given string by effect of an update of a
     /// registered variable name
 	//
@@ -477,6 +639,8 @@ private:
     /// updated.
 	void updateText(const std::wstring& s);
 
+	void updateHtmlText(const std::wstring& s);
+
     void insertTab(SWF::TextRecord& rec, boost::int32_t& x, float scale);
 
 	/// What happens when setFocus() is called on this TextField.
@@ -491,52 +655,47 @@ private:
 	/// Call this function when willing to invoke the onChanged event handler
 	void onChanged();
 
-	/// The actual text.
-    //
-    /// Because we have to deal with non-ascii characters (129-255), this
-    /// is a wide string; the cursor position and the position within the
-    /// string are then the same, which makes manipulating the string much
-    /// easier.
-	std::wstring _text;
-
-	/// This flag will be true as soon as the TextField
-	/// is assigned a text value. Only way to be false is
-	/// when definition has the hasText flag set to false
-	/// and no actionscript added text.
-	bool _textDefined;
-
-	/// bounds of dynamic text, as laid out
-	rect m_text_bounding_box;
-
 	/// Reset our text bounding box to the given point.
 	void reset_bounding_box(boost::int32_t x, boost::int32_t y)
 	{
 		m_text_bounding_box.set_to_point(x, y);
 	}
 
-	typedef std::vector<SWF::TextRecord> TextRecords;
-	TextRecords _textRecords;
-
-	/// used to pass a color on to shape_character::display()
-	std::vector<fill_style>	m_dummy_style;
-
-	std::vector<line_style>	m_dummy_line_style;
-
-	/// Convert the characters in _text into a series of
+	/// Convert the DisplayObjects in _text into a series of
 	/// text_glyph_records to be rendered.
 	void format_text();
 	
+	/// Move viewable lines based on m_cursor
+	void scrollLines();
+	
+	/// Handles a new line, this will be called several times, so this
+	/// will hopefully make code cleaner
+	void newLine(boost::int32_t& x, boost::int32_t& y, 
+				 SWF::TextRecord& rec, int& last_space_glyph,
+				 LineStarts::value_type& last_line_start_record, float div);
+					
+	/// De-reference and do appropriate action for character iterator
+	void handleChar(std::wstring::const_iterator& it,
+            const std::wstring::const_iterator& e, boost::int32_t& x,
+            boost::int32_t& y, SWF::TextRecord& rec, int& last_code,
+		    int& last_space_glyph,
+            LineStarts::value_type& last_line_start_record);
+	
 	/// Extracts an HTML tag.
 	///
 	/// @param tag  This string is filled with the extracted HTML tag.
-	/// @param it   An iterator pointing to the first character of the
-	///             HTML tag. It is left pointing to the character after the
+	/// @param attributes This is a map of attribute names and values
+	/// @param it   An iterator pointing to the first DisplayObject of the
+	///             HTML tag. It is left pointing to the DisplayObject after the
 	///             closing tag or the end of the string.
 	/// @param e    An iterator pointing to the end of the string.
 	/// @return     Whether the tag is complete or not (i.e. whether a '>'
 	///             was found).
-	bool parseHTML(std::wstring& tag, std::wstring::const_iterator& it,
-	                      const std::wstring::const_iterator& e) const;
+	bool parseHTML(std::wstring& tag,
+            std::map<std::string, std::string>& attributes,
+            std::wstring::const_iterator& it,
+            const std::wstring::const_iterator& e,
+			bool& selfclosing) const;
 
 	/// Does LEFT/CENTER/RIGHT alignment on the records in
 	/// m_text_glyph_records[], starting with
@@ -544,42 +703,7 @@ private:
 	/// m_text_glyph_records.
 	float align_line(TextAlignment align, int last_line_start_record, float x);
 
-	bool _underlined;
-
-	boost::uint16_t _leading;
-
-	TextAlignment _alignment;
-
-	boost::uint16_t _indent;
-
-	/// Indentation for every line (including the ones created by
-	/// effect of a word-wrap.
-	boost::uint16_t _blockIndent;
-
-	boost::uint16_t _leftMargin;
-
-	boost::uint16_t _rightMargin;
-
-	boost::uint16_t _fontHeight;
-
-	boost::intrusive_ptr<const Font> _font;
-
-	bool m_has_focus;
-	size_t m_cursor;
-	void show_cursor(const SWFMatrix& mat);
-	float m_xcursor;
-	float m_ycursor;
-
-    /// Corresponds to the multiline property.
-    bool _multiline;
-
-    /// Corresponds to the password property.
-    bool _password;
-
-    /// Corresponds to the maxChars property.
-    boost::int32_t _maxChars;
-
-	/// Associate a variable to the text of this character
+	/// Associate a variable to the text of this DisplayObject
 	//
 	/// Setting the associated variable actually changes the
 	/// displayed text. Getting the variable would return the
@@ -589,7 +713,7 @@ private:
 	/// current text before overriding it.
 	///
 	/// Since the variable target may be undefined at time
-	/// of instantiation of this EditText character, the
+	/// of instantiation of this EditText DisplayObject, the
 	/// class keeps track of wheter it succeeded registering
 	/// the variable and this function will do nothing in this
 	/// case. Thus it is safe to call it multiple time, using
@@ -605,10 +729,97 @@ private:
 	/// into sprite and a string_table::key components
 	///
 	VariableRef parseTextVariableRef(const std::string& variableName) const;
-	
-	// Text fields need to handle cxform specially 
-	cxform	get_world_cxform() const;
 
+    /// The immutable definition of our TextField
+    //
+    /// This is NULL for dynamic TextFields.
+    boost::intrusive_ptr<const SWF::DefineEditTextTag> _tag;
+
+	/// The actual text.
+    //
+    /// Because we have to deal with non-ascii DisplayObjects (129-255), this
+    /// is a wide string; the cursor position and the position within the
+    /// string are then the same, which makes manipulating the string much
+    /// easier.
+	std::wstring _text;
+
+	/// The html representation of our text
+	//
+	std::wstring _htmlText;
+
+	/// This flag will be true as soon as the TextField
+	/// is assigned a text value. Only way to be false is
+	/// when definition has the hasText flag set to false
+	/// and no actionscript added text.
+	bool _textDefined;
+
+	bool _htmlTextDefined;
+
+	bool _restrictDefined;
+
+	/// bounds of dynamic text, as laid out
+	rect m_text_bounding_box;
+
+	typedef std::vector<SWF::TextRecord> TextRecords;
+	TextRecords _textRecords;
+	std::vector<size_t> _recordStarts;
+	TextRecords _displayRecords;
+	bool _underlined;
+	bool _bullet;
+
+	std::string _url;
+	std::string _target;
+	std::string _restrict;
+	std::set<wchar_t> _restrictedchars;
+	TextFormatDisplay _display;
+	std::vector<int> _tabStops;
+	
+	boost::int32_t _numChars;
+	boost::int16_t _leading;
+
+	TextAlignment _alignment;
+
+	boost::uint16_t _indent;
+
+	/// Indentation for every line (including the ones created by
+	/// effect of a word-wrap.
+	boost::uint16_t _blockIndent;
+
+	boost::uint16_t _leftMargin;
+
+	boost::uint16_t _rightMargin;
+
+	boost::uint16_t _fontHeight;
+	
+	boost::intrusive_ptr<const Font> _font;
+
+	bool m_has_focus;
+	size_t m_cursor;
+	size_t _glyphcount;
+	size_t _scroll;
+	size_t _maxScroll;
+	size_t _hScroll;
+	size_t _maxHScroll;
+	size_t _bottomScroll;
+	/// Called in display(), this sets the cursor using m_cursor and _textRecords
+	//
+	/// @param renderer
+	/// @param mat
+	void show_cursor(Renderer& renderer, const SWFMatrix& mat);
+
+	LineStarts _line_starts;
+
+	size_t _linesindisplay;
+
+    /// Corresponds to the multiline property.
+    bool _multiline;
+
+    /// Corresponds to the password property.
+    bool _password;
+
+    /// Corresponds to the maxChars property.
+    size_t _maxChars;
+	
 	/// The flag keeping status of TextVariable registration
 	//
 	/// It will be set to true if there's no need to register
@@ -656,21 +867,13 @@ private:
     /// Represents the selected part of the text. The second element must
     /// never be less than the first.
     std::pair<size_t, size_t> _selection;
-
-protected:
-
-	/// Mark reachable reosurces (for GC)
-	//
-	/// Reachable resources are:
-	///  - The font being used (m_font) 
-	///  - Our definition
-	///  - Common character resources
-	///
-	void markReachableResources() const;
+	
 };
 
 /// Initialize the global TextField class
-void textfield_class_init(as_object& global);
+void textfield_class_init(as_object& global, const ObjectURI& uri);
+
+void registerTextFieldNative(as_object& global);
 
 } // namespace gnash
 
