@@ -20,7 +20,7 @@
 #include "smart_ptr.h" // GNASH_USE_GC
 #include "as_environment.h"
 #include "MovieClip.h"
-#include "shape_character_def.h"
+#include "movie_root.h"
 #include "as_value.h"
 #include "VM.h"
 #include "log.h"
@@ -29,6 +29,7 @@
 #include "namedStrings.h"
 #include "as_function.h" 
 #include "CallStack.h"
+#include "Global_as.h"
 
 #include <string>
 #include <utility> // for std::pair
@@ -89,7 +90,7 @@ as_environment::get_variable(const std::string& varname,
             IF_VERBOSE_ASCODING_ERRORS(
             log_aserror(_("find_object(\"%s\") [ varname = '%s' - "
                         "current target = '%s' ] failed"),
-                        path, varname, m_target ? m_target->get_text_value() : "<null>");
+                        path, varname, m_target);
             as_value tmp = get_variable_raw(path, scopeStack, retTarget);
             if ( ! tmp.is_undefined() )
             {
@@ -103,17 +104,18 @@ as_environment::get_variable(const std::string& varname,
     else
     {
     // TODO: have this checked by parse_path as an optimization 
-    if ( varname.find_first_of('/') != std::string::npos && varname.find_first_of(':') == std::string::npos )
-    {
-        // Consider it all a path ...
-            as_object* target = find_object(varname, &scopeStack); 
-        if ( target ) 
+    if (varname.find('/') != std::string::npos &&
+            varname.find(':') == std::string::npos)
         {
-            // ... but only if it resolves to a sprite
-            MovieClip* m = target->to_movie();
-            if ( m ) return as_value(m);
+            // Consider it all a path ...
+                as_object* target = find_object(varname, &scopeStack); 
+            if ( target ) 
+            {
+                // ... but only if it resolves to a sprite
+                MovieClip* m = target->to_movie();
+                if ( m ) return as_value(m);
+            }
         }
-    }
         return get_variable_raw(varname, scopeStack, retTarget);
     }
 }
@@ -525,7 +527,7 @@ as_environment::parse_path(const std::string& var_path, as_object** target,
     return true;
 }
 
-// Search for next '.' or '/' character in this word.  Return
+// Search for next '.' or '/' DisplayObject in this word.  Return
 // a pointer to it, or to NULL if it wasn't found.
 static const char*
 next_slash_or_dot(const char* word)
@@ -545,11 +547,11 @@ next_slash_or_dot(const char* word)
 //
 // Supports both /slash/syntax and dot.syntax
 //
-character*
+DisplayObject*
 as_environment::find_target(const std::string& path_in) const
 {
     as_object* o = find_object(path_in);
-    if ( o ) return o->to_character(); // can be NULL (not a character)...
+    if ( o ) return o->toDisplayObject(); // can be NULL (not a DisplayObject)...
     else return NULL;
 }
 
@@ -584,13 +586,13 @@ as_environment::find_object(const std::string& path,
     {
         // Absolute path.  Start at the (AS) root (handle _lockroot)
         MovieClip* root = 0;
-        if ( m_target ) root = const_cast<MovieClip*>(m_target->getAsRoot());
+        if ( m_target ) root = m_target->getAsRoot();
         else {
             if ( _original_target )
             {
                 log_debug("current target is undefined on "
                         "as_environment::find_object, we'll use original");
-                root = const_cast<MovieClip*>(_original_target->getAsRoot());
+                root = _original_target->getAsRoot();
             }
             else
             {
@@ -637,6 +639,7 @@ as_environment::find_object(const std::string& path,
 #ifdef DEBUG_TARGET_FINDING 
             log_debug(_("Path is %s, returning whatever we were up to"), path);
 #endif
+
             return env;
         }
 
@@ -750,7 +753,7 @@ as_environment::find_object(const std::string& path,
 
 #ifdef DEBUG_TARGET_FINDING 
             log_debug(_("Invoking get_path_element(%s) on object "
-                    "%p (%s)"), subpart, (void *)env, env->get_text_value());
+                    "%p"), subpart, (void *)env);
 #endif
 
             as_object* element = env->get_path_element(subpartKey);
@@ -939,17 +942,13 @@ as_environment::pushCallFrame(as_function* func)
     // TODO: override from gnashrc.
     
     // A stack size of 0 is apparently legitimate.
-    const boost::uint16_t maxstacksize = 
-        func->getVM().getRoot().getRecursionLimit();
+    const boost::uint16_t recursionLimit = getRoot(*func).getRecursionLimit();
+
+    // Don't proceed if local call frames would reach the recursion limit.
+    if (_localFrames.size() + 1 >= recursionLimit) {
 
-    // Doesn't proceed if the stack size would reach the limit; should
-    // this check be done somewhere after adding to the stack? Would
-    // it make any difference?
-    if ( _localFrames.size() + 1 >= maxstacksize )
-    {
         std::ostringstream ss;
-        ss << boost::format(_("Max stack count reached (%u)")) % 
-            _localFrames.size();
+        ss << boost::format(_("Recursion limit reached (%u)")) % recursionLimit;
 
         // throw something
         throw ActionLimitException(ss.str()); 
@@ -967,7 +966,7 @@ as_environment::popCallFrame()
 }
     
 void
-as_environment::set_target(character* target)
+as_environment::set_target(DisplayObject* target)
 {
     //assert(target);
     if ( ! _original_target )
@@ -1078,6 +1077,30 @@ as_environment::markReachableResources() const
 }
 #endif // GNASH_USE_GC
 
+string_table&
+getStringTable(const as_environment& env)
+{
+    return env.getVM().getStringTable();
+}
+
+movie_root&
+getRoot(const as_environment& env)
+{
+    return env.getVM().getRoot();
+}
+
+Global_as*
+getGlobal(const as_environment& env)
+{
+    return env.getVM().getGlobal();
+}
+
+int
+getSWFVersion(const as_environment& env)
+{
+    return env.getVM().getSWFVersion();
+}
+
 } // end of gnash namespace
 
 
