@@ -20,11 +20,12 @@
 
 #include "smart_ptr.h" // GNASH_USE_GC
 #include "as_object.h" // for inheritance
+#include "fn_call.h"
 
 // Forward declarations
 namespace gnash {
-	class fn_call;
-	class builtin_function;
+	class NativeFunction;
+	class Global_as;
 }
 
 namespace gnash {
@@ -33,11 +34,11 @@ namespace gnash {
 //
 /// In ActionScript, every Function is also a class.
 /// The *exported interface* of the class is defined
-/// as an 'prototype' member of the function object.
+/// as a 'prototype' member of the function object.
 ///
 /// Any instance of the class defined by this function will
 /// inherit any member of the class 'prototype'.
-/// To have an object inherit from a class you can set it's
+/// To have an object inherit from a class you can set its
 /// __proto__ member so to point to the class prototype, ie:
 ///
 ///   function MyClass() {}
@@ -56,9 +57,6 @@ namespace gnash {
 /// This class will automatically setup the 'prototype' member
 /// if not explicitly provided (ie: will set 'constructor' so
 /// that it points to the instance).
-/// 
-///
-///
 class as_function : public as_object
 {
 public:
@@ -70,7 +68,7 @@ public:
 	as_function* to_function() { return this; }
 
 	/// Dispatch.
-	virtual as_value operator()(const fn_call& fn)=0;
+	virtual as_value operator()(const fn_call& fn) = 0;
 
 	/// Alias for operator()
 	as_value call(const fn_call& fn) { return operator()(fn); }
@@ -88,8 +86,8 @@ public:
 	/// @param args
 	///	Arguments for the constructor invocation
 	///
-	boost::intrusive_ptr<as_object> constructInstance( as_environment& env,
-			std::auto_ptr< std::vector<as_value> > args);
+	boost::intrusive_ptr<as_object> constructInstance(const as_environment& env,
+			fn_call::Args& args);
 
 	/// Get this function's "prototype" member (exported interface).
 	///
@@ -99,17 +97,10 @@ public:
 	void extends(as_function& superclass);
 
 	/// Return true if this is a built-in class.
-	virtual bool isBuiltin()  { return false; }
-
-	/// TODO: check if a user-defined 'toString'
-	///       will be used when available.
-	std::string get_text_value() const
-	{
-		return "[type Function]";
-	}
+	virtual bool isBuiltin() { return false; }
 
 	/// Return the built-in Function constructor
-	static boost::intrusive_ptr<builtin_function> getFunctionConstructor();
+	static NativeFunction* getFunctionConstructor();
 
 #ifdef GNASH_USE_GC
 	/// Mark reachable resources. Override from GcResource
@@ -128,6 +119,23 @@ public:
 #endif // GNASH_USE_GC
 
 protected:
+	
+    /// Construct a function with no interface
+	//
+	/// For SWF>5 the function object will have derive from Function.
+	///
+	as_function(Global_as& gl);
+
+	/// Construct a function with given interface (possibly none)
+	//
+	/// @param iface
+	///	The interface exported by this class (its 'prototype' member).
+	///	If NULL, no prototype will be set (this is used for some
+	///	corner cases like TextField in SWF5 or below).
+	///	If not NULL, a 'constructor' member will be added to the
+	///	prototype, pointing to 'this'.
+	///
+	as_function(Global_as& gl, as_object* iface);
 
 #ifdef GNASH_USE_GC
 	/// Mark prototype (properties) as being reachable and invoke
@@ -144,30 +152,6 @@ protected:
 	}
 #endif // GNASH_USE_GC
 
-	/// Construct a function with no interface
-	//
-	/// For SWF>5 the function object will have derive from Function.
-	///
-	as_function();
-
-	/// Construct a function with given interface (possibly none)
-	//
-	/// @param iface
-	///	The interface exported by this class (its 'prototype' member).
-	///	If NULL, no prototype will be set (this is used for some
-	///	corner cases like TextField in SWF5 or below).
-	///	If not NULL, a 'constructor' member will be added to the
-	///	prototype, pointing to 'this'.
-	///
-	as_function(as_object* iface);
-
-	/// The "prototype" member.
-	//
-	/// Used for class constructor and members
-	/// to be inherited by instances of this
-	/// "Function" (class)
-	///
-	//boost::intrusive_ptr<as_object>	_properties;
 
 private:
 
@@ -175,15 +159,10 @@ private:
 };
 
 /// Initialize the global Function constructor
-void function_class_init(as_object& global);
+void function_class_init(as_object& global, const ObjectURI& uri);
+void registerFunctionNative(as_object& global);
 
-// To be made statics instead
-as_value function_apply(const fn_call& fn);
-as_value function_call(const fn_call& fn);
+} // gnash namespace
 
-
-} // end of gnash namespace
-
-// _GNASH_AS_FUNCTION_H_
 #endif
 
