@@ -25,15 +25,19 @@
 #include "smart_ptr.h" // GNASH_USE_GC
 #include "as_object.h"
 #include "as_function.h"
-#include "as_environment.h" // for enumerateProperties
-#include "Property.h" // for findGetterSetter
+#include "as_environment.h" 
+#include "movie_root.h" 
+#include "event_id.h" 
+#include "Property.h"
 #include "VM.h"
 #include "GnashException.h"
-#include "fn_call.h" // for generic methods
-#include "Object.h" // for getObjectInterface
-#include "action.h" // for call_method
-#include "Array_as.h" // for setPropFlags
-#include "as_function.h" // for inheritance of as_super
+#include "fn_call.h" 
+#include "Object.h" 
+#include "action.h" 
+#include "Array_as.h"
+#include "as_function.h"
+#include "Global_as.h" 
+#include "GnashAlgorithm.h"
 
 #include <set>
 #include <string>
@@ -43,11 +47,12 @@
 #include "asName.h"
 #include "asClass.h"
 
+
+namespace gnash {
+
 // Anonymous namespace used for module-static defs
 namespace {
 
-using namespace gnash;
-
 /// 'super' is a special kind of object
 //
 /// See http://wiki.gnashdev.org/wiki/index.php/ActionScriptSuper
@@ -62,30 +67,24 @@ class as_super : public as_function
 {
 public:
 
-	as_super(as_function* ctor, as_object* proto)
+	as_super(Global_as& gl, as_object* super)
 		:
-		_ctor(ctor),
-		_proto(proto)
+        as_function(gl),
+		_super(super)
 	{
-		set_prototype(proto);
-		//log_debug("as_super %p constructed with ctor %p and proto %p", this, ctor, proto);
+		set_prototype(prototype());
 	}
 
 	virtual bool isSuper() const { return true; }
 
 	virtual as_object* get_super(const char* fname=0);
 
-	std::string get_text_value() const
-	{
-		return "[object Object]";
-	}
-
 	// Fetching members from 'super' yelds a lookup on the associated prototype
 	virtual bool get_member(string_table::key name, as_value* val,
 		string_table::key nsname = 0)
 	{
-		//log_debug("as_super::get_member %s called - _proto is %p", getVM().getStringTable().value(name), _proto);
-		if ( _proto ) return _proto->get_member(name, val, nsname);
+        as_object* proto = prototype();
+		if (proto) return proto->get_member(name, val, nsname);
 		log_debug("Super has no associated prototype");
 		return false;
 	}
@@ -94,6 +93,7 @@ public:
 	virtual void set_member(string_table::key /*key*/, const as_value& /*val*/,
 		string_table::key /*nsname*/ = 0)
 	{
+		log_debug("set_member.");
 		// can't assign to super
 		IF_VERBOSE_ASCODING_ERRORS(
 		log_aserror("Can't set members on the 'super' object");
@@ -103,8 +103,17 @@ public:
 	/// Dispatch.
 	virtual as_value operator()(const fn_call& fn)
 	{
-		//log_debug("Super call operator. fn.this_ptr is %p", fn.this_ptr);
-		if ( _ctor ) return _ctor->call(fn);
+
+        // TODO: this is a hack to make sure objects are constructed, not
+        // converted (fn.isInstantiation() must be true).
+        fn_call::Args::container_type argsIn(fn.getArgs());
+        fn_call::Args args;
+        args.swap(argsIn);
+
+        fn_call fn2(fn.this_ptr, fn.env(), args, fn.super, true);
+        assert(fn2.isInstantiation());
+        as_function* ctor = constructor();
+		if (ctor) return ctor->call(fn2);
 		log_debug("Super has no associated constructor");
 		return as_value();
 	}
@@ -113,15 +122,21 @@ protected:
 
 	virtual void markReachableResources() const
 	{
-		if ( _ctor ) _ctor->setReachable();
-		if ( _proto ) _proto->setReachable();
+		if (_super) _super->setReachable();
 		markAsFunctionReachable();
 	}
 
 private:
 
-	as_function* _ctor;
-	as_object* _proto;
+    as_object* prototype() {
+        return _super ? _super->get_prototype().get() : 0;
+    }
+
+    as_function* constructor() {
+        return _super ? _super->get_constructor() : 0;
+    }
+
+	as_object* _super;
 };
 
 as_object*
@@ -133,85 +148,37 @@ as_super::get_super(const char* fname)
 
 	// Our class prototype is __proto__.
 	as_object* proto = get_prototype().get(); 
-	if ( ! proto )
-	{
-		//log_debug("We (a super) have no associated prototype, returning a null-referencing as_super from get_super()");
-		return new as_super(0, 0);
-	}
+	if (!proto) return new as_super(*getGlobal(*this), 0);
 
-	// proto's __proto__ is superProto 
-	as_object* superProto = proto->get_prototype().get();
+    if (!fname || getSWFVersion(*this) <= 6) {
+        return new as_super(*getGlobal(*this), proto);
+    }
 
-	// proto's __constructor__ is superCtor
-	as_function* superCtor = proto->get_constructor();
-	assert(superCtor == get_constructor());
+    string_table& st = getStringTable(*this);
+    string_table::key k = st.find(fname);
 
-	//log_debug("super %p proto is %p, its prototype %p", this, proto, proto->get_prototype());
+    as_object* owner = 0;
+    proto->findProperty(k, 0, &owner);
+    if (!owner) return 0;
 
-	VM& vm = getVM();
-	if ( fname && vm.getSWFVersion() > 6)
-	{
-		as_object* owner = 0;
-		string_table& st = vm.getStringTable();
-		string_table::key k = st.find(fname);
+    if (owner == proto) return new as_super(*getGlobal(*this), proto);
 
-		proto->findProperty(k, 0, &owner);
-		if ( ! owner )
-		{
-			//log_debug("get_super: can't find property %s", fname);
-			return 0;
-		}
+    as_object* tmp = proto;
+    while (tmp && tmp->get_prototype() != owner) {
+        tmp = tmp->get_prototype().get();
+    }
+    // ok, now 'tmp' should be the object whose __proto__ member
+    // contains the actual named method.
+    //
+    // in the C:B:A:F case this would be B when calling
+    // super.myName() from C.prototype.myName()
+    
+    // well, since we found the property, it must be somewhere!
+    assert(tmp); 
 
-		//log_debug("object containing method %s is %p, its __proto__ is %p", fname, owner, owner->get_prototype());
+    if (tmp != proto) { return new as_super(*getGlobal(*this), tmp); }
+    return new as_super(*getGlobal(*this), owner);
 
-		assert(owner);
-
-		if ( owner != proto )
-		{
-			as_object* tmp = proto;
-			while (tmp && tmp->get_prototype() != owner) tmp = tmp->get_prototype().get();
-			// ok, now 'tmp' should be the object whose __proto__ member contains
-			// the actual named method.
-			//
-			// in the C:B:A:F case this would be B when calling super.myName() from
-			// C.prototype.myName()
-			//
-
-			assert(tmp); // well, since we found the property, it must be somewhere!
-
-			//log_debug("tmp is %p", tmp);
-
-			if ( tmp != proto )
-			{
-				//assert(superProto == tmp->get_prototype().get());
-
-				//superCtor = superProto->get_constructor();
-				superCtor = tmp->get_constructor();
-				//if ( ! superCtor ) log_debug("superProto (owner) has no __constructor__");
-			}
-			else
-			{
-				//log_debug("tmp == proto");
-				superCtor = owner->get_constructor(); // most likely..
-				if ( superProto ) superProto = superProto->get_prototype().get();
-			}
-		}
-		else
-		{
-			// TODO: check if we've anything to do here...
-			//log_debug("owner == proto == %p", owner);
-			//if ( superProto ) superProto = superProto->get_prototype().get();
-			//superCtor = superProto->get_constructor();
-			//if ( superProto )
-			//{
-			//	superCtor = superProto->get_constructor();
-			//} // else superCtor = NULL ?
-		}
-	}
-
-	as_object* super = new as_super(superCtor, superProto);
-
-	return super;
 }
 
 
@@ -246,13 +213,74 @@ public:
 } // end of anonymous namespace
 
 
-namespace gnash {
+/// Destructor of ActiveRelay needs definition of movie_root.
+ActiveRelay::~ActiveRelay()
+{
+    getRoot(*_owner).removeAdvanceCallback(this);
+}
+
+
+const int as_object::DefaultFlags;
+
+as_object::as_object(Global_as& gl)
+	:
+    _relay(0),
+	_vm(getVM(gl)),
+	_members(_vm)
+{
+}
+
+as_object::as_object()
+	:
+    _relay(0),
+	_vm(VM::get()),
+	_members(_vm)
+{
+}
+
+as_object::as_object(as_object* proto)
+	:
+    _relay(0),
+	_vm(VM::get()),
+	_members(_vm)
+{
+	init_member(NSV::PROP_uuPROTOuu, as_value(proto));
+}
+
+as_object::as_object(boost::intrusive_ptr<as_object> proto)
+	:
+    _relay(0),
+	_vm(VM::get()),
+	_members(_vm)
+{
+	init_member(NSV::PROP_uuPROTOuu, as_value(proto));
+}
+
+as_object::as_object(const as_object& other)
+	:
+#ifndef GNASH_USE_GC
+	ref_counted(),
+#else
+	GcResource(), 
+#endif
+    _relay(0),
+	_vm(VM::get()),
+	_members(other._members)
+{
+}
+
+std::pair<bool,bool>
+as_object::delProperty(string_table::key name, string_table::key nsname)
+{
+	return _members.delProperty(name, nsname);
+}
+
 
 void
 as_object::add_property(const std::string& name, as_function& getter,
 		as_function* setter)
 {
-	string_table &st = _vm.getStringTable();
+	string_table &st = getStringTable(*this);
 	string_table::key k = st.find(name);
 
 	as_value cacheVal;
@@ -278,7 +306,7 @@ as_object::add_property(const std::string& name, as_function& getter,
 #if 1
 		// check if we have a trigger, if so, invoke it
 		// and set val to its return
-		TriggerContainer::iterator trigIter = _trigs.find(std::make_pair(k, 0));
+		TriggerContainer::iterator trigIter = _trigs.find(ObjectURI(k, 0));
 		if ( trigIter != _trigs.end() )
 		{
 			Trigger& trig = trigIter->second;
@@ -310,17 +338,17 @@ as_object::get_member(string_table::key name, as_value* val,
 	string_table::key nsname)
 {
 	assert(val);
-
 	Property* prop = findProperty(name, nsname);
-	if (!prop)
-    {
+	
+    if (!prop) {
+
         /// If the property isn't found, try the __resolve property.
         prop = findProperty(NSV::PROP_uuRESOLVE, nsname);
         if (!prop) return false;
 
         /// If __resolve exists, call it with the name of the undefined
         /// property.
-        string_table& st = _vm.getStringTable();
+        string_table& st = getStringTable(*this);
         const std::string& undefinedName = st.value(name);
         log_debug("__resolve exists, calling with '%s'", undefinedName);
 
@@ -328,18 +356,15 @@ as_object::get_member(string_table::key name, as_value* val,
         return true;
     }
 
-	try 
-	{
+	try {
 		*val = prop->getValue(*this);
 		return true;
 	}
-	catch (ActionLimitException& exc)
-	{
+	catch (ActionLimitException& exc) {
 		// will be logged by outer catcher
 		throw;
 	}
-	catch (ActionTypeError& exc)
-	{
+	catch (ActionTypeError& exc) {
 		// TODO: check if this should be an 'as' error.. (log_aserror)
 		log_error(_("Caught exception: %s"), exc.what());
 		return false;
@@ -374,23 +399,17 @@ as_object::get_super(const char* fname)
 	// Our class prototype is __proto__.
 	as_object* proto = get_prototype().get();
 
-	VM& vm = getVM();
-	if ( fname && vm.getSWFVersion() > 6)
+	if ( fname && getSWFVersion(*this) > 6)
 	{
 		as_object* owner = 0;
-		string_table& st = vm.getStringTable();
+		string_table& st = getStringTable(*this);
 		string_table::key k = st.find(fname);
-		/*Property* p =*/ findProperty(k, 0, &owner);
-		if ( owner != this ) proto = owner; // should be 0 if findProperty returned 0
+		findProperty(k, 0, &owner);
+        // should be 0 if findProperty returned 0
+		if (owner != this) proto = owner; 
 	}
 
-	// proto's __proto__ is superProto 
-	as_object* superProto = proto ? proto->get_prototype().get() : 0;
-
-	// proto's __constructor__ is superCtor
-	as_function* superCtor = proto ? proto->get_constructor() : 0;
-
-	as_object* super = new as_super(superCtor, superProto);
+	as_object* super = new as_super(*getGlobal(*this), proto);
 
 	return super;
 }
@@ -451,7 +470,7 @@ Property*
 as_object::findProperty(string_table::key key, string_table::key nsname, 
 	as_object **owner)
 {
-	int swfVersion = _vm.getSWFVersion();
+	int swfVersion = getSWFVersion(*this);
 
 	// don't enter an infinite loop looking for __proto__ ...
 	if (key == NSV::PROP_uuPROTOuu && !nsname)
@@ -459,13 +478,12 @@ as_object::findProperty(string_table::key key, string_table::key nsname,
 		Property* prop = _members.getProperty(key, nsname);
 		// TODO: add ignoreVisibility parameter to allow using 
         // __proto__ even when not visible ?
-		if (prop && prop->isVisible(swfVersion))
+		if (prop && prop->visible(swfVersion))
 		{
-			if (owner != NULL)
-				*owner = this;
+			if (owner) *owner = this;
 			return prop;
 		}
-		return NULL;
+		return 0;
 	}
 
 	// keep track of visited objects, avoid infinite loops.
@@ -484,11 +502,10 @@ as_object::findProperty(string_table::key key, string_table::key nsname,
 		if ((i > 255 && swfVersion == 5) || i > 257)
 			throw ActionLimitException("Lookup depth exceeded.");
 
-		Property* prop = obj->_members.getProperty(key);
-		if (prop && prop->isVisible(swfVersion) )
+		Property* prop = obj->_members.getProperty(key, nsname);
+		if (prop && prop->visible(swfVersion) )
 		{
-			if (owner != NULL)
-				*owner = obj.get();
+			if (owner) *owner = obj.get();
 			return prop;
 		}
 		else
@@ -500,16 +517,18 @@ as_object::findProperty(string_table::key key, string_table::key nsname,
 }
 
 Property*
-as_object::findUpdatableProperty(string_table::key key, string_table::key nsname)
+as_object::findUpdatableProperty(const ObjectURI& uri)
 {
-	int swfVersion = _vm.getSWFVersion();
+    const string_table::key key = getName(uri), nsname = getNamespace(uri);
+
+	const int swfVersion = getSWFVersion(*this);
 
 	Property* prop = _members.getProperty(key, nsname);
 	// 
 	// We won't scan the inheritance chain if we find a member,
 	// even if invisible.
 	// 
-	if ( prop )	return prop;  // TODO: what about isVisible ?
+	if ( prop )	return prop;  // TODO: what about visible ?
 
 	// don't enter an infinite loop looking for __proto__ ...
 	if (key == NSV::PROP_uuPROTOuu) return NULL;
@@ -529,7 +548,7 @@ as_object::findUpdatableProperty(string_table::key key, string_table::key nsname
 			throw ActionLimitException("Property lookup depth exceeded.");
 
 		Property* p = obj->_members.getProperty(key, nsname);
-		if (p && (p->isGetterSetter() | p->isStatic()) && p->isVisible(swfVersion))
+		if (p && (p->isGetterSetter() | p->isStatic()) && p->visible(swfVersion))
 		{
 			return p; // What should we do if this is not a getter/setter ?
 		}
@@ -538,22 +557,95 @@ as_object::findUpdatableProperty(string_table::key key, string_table::key nsname
 	return NULL;
 }
 
-/*protected*/
 void
-as_object::set_prototype(boost::intrusive_ptr<as_object> proto, int flags)
+as_object::set_prototype(const as_value& proto, int flags)
 {
 	// TODO: check what happens if __proto__ is set as a user-defined 
     // getter/setter
 	// TODO: check triggers !!
-	_members.setValue(NSV::PROP_uuPROTOuu, as_value(proto.get()), *this, 0,
-            flags);
+	_members.setValue(NSV::PROP_uuPROTOuu, proto, *this, 0, flags);
 }
 
 void
-as_object::reserveSlot(string_table::key name, string_table::key nsId,
-	unsigned short slotId)
+as_object::reserveSlot(const ObjectURI& uri, boost::uint16_t slotId)
 {
-	_members.reserveSlot(name, nsId, slotId);
+	_members.reserveSlot(uri, slotId);
+}
+
+bool
+as_object::get_member_slot(int order, as_value* val){
+	
+	const Property* prop = _members.getPropertyByOrder(order);
+	if (prop) {
+		return get_member(prop->getName(), val, prop->getNamespace());
+	}
+    return false;
+}
+
+
+bool
+as_object::set_member_slot(int order, const as_value& val, bool ifFound)
+{
+	const Property* prop = _members.getPropertyByOrder(order);
+	if (prop) {
+		return set_member(prop->getName(), val, prop->getNamespace(), ifFound);
+	}
+    return false;
+}
+
+void
+as_object::executeTriggers(Property* prop, const ObjectURI& uri,
+        const as_value& val)
+{
+
+    // check if we have a trigger, if so, invoke it
+    // and set val to it's return
+    TriggerContainer::iterator trigIter = _trigs.find(uri);
+    
+    if (trigIter == _trigs.end()) {
+        if (prop) {
+            prop->setValue(*this, val);
+            prop->clearVisible(getSWFVersion(*this));
+        }
+        return;
+    }
+
+    Trigger& trig = trigIter->second;
+
+    if (trig.dead()) {
+        _trigs.erase(trigIter);
+        return;
+    }
+
+    // WARNING: getValue might itself invoke a trigger
+    // (getter-setter)... ouch ?
+    // TODO: in this case, return the underlying value !
+    as_value curVal = prop ? prop->getCache() : as_value(); 
+
+    log_debug("Existing property %s is being watched: "
+            "firing trigger on update (current val:%s, "
+            "new val:%s)",
+            getStringTable(*this).value(getName(uri)), curVal, val);
+
+    as_value newVal = trig.call(curVal, val, *this);
+    
+    // This is a particularly clear and concise way of removing dead triggers.
+    EraseIf(_trigs, boost::bind(boost::mem_fn(&Trigger::dead), 
+            boost::bind(SecondElement<TriggerContainer::value_type>(), _1)));
+                    
+    // The trigger call could have deleted the property,
+    // so we check for its existance again, and do NOT put
+    // it back in if it was deleted
+    prop = findUpdatableProperty(uri);
+    if (!prop) {
+        log_debug("Property %s deleted by trigger on update",
+                getStringTable(*this).value(getName(uri)));
+        // Return true?
+        return;
+    }
+    prop->setValue(*this, newVal); 
+    prop->clearVisible(getSWFVersion(*this));
+    
 }
 
 // Handles read_only and static properties properly.
@@ -561,107 +653,45 @@ bool
 as_object::set_member(string_table::key key, const as_value& val,
 	string_table::key nsname, bool ifFound)
 {
-	//log_debug(_("set_member_default(%s)"), key);
-	Property* prop = findUpdatableProperty(key, nsname);
-	if (prop)
-	{
-		if (prop->isReadOnly())
-		{
-			IF_VERBOSE_ASCODING_ERRORS(log_aserror(_(""
-				"Attempt to set read-only property '%s'"),
-				_vm.getStringTable().value(key)););
+
+    ObjectURI uri(key, nsname);
+	Property* prop = findUpdatableProperty(uri);
+	
+    if (prop) {
+
+		if (prop->isReadOnly()) {
+			IF_VERBOSE_ASCODING_ERRORS(
+                    log_aserror(_("Attempt to set read-only property '%s'"),
+                    getStringTable(*this).value(key));
+            );
 			return true;
 		}
 
-		try
-		{
-			// check if we have a trigger, if so, invoke it
-			// and set val to it's return
-			TriggerContainer::iterator trigIter = _trigs.find(std::make_pair(key, nsname));
-			if ( trigIter != _trigs.end() )
-			{
-				Trigger& trig = trigIter->second;
-
-				// WARNING: getValue might itself invoke a trigger
-				// (getter-setter)... ouch ?
-				// TODO: in this case, return the underlying value !
-				as_value curVal = prop->getCache(); // getValue(*this); 
-
-				log_debug("Existing property %s is being watched: firing trigger on update (current val:%s, new val:%s)",
-					_vm.getStringTable().value(key), curVal, val);
-				as_value newVal = trig.call(curVal, val, *this);
-				// The trigger call could have deleted the property,
-				// so we check for its existance again, and do NOT put
-				// it back in if it was deleted
-				prop = findUpdatableProperty(key, nsname);
-				if ( ! prop )
-				{
-					log_debug("Property %s deleted by trigger on update", _vm.getStringTable().value(key));
-					return true;
-				}
-
-				//if ( prop->isGetterSetter() ) prop->setCache(newVal); 
-				prop->setValue(*this, newVal); 
-			}
-			else
-			{
-				// log_debug("No trigger for key %d ns %d", key, nsname);
-				prop->setValue(*this, val);
-			}
-
-			prop->clearVisible(_vm.getSWFVersion());
+		try {
+            executeTriggers(prop, uri, val);
 		}
-		catch (ActionTypeError& exc)
-		{
+		catch (ActionTypeError& exc) {
 			log_aserror(_("%s: Exception %s. Will create a new member"),
-				_vm.getStringTable().value(key), exc.what());
+				getStringTable(*this).value(key), exc.what());
 		}
 
 		return true;
 	}
 
 	// Else, add new property...
-	if ( ifFound ) return false;
+	if (ifFound) return false;
 
 	// Property does not exist, so it won't be read-only. Set it.
-	if (!_members.setValue(key, val, *this, nsname))
-	{
+	if (!_members.setValue(key, val, *this, nsname)) {
+
 		IF_VERBOSE_ASCODING_ERRORS(
 			log_aserror(_("Unknown failure in setting property '%s' on "
-			"object '%p'"), _vm.getStringTable().value(key), (void*) this);
+			"object '%p'"), getStringTable(*this).value(key), (void*) this);
 	    );
 		return false;
 	}
 
-	// Now check if we have a trigger, if so, invoke it
-	// and reset val to it's return
-	// NOTE that we do this *after* setting it in first place 
-	// as the trigger seems allowed to delete the property again
-	TriggerContainer::iterator trigIter = _trigs.find(std::make_pair(key, nsname));
-	if ( trigIter != _trigs.end() )
-	{
-		Trigger& trig = trigIter->second;
-
-		log_debug("Property %s is being watched, calling trigger on create", _vm.getStringTable().value(key));
-
-		// NOTE: the trigger call might delete the propery being added
-		//       so we first add the property, then call the trigger
-		//       and finally check if the property still exists (ufff...)
-		//
-
-		as_value curVal; // undefined, didn't exist...
-		as_value newVal = trig.call(curVal, val, *this);
-		Property* prop = _members.getProperty(key);
-		if ( ! prop )
-		{
-			log_debug("Property %s deleted by trigger on create", 
-                    _vm.getStringTable().value(key));
-		}
-		else
-		{
-			prop->setValue(*this, newVal);
-		}
-	}
+    executeTriggers(prop, uri, val);
 
 	return false;
 }
@@ -671,7 +701,7 @@ void
 as_object::init_member(const std::string& key1, const as_value& val, int flags,
 	string_table::key nsname)
 {
-	init_member(_vm.getStringTable().find(PROPNAME(key1)), val, flags, nsname);
+	init_member(getStringTable(*this).find(key1), val, flags, nsname);
 }
 
 void
@@ -680,7 +710,8 @@ as_object::init_member(string_table::key key, const as_value& val, int flags,
 {
 
 	if (order >= 0 && !_members.
-		reserveSlot(static_cast<unsigned short>(order), key, nsname))
+		reserveSlot(ObjectURI(key, nsname),
+            static_cast<boost::uint16_t>(order)))
 	{
 		log_error(_("Attempt to set a slot for either a slot or a property "
 			"which already exists."));
@@ -692,7 +723,7 @@ as_object::init_member(string_table::key key, const as_value& val, int flags,
 	{
 		log_error(_("Attempt to initialize read-only property ``%s''"
 			" on object ``%p'' twice"),
-			_vm.getStringTable().value(key), (void*)this);
+			getStringTable(*this).value(key), (void*)this);
 		// We shouldn't attempt to initialize a member twice, should we ?
 		abort();
 	}
@@ -702,7 +733,7 @@ void
 as_object::init_property(const std::string& key, as_function& getter,
 		as_function& setter, int flags, string_table::key nsname)
 {
-	string_table::key k = _vm.getStringTable().find(PROPNAME(key));
+	string_table::key k = getStringTable(*this).find(PROPNAME(key));
 	init_property(k, getter, setter, flags, nsname);
 }
 
@@ -721,7 +752,7 @@ void
 as_object::init_property(const std::string& key, as_c_function_ptr getter,
 		as_c_function_ptr setter, int flags, string_table::key nsname)
 {
-	string_table::key k = _vm.getStringTable().find(PROPNAME(key));
+	string_table::key k = getStringTable(*this).find(PROPNAME(key));
 	init_property(k, getter, setter, flags, nsname);
 }
 
@@ -756,10 +787,10 @@ void
 as_object::init_readonly_property(const std::string& key, as_function& getter,
 	int initflags, string_table::key nsname)
 {
-	string_table::key k = _vm.getStringTable().find(PROPNAME(key));
+	string_table::key k = getStringTable(*this).find(PROPNAME(key));
 
-	init_property(k, getter, getter, initflags | as_prop_flags::readOnly
-		| as_prop_flags::isProtected, nsname);
+	init_property(k, getter, getter, initflags | PropFlags::readOnly
+		| PropFlags::isProtected, nsname);
 	assert(_members.getProperty(k, nsname));
 }
 
@@ -767,8 +798,8 @@ void
 as_object::init_readonly_property(const string_table::key& k,
         as_function& getter, int initflags, string_table::key nsname)
 {
-	init_property(k, getter, getter, initflags | as_prop_flags::readOnly
-		| as_prop_flags::isProtected, nsname);
+	init_property(k, getter, getter, initflags | PropFlags::readOnly
+		| PropFlags::isProtected, nsname);
 	assert(_members.getProperty(k, nsname));
 }
 
@@ -776,10 +807,10 @@ void
 as_object::init_readonly_property(const std::string& key,
         as_c_function_ptr getter, int initflags, string_table::key nsname)
 {
-	string_table::key k = _vm.getStringTable().find(PROPNAME(key));
+	string_table::key k = getStringTable(*this).find(PROPNAME(key));
 
-	init_property(k, getter, getter, initflags | as_prop_flags::readOnly
-		| as_prop_flags::isProtected, nsname);
+	init_property(k, getter, getter, initflags | PropFlags::readOnly
+		| PropFlags::isProtected, nsname);
 	assert(_members.getProperty(k, nsname));
 }
 
@@ -787,8 +818,8 @@ void
 as_object::init_readonly_property(const string_table::key& k,
         as_c_function_ptr getter, int initflags, string_table::key nsname)
 {
-	init_property(k, getter, getter, initflags | as_prop_flags::readOnly
-		| as_prop_flags::isProtected, nsname);
+	init_property(k, getter, getter, initflags | PropFlags::readOnly
+		| PropFlags::isProtected, nsname);
 	assert(_members.getProperty(k, nsname));
 }
 
@@ -812,7 +843,9 @@ as_object::add_interface(as_object* obj)
 bool
 as_object::instanceOf(as_object* ctor)
 {
-//#define GNASH_DEBUG_INSTANCE_OF 1
+
+    /// An object is never an instance of a null prototype.
+    if (!ctor) return false;
 
 	as_value protoVal;
 	if ( ! ctor->get_member(NSV::PROP_PROTOTYPE, &protoVal) )
@@ -823,7 +856,7 @@ as_object::instanceOf(as_object* ctor)
 #endif
 		return false;
 	}
-	as_object* ctorProto = protoVal.to_object().get();
+	as_object* ctorProto = protoVal.to_object(*getGlobal(*this)).get();
 	if ( ! ctorProto )
 	{
 #ifdef GNASH_DEBUG_INSTANCE_OF
@@ -950,7 +983,7 @@ as_object::setPropFlags(const as_value& props_val, int set_false, int set_true)
 			}
 
 			// set_member_flags will take care of case conversion
-			if (!set_member_flags(_vm.getStringTable().find(prop), set_true, set_false) )
+			if (!set_member_flags(getStringTable(*this).find(prop), set_true, set_false) )
 			{
 				IF_VERBOSE_ASCODING_ERRORS(
 				log_aserror(_("Can't set propflags on object "
@@ -983,7 +1016,7 @@ as_object::setPropFlags(const as_value& props_val, int set_false, int set_true)
 		return;
 	}
 
-	boost::intrusive_ptr<as_object> props = props_val.to_object();
+	boost::intrusive_ptr<as_object> props = props_val.to_object(*getGlobal(*this));
 	Array_as* ary = dynamic_cast<Array_as*>(props.get());
 	if ( ! ary )
 	{
@@ -998,7 +1031,8 @@ as_object::setPropFlags(const as_value& props_val, int set_false, int set_true)
 
 	// The passed argument has to be considered an array
 	//std::pair<size_t, size_t> result = 
-	FlagsSetterVisitor visitor(getVM().getStringTable(), _members, set_true, set_false);
+	FlagsSetterVisitor visitor(getStringTable(*this), _members, set_true,
+            set_false);
 	ary->visitAll(visitor);
 	//_members.setFlagsAll(props->_members, set_true, set_false);
 }
@@ -1016,7 +1050,7 @@ as_object::copyProperties(const as_object& o)
 void
 as_object::enumerateProperties(as_environment& env) const
 {
-	assert( env.top(0).is_null() );
+	assert(env.top(0).is_undefined());
 
 	enumerateNonProperties(env);
 
@@ -1055,54 +1089,6 @@ as_object::enumerateProperties(SortedPropertyList& to) const
 
 }
 
-as_object::as_object(movie_root& mr)
-	:
-	_vm(mr.getVM()),
-	_members(_vm)
-{
-}
-
-
-as_object::as_object()
-	:
-	_vm(VM::get()),
-	_members(_vm)
-{
-}
-
-as_object::as_object(as_object* proto)
-	:
-	_vm(VM::get()),
-	_members(_vm)
-{
-	init_member(NSV::PROP_uuPROTOuu, as_value(proto));
-}
-
-as_object::as_object(boost::intrusive_ptr<as_object> proto)
-	:
-	_vm(VM::get()),
-	_members(_vm)
-{
-	init_member(NSV::PROP_uuPROTOuu, as_value(proto));
-}
-
-as_object::as_object(const as_object& other)
-	:
-#ifndef GNASH_USE_GC
-	ref_counted(),
-#else
-	GcResource(), 
-#endif
-	_vm(VM::get()),
-	_members(other._members)
-{
-}
-
-std::pair<bool,bool>
-as_object::delProperty(string_table::key name, string_table::key nsname)
-{
-	return _members.delProperty(name, nsname);
-}
 
 Property*
 as_object::getOwnProperty(string_table::key key, string_table::key nsname)
@@ -1116,35 +1102,18 @@ as_object::hasOwnProperty(string_table::key key, string_table::key nsname)
 	return getOwnProperty(key, nsname) != NULL;
 }
 
-as_value
-as_object::tostring_method(const fn_call& fn)
-{
-	boost::intrusive_ptr<as_object> obj = fn.this_ptr;
-
-	std::string text_val = obj->get_text_value();
-	return as_value(text_val);
-}
-
-as_value
-as_object::valueof_method(const fn_call& fn)
-{
-	boost::intrusive_ptr<as_object> obj = fn.this_ptr;
-
-	return obj->get_primitive_value();
-}
-
 boost::intrusive_ptr<as_object>
 as_object::get_prototype()
 {
-	int swfVersion = _vm.getSWFVersion();
+	int swfVersion = getSWFVersion(*this);
 
 	Property* prop = _members.getProperty(NSV::PROP_uuPROTOuu);
 	if ( ! prop ) return 0;
-	if ( ! prop->isVisible(swfVersion) ) return 0;
+	if ( ! prop->visible(swfVersion) ) return 0;
 
 	as_value tmp = prop->getValue(*this);
 
-	return tmp.to_object();
+	return tmp.to_object(*getGlobal(*this));
 }
 
 bool
@@ -1154,7 +1123,7 @@ as_object::on_event(const event_id& id )
 
 	if (get_member(id.functionKey(), &event_handler) )
 	{
-		call_method0(event_handler, NULL, this);
+		call_method0(event_handler, as_environment(_vm), this);
 		return true;
 	}
 
@@ -1181,7 +1150,7 @@ as_object::callMethod(string_table::key methodName)
 
 	as_environment env(_vm);
 
-	return call_method0(method, &env, this);
+	return call_method0(method, env, this);
 }
 
 as_value
@@ -1196,10 +1165,10 @@ as_object::callMethod(string_table::key methodName, const as_value& arg0)
 
 	as_environment env(_vm);
 
-	std::auto_ptr< std::vector<as_value> > args ( new std::vector<as_value> );
-	args->push_back(arg0);
+    fn_call::Args args;
+    args += arg0;
 
-	return call_method(method, &env, this, args);
+	return call_method(method, env, this, args);
 }
 
 as_value
@@ -1215,11 +1184,10 @@ as_object::callMethod(string_table::key methodName, const as_value& arg0,
 
 	as_environment env(_vm);
 
-	std::auto_ptr< std::vector<as_value> > args ( new std::vector<as_value> );
-	args->push_back(arg0);
-	args->push_back(arg1);
+    fn_call::Args args;
+    args += arg0, arg1;
 
-	return call_method(method, &env, this, args);
+	return call_method(method, env, this, args);
 }
 
 as_value
@@ -1236,12 +1204,10 @@ as_object::callMethod(string_table::key methodName,
 
 	as_environment env(_vm);
 
-	std::auto_ptr< std::vector<as_value> > args ( new std::vector<as_value> );
-	args->push_back(arg0);
-	args->push_back(arg1);
-	args->push_back(arg2);
+    fn_call::Args args;
+    args += arg0, arg1, arg2;
 
-	ret = call_method(method, &env, this, args);
+	ret = call_method(method, env, this, args);
 
 	return ret;
 }
@@ -1259,13 +1225,10 @@ as_object::callMethod(string_table::key methodName, const as_value& arg0,
 
 	as_environment env(_vm);
 
-	std::auto_ptr< std::vector<as_value> > args ( new std::vector<as_value> );
-	args->push_back(arg0);
-	args->push_back(arg1);
-	args->push_back(arg2);
-	args->push_back(arg3);
+    fn_call::Args args;
+    args += arg0, arg1, arg2, arg3;
 
-	return call_method(method, &env, this, args);
+	return call_method(method, env, this, args);
 
 }
 
@@ -1279,7 +1242,7 @@ as_object::get_path_element(string_table::key key)
 	{
 #ifdef DEBUG_TARGET_FINDING 
 		log_debug("Member %s not found in object %p",
-			_vm.getStringTable().value(key), (void*)this);
+			getStringTable(*this).value(key), (void*)this);
 #endif
 		return NULL;
 	}
@@ -1287,12 +1250,12 @@ as_object::get_path_element(string_table::key key)
 	{
 #ifdef DEBUG_TARGET_FINDING 
 		log_debug("Member %s of object %p is not an object (%s)",
-			_vm.getStringTable().value(key), (void*)this, tmp);
+			getStringTable(*this).value(key), (void*)this, tmp);
 #endif
 		return NULL;
 	}
 
-	return tmp.to_object().get();
+	return tmp.to_object(*getGlobal(*this)).get();
 }
 
 void
@@ -1325,11 +1288,11 @@ as_object::watch(string_table::key key, as_function& trig,
 		const as_value& cust, string_table::key ns)
 {
 	
-	FQkey k = std::make_pair(key, ns);
-	std::string propname = VM::get().getStringTable().value(key);
+	ObjectURI k(key, ns);
+	std::string propname = getStringTable(*this).value(key);
 
 	TriggerContainer::iterator it = _trigs.find(k);
-	if ( it == _trigs.end() )
+	if (it == _trigs.end())
 	{
 		return _trigs.insert(
                 std::make_pair(k, Trigger(propname, trig, cust))).second;
@@ -1341,21 +1304,21 @@ as_object::watch(string_table::key key, as_function& trig,
 bool
 as_object::unwatch(string_table::key key, string_table::key ns)
 {
-	TriggerContainer::iterator trigIter = _trigs.find(std::make_pair(key, ns));
+	TriggerContainer::iterator trigIter = _trigs.find(ObjectURI(key, ns));
 	if ( trigIter == _trigs.end() )
 	{
 		log_debug("No watch for property %s",
-                getVM().getStringTable().value(key));
+                getStringTable(*this).value(key));
 		return false;
 	}
 	Property* prop = _members.getProperty(key, ns);
 	if ( prop && prop->isGetterSetter() )
 	{
 		log_debug("Watch on %s not removed (is a getter-setter)",
-                getVM().getStringTable().value(key));
+                getStringTable(*this).value(key));
 		return false;
 	}
-	_trigs.erase(trigIter);
+	trigIter->second.kill();
 	return true;
 }
 
@@ -1370,6 +1333,9 @@ as_object::markAsObjectReachable() const
 	{
 		it->second.setReachable();
 	}
+
+    /// Proxy objects can contain references to other as_objects.
+    if (_relay) _relay->setReachable();
 }
 #endif // GNASH_USE_GC
 
@@ -1381,8 +1347,11 @@ Trigger::setReachable() const
 }
 
 as_value
-Trigger::call(const as_value& oldval, const as_value& newval, as_object& this_obj)
+Trigger::call(const as_value& oldval, const as_value& newval,
+        as_object& this_obj)
 {
+    assert(!_dead);
+
 	if ( _executing ) return newval;
 
 	_executing = true;
@@ -1390,11 +1359,8 @@ Trigger::call(const as_value& oldval, const as_value& newval, as_object& this_ob
 	try {
 		as_environment env(VM::get()); // TODO: get VM in some other way 
 
-		std::auto_ptr<std::vector<as_value> > args (new std::vector<as_value>);
-		args->push_back(_propname);
-		args->push_back(oldval);
-		args->push_back(newval);
-		args->push_back(_customArg);
+        fn_call::Args args;
+        args += _propname, oldval, newval, _customArg;
 
 		fn_call fn(&this_obj, env, args);
 
@@ -1424,6 +1390,43 @@ as_object::visitNonHiddenPropertyValues(AbstractPropertyVisitor& visitor) const
     _members.visitNonHiddenValues(visitor, *this);
 }
 
+/// Get the VM from an as_object
+VM&
+getVM(const as_object& o)
+{
+    return o.vm();
+}
+
+/// Get the movie_root from an as_object
+movie_root&
+getRoot(const as_object& o)
+{
+    return o.vm().getRoot();
+}
+
+/// Get the string_table from an as_object
+string_table&
+getStringTable(const as_object& o)
+{
+    return o.vm().getStringTable();
+}
+
+const RunResources&
+getRunResources(const as_object& o)
+{
+    return o.vm().getRoot().runResources();
+}
+
+int
+getSWFVersion(const as_object& o)
+{
+    return o.vm().getSWFVersion();
+}
+
+Global_as* getGlobal(const as_object& o)
+{
+    return o.vm().getGlobal();
+}
 
 
 } // end of gnash namespace
