@@ -36,6 +36,7 @@
 #include <boost/type_traits/is_floating_point.hpp>
 #include <boost/utility/enable_if.hpp>
 #include <boost/static_assert.hpp>
+#include <boost/shared_ptr.hpp>
 
 #include "utility.h" // UNUSED
 #include "string_table.h"
@@ -44,10 +45,11 @@
 namespace gnash {
     class VM;
 	class as_object;
+	class Global_as;
 	class fn_call;
 	class as_function;
 	class MovieClip;
-	class character;
+	class DisplayObject;
 	class asNamespace;
 	class asName;
     class SimpleBuffer;
@@ -118,6 +120,10 @@ public:
 		NULLTYPE,
 		NULLTYPE_EXCEPT,
 
+		/// NULL value
+		UNSUPPORTED,
+		UNSUPPORTED_EXCEPT,
+
 		/// Boolean value
 		BOOLEAN,
 		BOOLEAN_EXCEPT,
@@ -276,9 +282,9 @@ public:
     /// @param d      The 32-bit int represented as a double. This is only a
     ///               valid number if the return value is true.
     /// @param whole  If true, expect the whole string to be valid, i.e.
-    ///               throw if there are any invalid characters. If false,
+    ///               throw if there are any invalid DisplayObjects. If false,
     ///               returns any valid number up to the first invalid
-    ///               character.
+    ///               DisplayObject.
     /// @return       True if the string was non-decimal and successfully
     ///               parsed.
     static bool parseNonDecimalInt(const std::string& s, double& d,
@@ -288,6 +294,8 @@ public:
 	const char* typeOf() const;
 
 	/// Get the primitive type of this value
+    //
+    /// Only used in AVM2
 	primitive_types ptype() const;
 
 	// Chad: Document
@@ -370,7 +378,7 @@ public:
 	double	to_number() const;
 
 	/// Get an AMF element representation for this value
-	std::auto_ptr<amf::Element> to_element() const;
+        boost::shared_ptr<amf::Element> to_element() const;
 
 	/// Conversion to 32bit integer
 	//
@@ -427,54 +435,48 @@ public:
 	/// as the returned object might be a newly allocated one in case
 	/// of a conversion from a primitive string, number or boolean value.
 	///
-	/// string values will be converted to String objects,
-	/// numeric values will be converted to Number objects,
-	/// boolean values are currently NOT converted, but should (FIXME!)
+	/// string values are converted to String objects
+	/// numeric values are converted to Number objects
+	/// boolean values are converted to Boolean objects
 	///
 	/// If you want to avoid the conversion, check with is_object() before
 	/// calling this function.
-	///
-	boost::intrusive_ptr<as_object> to_object() const;
+    //
+    /// @param global   The global object object for the conversion. This
+    ///                 contains the prototypes or constructors necessary for
+    ///                 conversion.
+	boost::intrusive_ptr<as_object> to_object(Global_as& global) const;
 
 	/// Return value as a sprite or NULL if this is not possible.
 	//
-	/// This is just a wrapper around to_character() performing 
+	/// This is just a wrapper around toDisplayObject() performing 
 	/// an additional final cast.
 	///
 	MovieClip* to_sprite(bool skipRebinding=false) const;
 
-	/// Return value as a character or NULL if this is not possible.
+	/// Return value as a DisplayObject or NULL if this is not possible.
 	//
-	/// If the value is a MOVIECLIP value, the stored character target
+	/// If the value is a MOVIECLIP value, the stored DisplayObject target
 	/// is evaluated using the root movie's environment.
-	/// If the target points to something that doesn't cast to a character,
+	/// If the target points to something that doesn't cast to a DisplayObject,
 	/// NULL is returned.
 	///
 	/// Note that if the value is NOT a MOVIECLIP type, NULL is always
 	/// returned.
 	///
 	/// @param skipRebinding
-	/// 	If true a reference to a destroyed character is still returned
+	/// 	If true a reference to a destroyed DisplayObject is still returned
 	///	as such, rather then attempted to be resolved as a soft-reference.
 	///	Main use for this is during paths resolution, to avoid
 	///	infinite loops. See bug #21647.
 	///
-	character* to_character(bool skipRebinding=false) const;
+	DisplayObject* toDisplayObject(bool skipRebinding=false) const;
 
 	/// \brief
 	/// Return value as an ActionScript function ptr
 	/// or NULL if it is not an ActionScript function.
 	as_function*	to_as_function() const;
 
-	/// Convert this value to a primitive type
-	//
-	/// Primitive types are: undefined, null, boolean, string, number.
-	/// See ECMA-2.6.2 (sections 4.3.2 and 8.6.2.6).
-	///
-	/// @throw ActionTypeError if an object can't be converted to a primitive
-	///
-	as_value& convert_to_primitive();
-
 	/// Return value as a primitive type
 	//
 	/// Primitive types are: undefined, null, boolean, string, number.
@@ -506,7 +508,7 @@ public:
 	///
 	/// @throw ActionTypeError if an object can't be converted to a primitive
 	///
-	as_value& convert_to_primitive(AsType hint);
+	as_value& convert_to_primitive();
 
 	/// Force type to number.
 	void convert_to_number();
@@ -540,7 +542,7 @@ public:
 
 	void set_sprite(MovieClip& sp);
 
-	void set_character(character& sp);
+	void setDisplayObject(DisplayObject& sp);
 
 	void set_int(int val) { set_double(val); }
 
@@ -565,6 +567,9 @@ public:
 	/// Set this value to the NULL value
 	void set_null();
 
+	/// Set this value to the Unsupported value
+	void set_unsupported();
+
 	/// Equality operator, follows strict equality semantic
 	//
 	/// See strictly_equals
@@ -590,11 +595,14 @@ public:
 
 	bool is_bool() const { return (m_type == BOOLEAN); }
 
-	bool is_exception() const
+	bool is_unsupported() const { return (m_type == UNSUPPORTED); }
+
+        bool is_exception() const
 	{ return (m_type == UNDEFINED_EXCEPT || m_type == NULLTYPE_EXCEPT
 		|| m_type == BOOLEAN_EXCEPT || m_type == NUMBER_EXCEPT
 		|| m_type == OBJECT_EXCEPT || m_type == AS_FUNCTION_EXCEPT
-		|| m_type == MOVIECLIP_EXCEPT || m_type == STRING_EXCEPT);
+		|| m_type == MOVIECLIP_EXCEPT || m_type == STRING_EXCEPT
+		|| m_type == UNSUPPORTED_EXCEPT);
 	}
 
 	// Flag or unflag an as_value as an exception -- this gets flagged
@@ -655,7 +663,7 @@ private:
 	AsType m_type;
 
 	typedef MovieClip* SpritePtr;
-	typedef character* CharacterPtr;
+	typedef DisplayObject* CharacterPtr;
 	typedef boost::intrusive_ptr<as_function> AsFunPtr;
 	typedef boost::intrusive_ptr<as_object> AsObjPtr;
 	
@@ -685,7 +693,7 @@ private:
 	///
 	SpritePtr getSprite(bool skipRebinding=false) const;
 
-	/// Get the character pointer variant member (we assume m_type == MOVIECLIP)
+	/// Get the DisplayObject pointer variant member (we assume m_type == MOVIECLIP)
 	//
 	/// NOTE: this is possibly NULL !
 	///
