@@ -24,7 +24,7 @@
 
 namespace gnash {
 
-class Date_as : public as_object
+class Date_as : public Relay
 {
 public:
 
@@ -34,20 +34,17 @@ public:
 
     double getTimeValue() const { return _timeValue; }
 
-    static void registerNative(as_object& global);
-
-    static void init(as_object& where);
-
     std::string toString() const;
 
-    bool isDateObject() { return true; }
-
 private:
 
     double _timeValue;
 
 };
 
+void registerDateNative(as_object& global);
+
+void date_class_init(as_object& where, const ObjectURI& uri);
 
 } // end of gnash namespace
 
