@@ -1,6 +1,6 @@
 // Error_as.h:  ActionScript "Error" class, for Gnash.
 //
-//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -24,19 +24,14 @@
 #include "gnashconfig.h"
 #endif
 
-//#include <smart_ptr.h> // for intrusive_ptr and GC stuff
-
 namespace gnash {
 
 class as_object;
+class ObjectURI;
 
 /// Initialize the global Error class
-void Error_class_init(as_object& global);
-
-/// Return a Error instance (in case the core lib needs it)
-//boost::intrusive_ptr<as_object> init_Error_instance();
+void Error_class_init(as_object& global, const ObjectURI& uri);
 
 } // end of gnash namespace
 
-// GNASH_ASOBJ_ERROR_H
 #endif
