@@ -16,10 +16,9 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
 
-#ifndef __GMATH_H__
-#define __GMATH_H__
+#ifndef GNASH_ASOBJ_MATH_H
+#define GNASH_ASOBJ_MATH_H
 
-#include "impl.h"
 #include "as_object.h"
 
 namespace gnash {
@@ -27,7 +26,7 @@ namespace gnash {
 // Math isn't a proper class, so doesn't need a constructor.
 
 void registerMathNative(as_object& global);
-void math_class_init(as_object& global);
+void math_class_init(as_object& global, const ObjectURI& uri);
 
 } // end of gnash namespace
 
