@@ -25,10 +25,12 @@
 #include "action.h" // for call_method
 #include "as_value.h"
 #include "as_object.h" // for inheritance
+#include "movie_root.h"
 #include "fn_call.h"
+#include "Global_as.h"
 #include "as_function.h"
 #include "MovieClip.h"
-#include "character.h" // for loadClip (get_parent)
+#include "DisplayObject.h" // for loadClip (get_parent)
 #include "log.h"
 #include "URL.h" // for url parsing
 #include "VM.h" // for the string table.
@@ -46,41 +48,54 @@
 
 namespace gnash {
 
-  /// This class is used to queue a function call action
-  //
-  /// Exact use is to queue onLoadInit, which should be invoked
-  /// after actions of in first frame of a loaded movie are executed.
-  /// Since those actions are queued the only way to execute something
-  /// after them is to queue the function call as well.
-  ///
-  /// The class might be made more general and accessible outside
-  /// of the MovieClipLoader class. For now it only works for
-  /// calling a function with a two argument.
-  ///
-  class DelayedFunctionCall: public ExecutableCode {
 
-  public:
+// Forward declarations
+namespace {
+    as_value moviecliploader_loadclip(const fn_call& fn);
+    as_value moviecliploader_unloadclip(const fn_call& fn);
+    as_value moviecliploader_getprogress(const fn_call& fn);
+    as_value moviecliploader_new(const fn_call& fn);
+    void attachMovieClipLoaderInterface(as_object& o);
+    as_object* getMovieClipLoaderInterface();
+}
 
-    DelayedFunctionCall(as_object* target, string_table::key name, const as_value& arg1, const as_value& arg2)
-      :
-      _target(target),
-      _name(name),
-      _arg1(arg1),
-      _arg2(arg2)
+/// This class is used to queue a function call action
+//
+/// Exact use is to queue onLoadInit, which should be invoked
+/// after actions of in first frame of a loaded movie are executed.
+/// Since those actions are queued the only way to execute something
+/// after them is to queue the function call as well.
+///
+/// The class might be made more general and accessible outside
+/// of the MovieClipLoader class. For now it only works for
+/// calling a function with a two argument.
+///
+class DelayedFunctionCall : public ExecutableCode
+{
+
+public:
+
+    DelayedFunctionCall(as_object* target, string_table::key name,
+            const as_value& arg1, const as_value& arg2)
+        :
+        _target(target),
+        _name(name),
+        _arg1(arg1),
+        _arg2(arg2)
     {}
 
 
     ExecutableCode* clone() const
     {
-      return new DelayedFunctionCall(*this);
+        return new DelayedFunctionCall(*this);
     }
 
     virtual void execute()
     {
-      _target->callMethod(_name, _arg1, _arg2);
+        _target->callMethod(_name, _arg1, _arg2);
     }
 
-  #ifdef GNASH_USE_GC
+#ifdef GNASH_USE_GC
     /// Mark reachable resources (for the GC)
     //
     /// Reachable resources are:
@@ -92,50 +107,15 @@ namespace gnash {
       _arg1.setReachable();
       _arg2.setReachable();
     }
-  #endif // GNASH_USE_GC
+#endif // GNASH_USE_GC
 
-  private:
+private:
 
     as_object* _target;
     string_table::key _name;
     as_value _arg1, _arg2;
 
-  };
-
-// Forward declarations
-static as_value moviecliploader_loadclip(const fn_call& fn);
-static as_value moviecliploader_unloadclip(const fn_call& fn);
-static as_value moviecliploader_getprogress(const fn_call& fn);
-static as_value moviecliploader_new(const fn_call& fn);
-
-static void
-attachMovieClipLoaderInterface(as_object& o)
-{
-  	o.init_member("loadClip", new builtin_function(moviecliploader_loadclip));
-	o.init_member("unloadClip",
-            new builtin_function(moviecliploader_unloadclip));
-	o.init_member("getProgress",
-            new builtin_function(moviecliploader_getprogress));
-
-	// NOTE: we want addListener/removeListener/broadcastMessage
-	//       but don't what the _listeners property here...
-	// TODO: add an argument to AsBroadcaster::initialize skip listeners ?
-	AsBroadcaster::initialize(o);
-	o.delProperty(NSV::PROP_uLISTENERS);
-  
-}
-
-static as_object*
-getMovieClipLoaderInterface()
-{
-	static boost::intrusive_ptr<as_object> o;
-	if ( o == NULL )
-	{
-		o = new as_object(getObjectInterface());
-		attachMovieClipLoaderInterface(*o);
-	}
-	return o.get();
-}
+};
 
 class MovieClipLoader: public as_object
 {
@@ -143,20 +123,20 @@ public:
 
 	MovieClipLoader();
 
-	~MovieClipLoader();
+	~MovieClipLoader() {}
 
 	/// MovieClip
-	bool loadClip(const std::string& url, MovieClip& target);
+	bool loadClip(const std::string& url, MovieClip* target);
 
 	void unloadClip();
 
-private:
+protected:
 
-	bool          _started;
-	bool          _completed;
-	std::string     _filespec;
-	int           _progress;
-	bool          _error;
+    void markReachableResources() const {
+        markAsObjectReachable();
+    }
+
+private:
 };
 
 MovieClipLoader::MovieClipLoader()
@@ -169,26 +149,22 @@ MovieClipLoader::MovieClipLoader()
 	set_member(NSV::PROP_uLISTENERS, ar);
 }
 
-MovieClipLoader::~MovieClipLoader()
-{
-}
-
 bool
-MovieClipLoader::loadClip(const std::string& url_str, MovieClip& target)
+MovieClipLoader::loadClip(const std::string& url_str, MovieClip* target)
 {
     
-    movie_root& mr = _vm.getRoot();
+    movie_root& mr = getRoot(*this);
 
-	URL url(url_str, mr.runInfo().baseURL());
+	URL url(url_str, mr.runResources().baseURL());
 	
 #if GNASH_DEBUG
 	log_debug(_(" resolved url: %s"), url.str());
 #endif
 			 
-	as_value targetVal(&target);
+	as_value targetVal(target);
 	log_debug("Target is %s", targetVal);
 
-	bool ret = target.loadMovie(url);
+	bool ret = target->loadMovie(url);
 	if ( ! ret ) 
 	{
 
@@ -238,7 +214,7 @@ MovieClipLoader::loadClip(const std::string& url_str, MovieClip& target)
             new DelayedFunctionCall(this, NSV::PROP_BROADCAST_MESSAGE, 
                 "onLoadInit", targetVal));
 
-	_vm.getRoot().pushAction(code, movie_root::apDOACTION);
+	getRoot(*this).pushAction(code, movie_root::apDOACTION);
 
 	return true;
 }
@@ -246,10 +222,55 @@ MovieClipLoader::loadClip(const std::string& url_str, MovieClip& target)
 void
 MovieClipLoader::unloadClip()
 {
-  GNASH_REPORT_FUNCTION;
+    GNASH_REPORT_FUNCTION;
 }
 
-static as_value
+/// Extern.
+void
+moviecliploader_class_init(as_object& global, const ObjectURI& uri)
+{
+	// This is going to be the global Number "class"/"function"
+    Global_as* gl = getGlobal(global);
+    as_object* cl = gl->createClass(&moviecliploader_new,
+                getMovieClipLoaderInterface());
+	global.init_member(getName(uri), cl, as_object::DefaultFlags,
+            getNamespace(uri)); 
+}
+
+
+namespace {
+
+void
+attachMovieClipLoaderInterface(as_object& o)
+{
+    Global_as* gl = getGlobal(o);
+  	o.init_member("loadClip", gl->createFunction(moviecliploader_loadclip));
+	o.init_member("unloadClip",
+            gl->createFunction(moviecliploader_unloadclip));
+	o.init_member("getProgress",
+            gl->createFunction(moviecliploader_getprogress));
+
+	// NOTE: we want addListener/removeListener/broadcastMessage
+	//       but don't what the _listeners property here...
+	// TODO: add an argument to AsBroadcaster::initialize skip listeners ?
+	AsBroadcaster::initialize(o);
+	o.delProperty(NSV::PROP_uLISTENERS);
+  
+}
+
+as_object*
+getMovieClipLoaderInterface()
+{
+	static boost::intrusive_ptr<as_object> o;
+	if ( o == NULL )
+	{
+		o = new as_object(getObjectInterface());
+		attachMovieClipLoaderInterface(*o);
+	}
+	return o.get();
+}
+
+as_value
 moviecliploader_loadclip(const fn_call& fn)
 {
 
@@ -271,7 +292,7 @@ moviecliploader_loadclip(const fn_call& fn)
 
 	as_value tgt_arg = fn.arg(1);
 	std::string tgt_str = tgt_arg.to_string();
-	character* target = fn.env().find_target(tgt_str);
+	DisplayObject* target = fn.env().find_target(tgt_str);
 	if ( ! target )
 	{
 		IF_VERBOSE_ASCODING_ERRORS(
@@ -296,14 +317,14 @@ moviecliploader_loadclip(const fn_call& fn)
 		str_url, (void*)sprite);
 #endif
 
-	ptr->loadClip(str_url, *sprite);
+	ptr->loadClip(str_url, sprite);
 
 	// We always want to return true unless something went wrong
 	return as_value(true);
 
 }
 
-static as_value
+as_value
 moviecliploader_unloadclip(const fn_call& fn)
 {
   const std::string filespec = fn.arg(0).to_string();
@@ -311,7 +332,7 @@ moviecliploader_unloadclip(const fn_call& fn)
   return as_value();
 }
 
-static as_value
+as_value
 moviecliploader_new(const fn_call& /* fn */)
 {
 
@@ -322,7 +343,7 @@ moviecliploader_new(const fn_call& /* fn */)
 
 // Invoked every time the loading content is written to disk during
 // the loading process.
-static as_value
+as_value
 moviecliploader_getprogress(const fn_call& fn)
 {
 
@@ -337,7 +358,7 @@ moviecliploader_getprogress(const fn_call& fn)
 		return as_value();
 	}
 
-	boost::intrusive_ptr<as_object> target = fn.arg(0).to_object();
+	boost::intrusive_ptr<as_object> target = fn.arg(0).to_object(*getGlobal(fn));
   
 	if ( ! target.get() )
 	{
@@ -364,7 +385,7 @@ moviecliploader_getprogress(const fn_call& fn)
 	size_t bytesLoaded = sp->get_bytes_loaded();
 	size_t bytesTotal = sp->get_bytes_total();
 
-	string_table& st = ptr->getVM().getStringTable();
+	string_table& st = getStringTable(fn);
 
 	// We want these to be enumerable
 	mcl_obj->set_member(st.find("bytesLoaded"), bytesLoaded);
@@ -373,18 +394,5 @@ moviecliploader_getprogress(const fn_call& fn)
 	return as_value(mcl_obj.get()); // will keep alive
 }
 
-void
-moviecliploader_class_init(as_object& global)
-{
-	// This is going to be the global Number "class"/"function"
-	static boost::intrusive_ptr<builtin_function> cl=NULL;
-
-	if ( cl == NULL )
-	{
-		cl=new builtin_function(&moviecliploader_new,
-                getMovieClipLoaderInterface());
-	}
-	global.init_member("MovieClipLoader", cl.get()); 
-}
-
+} // anonymous namespace
 } // end of gnash namespace
