@@ -22,9 +22,10 @@
 namespace gnash {
 
 class as_object;
+class ObjectURI;
 
 /// Initialize the global ConvolutionFilter class
-void ConvolutionFilter_class_init(as_object& global);
+void convolutionfilter_class_init(as_object& where, const ObjectURI& uri);
 
 } // end of gnash namespace
 
