@@ -21,12 +21,13 @@
 #include "as_object.h" // for inheritance
 #include "log.h"
 #include "fn_call.h"
+#include "Global_as.h"
 #include "smart_ptr.h" // for boost intrusive_ptr
 #include "builtin_function.h" // need builtin_function
 #include "GnashException.h" // for ActionException
 #include "Object.h" // for AS inheritance
-#include "VM.h" // for addStatics
-#include "Point_as.h" // Matrix needs to operate on Points.
+#include "VM.h" 
+#include "Point_as.h"
 
 #include <cmath>
 #include <boost/numeric/ublas/matrix.hpp> // boost matrix
@@ -91,18 +92,22 @@ attachMatrixInterface(as_object& o)
 {
     int fl = 0;
 
-    o.init_member("clone", new builtin_function(Matrix_clone), fl);
-    o.init_member("concat", new builtin_function(Matrix_concat), fl);
-    o.init_member("createBox", new builtin_function(Matrix_createBox), fl);
-    o.init_member("createGradientBox", new builtin_function(Matrix_createGradientBox), fl);
-    o.init_member("deltaTransformPoint", new builtin_function(Matrix_deltaTransformPoint), fl);
-    o.init_member("identity", new builtin_function(Matrix_identity), fl);
-    o.init_member("invert", new builtin_function(Matrix_invert), fl);
-    o.init_member("rotate", new builtin_function(Matrix_rotate), fl);
-    o.init_member("scale", new builtin_function(Matrix_scale), fl);
-    o.init_member("toString", new builtin_function(Matrix_toString), fl);
-    o.init_member("transformPoint", new builtin_function(Matrix_transformPoint), fl);
-    o.init_member("translate", new builtin_function(Matrix_translate), fl);
+    Global_as* gl = getGlobal(o);
+    o.init_member("clone", gl->createFunction(Matrix_clone), fl);
+    o.init_member("concat", gl->createFunction(Matrix_concat), fl);
+    o.init_member("createBox", gl->createFunction(Matrix_createBox), fl);
+    o.init_member("createGradientBox",
+            gl->createFunction(Matrix_createGradientBox), fl);
+    o.init_member("deltaTransformPoint",
+            gl->createFunction(Matrix_deltaTransformPoint), fl);
+    o.init_member("identity", gl->createFunction(Matrix_identity), fl);
+    o.init_member("invert", gl->createFunction(Matrix_invert), fl);
+    o.init_member("rotate", gl->createFunction(Matrix_rotate), fl);
+    o.init_member("scale", gl->createFunction(Matrix_scale), fl);
+    o.init_member("toString", gl->createFunction(Matrix_toString), fl);
+    o.init_member("transformPoint",
+            gl->createFunction(Matrix_transformPoint), fl);
+    o.init_member("translate", gl->createFunction(Matrix_translate), fl);
 }
 
 
@@ -138,18 +143,6 @@ public:
 };
 
 
-as_function* getFlashGeomMatrixConstructor()
-{
-    static builtin_function* cl = NULL;
-    if ( ! cl )
-    {
-        cl=new builtin_function(&Matrix_ctor, getMatrixInterface());
-        VM::get().addStatic(cl);
-    }
-    return cl;
-}
-
-
 /// Return an exact copy of the matrix.
 static as_value
 Matrix_clone(const fn_call& fn)
@@ -209,7 +202,7 @@ Matrix_concat(const fn_call& fn)
     }
 
     // The object to concatenate doesn't have to be a matrix.    
-    as_object* obj = arg.to_object().get();
+    as_object* obj = arg.to_object(*getGlobal(fn)).get();
     assert(obj);
 
     MatrixType concatMatrix;
@@ -389,7 +382,8 @@ Matrix_deltaTransformPoint(const fn_call& fn)
         IF_VERBOSE_ASCODING_ERRORS(
             std::ostringstream ss;
             fn.dump_args(ss);
-            log_aserror("Matrix.deltaTransformPoint(%s): needs one argument", ss.str());
+            log_aserror("Matrix.deltaTransformPoint(%s): needs one argument",
+                ss.str());
         );
         return as_value();
     }
@@ -402,24 +396,35 @@ Matrix_deltaTransformPoint(const fn_call& fn)
         IF_VERBOSE_ASCODING_ERRORS(
             std::ostringstream ss;
             fn.dump_args(ss);
-            log_aserror("Matrix.deltaTransformPoint(%s): needs an object", ss.str());
+            log_aserror("Matrix.deltaTransformPoint(%s): needs an object",
+                ss.str());
         );
         return as_value();
     }
 
     // It doesn't have to be a point. If it has x and y
     // properties, they will be used.    
-    as_object* obj = arg.to_object().get();
+    as_object* obj = arg.to_object(*getGlobal(fn)).get();
     assert(obj);
 
     const PointType& point = transformPoint(obj, ptr.get());
 
     // Construct a Point and set its properties.
-    boost::intrusive_ptr<as_object> ret = init_Point_instance();
-    ret->set_member(NSV::PROP_X, point(0));
-    ret->set_member(NSV::PROP_Y, point(1));
+    as_value pointClass(fn.env().find_object("flash.geom.Point"));
 
-    return as_value(ret.get());
+    boost::intrusive_ptr<as_function> pointCtor = pointClass.to_as_function();
+
+    if (!pointCtor) {
+        log_error("Failed to construct flash.geom.Point!");
+        return as_value();
+    }
+
+    fn_call::Args args;
+    args += point(0), point(1);
+
+    as_value ret = pointCtor->constructInstance(fn.env(), args);
+
+    return as_value(ret);
 }
 
 
@@ -658,12 +663,14 @@ Matrix_toString(const fn_call& fn)
     
     std::ostringstream ss;
     
-    ss << "(a=" << a.to_string() << ", "
-          "b="<< b.to_string() << ", "
-          "c="<< c.to_string() << ", "
-          "d="<< d.to_string() << ", "
-          "tx="<< tx.to_string() << ", "
-          "ty="<< ty.to_string() << ")";
+    const int version = getSWFVersion(fn);
+
+    ss << "(a=" << a.to_string_versioned(version) << ", "
+          "b="<< b.to_string_versioned(version) << ", "
+          "c="<< c.to_string_versioned(version) << ", "
+          "d="<< d.to_string_versioned(version) << ", "
+          "tx="<< tx.to_string_versioned(version) << ", "
+          "ty="<< ty.to_string_versioned(version) << ")";
     
     return as_value(ss.str());
 }
@@ -697,15 +704,15 @@ Matrix_transformPoint(const fn_call& fn)
         return as_value();
     }
     
-    as_object* obj = arg.to_object().get();
+    as_object* obj = arg.to_object(*getGlobal(fn)).get();
     assert(obj);
-    if ( ! obj->instanceOf(getFlashGeomPointConstructor()) )
-    {
+    if (!obj->instanceOf(getFlashGeomPointConstructor(fn))) {
         /// Isn't a point.
         IF_VERBOSE_ASCODING_ERRORS(
             std::ostringstream ss;
             fn.dump_args(ss);
-            log_aserror("Matrix.transformPoint(%s): object must be a Point", ss.str());
+            log_aserror("Matrix.transformPoint(%s): object must be a Point",
+                ss.str());
         );
         return as_value();
     }
@@ -716,11 +723,22 @@ Matrix_transformPoint(const fn_call& fn)
     
     const PointType& point = transformPoint(obj, ptr.get());
 
-    boost::intrusive_ptr<as_object> ret = init_Point_instance();
-    ret->set_member(NSV::PROP_X, point(0) + tx.to_number());
-    ret->set_member(NSV::PROP_Y, point(1) + ty.to_number());
+    // Construct a Point and set its properties.
+    as_value pointClass(fn.env().find_object("flash.geom.Point"));
 
-    return as_value(ret.get());
+    boost::intrusive_ptr<as_function> pointCtor = pointClass.to_as_function();
+
+    if (!pointCtor) {
+        log_error("Failed to construct flash.geom.Point!");
+        return as_value();
+    }
+
+    fn_call::Args args;
+    args += point(0) + tx.to_number(), point(1) + ty.to_number();
+
+    as_value ret = pointCtor->constructInstance(fn.env(), args);
+
+    return as_value(ret);
 }
 
 static as_value
@@ -896,20 +914,21 @@ Matrix_ctor(const fn_call& fn)
 
 
 static as_value
-get_flash_geom_matrix_constructor(const fn_call& /*fn*/)
+get_flash_geom_matrix_constructor(const fn_call& fn)
 {
     log_debug("Loading flash.geom.Matrix class");
-
-    return getFlashGeomMatrixConstructor();
+    as_object* proto = getMatrixInterface();
+    Global_as* gl = getGlobal(fn);
+    return gl->createClass(&Matrix_ctor, proto);
 }
 
 // extern 
-void Matrix_class_init(as_object& where)
+void matrix_class_init(as_object& where, const ObjectURI& uri)
 {
-    // This is going to be the Matrix "class"/"function"
-    // in the 'where' package
-    string_table& st = where.getVM().getStringTable();
-    where.init_destructive_property(st.find("Matrix"), get_flash_geom_matrix_constructor);
+    // TODO: this may not be correct, but it should be enumerable.
+    const int flags = 0;
+    where.init_destructive_property(getName(uri),
+            get_flash_geom_matrix_constructor, flags, getNamespace(uri));
 }
 
 } // end of gnash namespace
