@@ -30,15 +30,14 @@ namespace gnash {
 
 class as_function;
 class as_object;
+class ObjectURI;
+class fn_call;
 
 /// Initialize the global Point class
-void Point_class_init(as_object& global);
-
-/// Return a Point instance (in case the core lib needs it)
-boost::intrusive_ptr<as_object> init_Point_instance();
+void point_class_init(as_object& where, const ObjectURI& uri);
 
 /// Return the Point constructor, for use by Rectangle 
-as_function* getFlashGeomPointConstructor();
+as_function* getFlashGeomPointConstructor(const fn_call& fn);
 
 } // end of gnash namespace
 
