@@ -31,7 +31,7 @@
 #include "log.h"
 #include "as_value.h"
 #include "as_environment.h"
-#include "swf.h"
+#include "SWF.h"
 #include "ASHandlers.h"
 #include "movie_root.h"
 
@@ -325,7 +325,7 @@ void
 Debugger::disassemble(const unsigned char *data)
 {
 //    GNASH_REPORT_FUNCTION;
-    as_arg_t fmt = ARG_HEX;
+    ArgumentType fmt = ARG_HEX;
     ActionType action_id = static_cast<ActionType>(data[0]);
     int val = 0;
     string str;
@@ -571,7 +571,7 @@ Debugger::dumpStackFrame(as_environment &env)
 // FIXME: we want to print the name of the function
 //  	    if (val.is_as_function()) {
 // //		cerr << val.get_symbol_handle() << endl;
-// 		string name = this->lookupSymbol(val.to_object());
+// 		string name = this->lookupSymbol(val.to_object(*getGlobal(fn)));
 // 		if (name.size()) {
 // 		    cerr << name << " ";
 // 		}
@@ -579,7 +579,7 @@ Debugger::dumpStackFrame(as_environment &env)
         cerr << env.bottom(i);
 
 	    if (val.is_object()) {
-		boost::intrusive_ptr<as_object> o = val.to_object();
+		boost::intrusive_ptr<as_object> o = val.to_object(*getGlobal(fn));
 		string name = lookupSymbol(o.get());
 		if (name.size()) {
 		    cerr << " \"" << name << "\"";
