@@ -1,6 +1,6 @@
 // fill_style.cpp:  Graphical region filling styles, for Gnash.
 // 
-//   Copyright (C) 2007, 2008 Free Software Foundation, Inc.
+//   Copyright (C) 2007, 2008, 2009 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -21,14 +21,17 @@
 
 #include "smart_ptr.h" // GNASH_USE_GC
 #include "fill_style.h"
-#include "impl.h"
 #include "log.h"
-#include "render.h"
 #include "SWFStream.h"
 #include "movie_definition.h"
-#include "swf.h"
+#include "SWF.h"
 #include "GnashException.h"
+#include "GnashNumeric.h"
+#include "Renderer.h"
+#include "RunResources.h"
+
 #include <cmath> // sqrt, floor
+#include <iostream> // for output operator
 
 namespace gnash {
 
@@ -37,11 +40,11 @@ namespace gnash {
 //
 
 void
-gradient_record::read(SWFStream& in, int tag_type)
+gradient_record::read(SWFStream& in, SWF::TagType tag)
 {
     in.ensureBytes(1);
     m_ratio = in.read_u8();
-    m_color.read(in, tag_type);
+    m_color.read(in, tag);
 }
 
 //
@@ -49,21 +52,21 @@ gradient_record::read(SWFStream& in, int tag_type)
 //
 fill_style::fill_style()
     :
-    m_type(SWF::FILL_SOLID),
+    _bitmapInfo(0),
     m_color(), // FF.FF.FF.FF
-    m_gradient_bitmap_info(0),
-    m_bitmap_character(0),
     m_spread_mode(SWF::GRADIENT_SPREAD_PAD),
-    m_interpolation(SWF::GRADIENT_INTERPOL_NORMAL)
+    m_interpolation(SWF::GRADIENT_INTERPOL_NORMAL),
+    m_type(SWF::FILL_SOLID),
+    _bitmapSmoothingPolicy(BITMAP_SMOOTHING_UNSPECIFIED)
 {
 }
 
 
 void
-fill_style::read(SWFStream& in, int tag_type, movie_definition& md,
-    fill_style *pOther)
+fill_style::read(SWFStream& in, SWF::TagType t, movie_definition& md,
+        const RunResources& r, fill_style *pOther)
 {
-    const bool is_morph = pOther != NULL;
+    const bool is_morph = (pOther != NULL);
 
     in.ensureBytes(1);
     m_type = in.read_u8();
@@ -73,14 +76,14 @@ fill_style::read(SWFStream& in, int tag_type, movie_definition& md,
     }
         
     IF_VERBOSE_PARSE(
-        log_parse("  fill_style read type = 0x%X", m_type);
+        log_parse("  fill_style read type = 0x%X", (int)m_type);
     );
 
     if (m_type == SWF::FILL_SOLID)
     {
         // 0x00: solid fill
-        if ( tag_type == SWF::DEFINESHAPE3 || tag_type == SWF::DEFINESHAPE4
-            || tag_type == SWF::DEFINESHAPE4_ || is_morph)
+        if ( t == SWF::DEFINESHAPE3 || t == SWF::DEFINESHAPE4
+            || t == SWF::DEFINESHAPE4_ || is_morph)
         {
             m_color.read_rgba(in);
             if (is_morph)   pOther->m_color.read_rgba(in);
@@ -88,7 +91,8 @@ fill_style::read(SWFStream& in, int tag_type, movie_definition& md,
         else 
         {
             // For DefineMorphShape tags we should use morph_fill_style 
-            assert( tag_type == SWF::DEFINESHAPE || tag_type == SWF::DEFINESHAPE2 );
+            assert( t == SWF::DEFINESHAPE ||
+                    t == SWF::DEFINESHAPE2 );
             m_color.read_rgb(in);
         }
 
@@ -105,37 +109,37 @@ fill_style::read(SWFStream& in, int tag_type, movie_definition& md,
         // 0x12: radial gradient fill
         // 0x13: focal gradient fill
 
-        matrix  input_matrix;
+        SWFMatrix  input_matrix;
         input_matrix.read(in);
 
         // shouldn't this be in initializer's list ?
-        m_gradient_matrix.set_identity();
+        _matrix.set_identity();
         if (m_type == SWF::FILL_LINEAR_GRADIENT)
         {
-            m_gradient_matrix.set_translation(128, 0);
-            m_gradient_matrix.set_scale(1.0/128, 1.0/128);
+            _matrix.set_translation(128, 0);
+            _matrix.set_scale(1.0/128, 1.0/128);
         }
         else // FILL_RADIAL_GRADIENT or FILL_FOCAL_GRADIENT
         {
-            m_gradient_matrix.set_translation(32, 32);
-            m_gradient_matrix.set_scale(1.0/512, 1.0/512);
+            _matrix.set_translation(32, 32);
+            _matrix.set_scale(1.0/512, 1.0/512);
         }
 
-        matrix m = input_matrix;
+        SWFMatrix m = input_matrix;
         m.invert();
 
         if (is_morph)
         {
-            pOther->m_gradient_matrix = m_gradient_matrix;
+            pOther->_matrix = _matrix;
         }
-        m_gradient_matrix.concatenate(m);
+        _matrix.concatenate(m);
         
         if (is_morph)
         {
             input_matrix.read(in);
             m = input_matrix;
             m.invert();
-            pOther->m_gradient_matrix.concatenate(m);
+            pOther->_matrix.concatenate(m);
         }
         
         // GRADIENT
@@ -143,8 +147,8 @@ fill_style::read(SWFStream& in, int tag_type, movie_definition& md,
 
         uint8_t grad_props = in.read_u8();
     
-        if (tag_type == SWF::DEFINESHAPE4 ||
-            tag_type == SWF::DEFINESHAPE4_) {
+        if (t == SWF::DEFINESHAPE4 ||
+            t == SWF::DEFINESHAPE4_) {
             uint8_t spread_mode = grad_props >> 6;
             switch(spread_mode) {
                 case 0:
@@ -171,9 +175,10 @@ fill_style::read(SWFStream& in, int tag_type, movie_definition& md,
                     m_interpolation = SWF::GRADIENT_INTERPOL_LINEAR;
                     break;
                 default:
-                IF_VERBOSE_MALFORMED_SWF(
-                    log_swferror("Illegal interpolation mode in gradient definition.");
-                );
+                    IF_VERBOSE_MALFORMED_SWF(
+                        log_swferror("Illegal interpolation mode in gradient "
+                            "definition.");
+                    );
             }
         }
     
@@ -186,13 +191,13 @@ fill_style::read(SWFStream& in, int tag_type, movie_definition& md,
             return;
         }
     
-        if ( num_gradients > 8 + ((tag_type == SWF::DEFINESHAPE4 ||
-            tag_type == SWF::DEFINESHAPE4_) ? 7 : 0))
+        if ( num_gradients > 8 + ((t == SWF::DEFINESHAPE4 ||
+            t == SWF::DEFINESHAPE4_) ? 7 : 0))
         {
            // see: http://sswf.sourceforge.net/SWFalexref.html#swf_gradient
             IF_VERBOSE_MALFORMED_SWF(
-                log_swferror(_("Unexpected num gradients (%d), expected 1 to 8"),
-                        num_gradients);
+                log_swferror(_("Unexpected num gradients (%d), "
+                        "expected 1 to 8"), static_cast<int>(num_gradients));
             );
         }
     
@@ -200,12 +205,11 @@ fill_style::read(SWFStream& in, int tag_type, movie_definition& md,
             pOther->m_gradients.resize(num_gradients);
         }
                 
-    
         m_gradients.resize(num_gradients);
         for (unsigned int i = 0; i < num_gradients; i++) {
-            m_gradients[i].read(in, tag_type);
+            m_gradients[i].read(in, t);
             if (is_morph) {
-                pOther->m_gradients[i].read(in, tag_type);
+                pOther->m_gradients[i].read(in, t);
             }
         }
     
@@ -214,10 +218,8 @@ fill_style::read(SWFStream& in, int tag_type, movie_definition& md,
         {
            in.ensureBytes(2);
            m_focal_point = in.read_short_sfixed();
-           if (m_focal_point < -1.0f)
-               m_focal_point = -1.0f;
-           else if (m_focal_point > 1.0f)
-               m_focal_point = 1.0f;
+           if (m_focal_point < -1.0f) m_focal_point = -1.0f;
+           else if (m_focal_point > 1.0f) m_focal_point = 1.0f;
         }
     
         if (is_morph) {
@@ -225,25 +227,25 @@ fill_style::read(SWFStream& in, int tag_type, movie_definition& md,
         }
     
         IF_VERBOSE_PARSE (
-           log_parse("  gradients: num_gradients = %d", num_gradients);
+           log_parse("  gradients: num_gradients = %d",
+               static_cast<int>(num_gradients));
         );
     
-        // @@ hack.
+        // @@ hack. What is it supposed to do?
         if (num_gradients > 0) {
             m_color = m_gradients[0].m_color;
             if (is_morph)
                pOther->m_color = pOther->m_gradients[0].m_color;
         }
     
-        m_gradient_bitmap_info = create_gradient_bitmap();
-        if (is_morph)
-        {
-            pOther->m_gradient_bitmap_info = 
-                pOther->need_gradient_bitmap();
-            md.add_bitmap_info(pOther->m_gradient_bitmap_info.get());
+        Renderer* renderer = r.renderer();
+        if (renderer) {
+
+            _bitmapInfo = create_gradient_bitmap(*renderer);
+            if (is_morph) {
+                pOther->_bitmapInfo = pOther->need_gradient_bitmap(*renderer);
+            }
         }
-            // Make sure our SWFMovieDefinition knows about this bitmap.
-        md.add_bitmap_info(m_gradient_bitmap_info.get());
     }
     else if (m_type == SWF::FILL_TILED_BITMAP
           || m_type == SWF::FILL_CLIPPED_BITMAP
@@ -255,48 +257,60 @@ fill_style::read(SWFStream& in, int tag_type, movie_definition& md,
         // 0x42: tiled bitmap fill with hard edges
         // 0x43: clipped bitmap fill with hard edges
 
+        if ( m_type == SWF::FILL_TILED_BITMAP_HARD ||
+             m_type == SWF::FILL_CLIPPED_BITMAP_HARD )
+        {
+            _bitmapSmoothingPolicy = BITMAP_SMOOTHING_OFF;
+        }
+        else if ( md.get_version() >= 8 )
+        {
+            _bitmapSmoothingPolicy = BITMAP_SMOOTHING_ON;
+        }
+        else
+        {
+            _bitmapSmoothingPolicy = BITMAP_SMOOTHING_UNSPECIFIED;
+        }
+
         in.ensureBytes(2);
         int bitmap_char_id = in.read_u16();
         IF_VERBOSE_PARSE
         (
-            log_parse("  bitmap_char = %d", bitmap_char_id);
+            log_parse("  bitmap_char = %d, smoothing_policy = %s",
+                bitmap_char_id, _bitmapSmoothingPolicy);
         );
 
-        // Look up the bitmap character.
-        m_bitmap_character = md.get_bitmap_character_def(bitmap_char_id);
+        // Look up the bitmap DisplayObject.
+        _bitmapInfo = md.getBitmap(bitmap_char_id);
         IF_VERBOSE_MALFORMED_SWF(
-            if ( m_bitmap_character == NULL )
+            if (!_bitmapInfo)
             {
-                static bool warned_about_invalid_char=false;
-                if ( ! warned_about_invalid_char )
-                {
+                LOG_ONCE(
                     log_swferror(_("Bitmap fill specifies '%d' as associated"
-                        " bitmap character id,"
-                        " but that character is not found"
+                        " bitmap DisplayObject id,"
+                        " but that DisplayObject is not found"
                         " in the Characters Dictionary."
                         " It seems common to find such "
                         " malformed SWF, so we'll only warn once "
                         "about this."), bitmap_char_id);
-                    warned_about_invalid_char=true;
-                }
+                );
             }
         );
 
-        matrix  m;
+        SWFMatrix  m;
         m.read(in);
 
         // For some reason, it looks like they store the inverse of the
-        // TWIPS-to-texcoords matrix.
-        m_bitmap_matrix = m.invert();
+        // TWIPS-to-texcoords SWFMatrix.
+        _matrix = m.invert();
 
         if (is_morph)
         {
-            pOther->m_bitmap_character = m_bitmap_character;
+            pOther->_bitmapInfo = _bitmapInfo;
             m.read(in);
-            pOther->m_bitmap_matrix = m.invert();
+            pOther->_matrix = m.invert();
         }
         IF_VERBOSE_PARSE(
-           log_parse("matrix: %s", m_bitmap_matrix);
+           log_parse("SWFMatrix: %s", _matrix);
         );
     }
     else
@@ -310,45 +324,45 @@ fill_style::read(SWFStream& in, int tag_type, movie_definition& md,
 }
 
 
-bitmap_info* 
-fill_style::get_bitmap_info() const 
+const BitmapInfo* 
+fill_style::get_bitmap_info(Renderer& renderer) const 
 {    
-  assert(m_type != SWF::FILL_SOLID);
-  
-  if (m_type == SWF::FILL_TILED_BITMAP
-   || m_type == SWF::FILL_CLIPPED_BITMAP
-   || m_type == SWF::FILL_TILED_BITMAP_HARD
-   || m_type == SWF::FILL_CLIPPED_BITMAP_HARD) {
+    assert(m_type != SWF::FILL_SOLID);
 
-   if (m_bitmap_character!=NULL)
-     return m_bitmap_character->get_bitmap_info();
-   else
-     return NULL;
+    switch (m_type)
+    {
+        case SWF::FILL_TILED_BITMAP:
+        case SWF::FILL_CLIPPED_BITMAP:
+        case SWF::FILL_TILED_BITMAP_HARD:
+        case SWF::FILL_CLIPPED_BITMAP_HARD:
+            if (_bitmapInfo)
+            {
+                return _bitmapInfo.get();
+            }
+            return NULL;
    
-  } else
-  if (m_type == SWF::FILL_LINEAR_GRADIENT
-   || m_type == SWF::FILL_RADIAL_GRADIENT) {
-   
-   return need_gradient_bitmap();
-   
-  } else {
-    log_error(_("Unknown fill style %d"), m_type);
-    abort();
-  }  
+        case SWF::FILL_LINEAR_GRADIENT:
+        case SWF::FILL_RADIAL_GRADIENT:
+            return need_gradient_bitmap(renderer);
+        default:
+            log_error(_("Unknown fill style %d"), m_type);
+            // Seems a bit drastic...
+            std::abort();
+    }
 }
 
-matrix
-fill_style::get_bitmap_matrix() const 
+const SWFMatrix&
+fill_style::getBitmapMatrix() const 
 {
   assert(m_type != SWF::FILL_SOLID);
-  return m_bitmap_matrix;
+  return _matrix;
 }
 
-matrix
-fill_style::get_gradient_matrix() const 
+const SWFMatrix&
+fill_style::getGradientMatrix() const 
 {
   // TODO: Why do we separate bitmap and gradient matrices? 
-  return m_gradient_matrix;
+  return _matrix;
 }
 
 rgba
@@ -376,7 +390,7 @@ fill_style::sample_gradient(boost::uint8_t ratio) const
                     "have position==%d (expected 0)."
                     " This seems to be common, so will"
                     " warn only once."),
-                    (int)m_gradients[0].m_ratio)
+                    static_cast<int>(m_gradients[0].m_ratio));
             );
         );
         return m_gradients[0].m_color;
@@ -413,7 +427,7 @@ fill_style::sample_gradient(boost::uint8_t ratio) const
             );
         }
 
-        rgba    result;
+        rgba result;
         result.set_lerp(gr0.m_color, gr1.m_color, f);
         return result;
     }
@@ -422,93 +436,97 @@ fill_style::sample_gradient(boost::uint8_t ratio) const
     return m_gradients.back().m_color;
 }
 
-gnash::bitmap_info*
-fill_style::create_gradient_bitmap() const
+const BitmapInfo*
+fill_style::create_gradient_bitmap(Renderer& renderer) const
 {
     assert(m_type == SWF::FILL_LINEAR_GRADIENT
         || m_type == SWF::FILL_RADIAL_GRADIENT
         || m_type == SWF::FILL_FOCAL_GRADIENT);
 
-    std::auto_ptr<image::ImageRGBA> im(NULL);
+    std::auto_ptr<ImageRGBA> im;
 
-    if (m_type == SWF::FILL_LINEAR_GRADIENT)
+    switch (m_type)
     {
-        // Linear gradient.
-        im.reset(new image::ImageRGBA(256, 1));
+        case SWF::FILL_LINEAR_GRADIENT:
+            // Linear gradient.
+            im.reset(new ImageRGBA(256, 1));
 
-        for (size_t i = 0; i < im->width(); i++)
-    {
-            rgba    sample = sample_gradient(i);
-            im->setPixel(i, 0, sample.m_r, sample.m_g, sample.m_b, sample.m_a);
-        }
-    }
-    else if (m_type == SWF::FILL_RADIAL_GRADIENT)
-    {
-        // Radial gradient.
-        im.reset(new image::ImageRGBA(64, 64));
-
-        for (size_t j = 0; j < im->height(); j++) {
-            for (size_t i = 0; i < im->width(); i++) {
-                float   radius = (im->height() - 1) / 2.0f;
-                float   y = (j - radius) / radius;
-                float   x = (i - radius) / radius;
-                int ratio = static_cast<int>(std::floor(255.5f * std::sqrt(x * x + y * y)));
-                if (ratio > 255) {
-                    ratio = 255;
-                }
-                rgba    sample = sample_gradient( ratio );
-                im->setPixel(i, j, sample.m_r, sample.m_g, sample.m_b, sample.m_a);
-            }
-        }
-    }
-    else if (m_type == SWF::FILL_FOCAL_GRADIENT)
-    {
-        // Focal gradient.
-        im.reset(new image::ImageRGBA(64, 64));
-
-        for (size_t j = 0; j < im->height(); j++)
-        {
             for (size_t i = 0; i < im->width(); i++)
             {
-                float radiusy = (im->height() - 1) / 2.0f;
-                float radiusx = radiusy + std::abs(radiusy * m_focal_point);
-                float y = (j - radiusy) / radiusy;
-                float x = (i - radiusx) / radiusx;
-                int ratio = static_cast<int>(std::floor(255.5f * std::sqrt(x*x + y*y)));
-                if (ratio > 255)
-                {
-                    ratio = 255;
-                }
-                rgba sample = sample_gradient(ratio);
-                im->setPixel(i, j, sample.m_r, sample.m_g, sample.m_b, sample.m_a);
+                rgba sample = sample_gradient(i);
+                im->setPixel(i, 0, sample.m_r, sample.m_g,
+                        sample.m_b, sample.m_a);
             }
-        }
+            break;
+        case SWF::FILL_RADIAL_GRADIENT:
+            // Radial gradient.
+            im.reset(new ImageRGBA(64, 64));
+
+            for (size_t j = 0; j < im->height(); j++) {
+                for (size_t i = 0; i < im->width(); i++) {
+                    float   radius = (im->height() - 1) / 2.0f;
+                    float   y = (j - radius) / radius;
+                    float   x = (i - radius) / radius;
+                    int ratio = static_cast<int>(
+                            std::floor(255.5f * std::sqrt(x * x + y * y)));
+                    if (ratio > 255) {
+                        ratio = 255;
+                    }
+                    rgba sample = sample_gradient(ratio);
+                    im->setPixel(i, j, sample.m_r, sample.m_g,
+                            sample.m_b, sample.m_a);
+                }
+            }
+            break;
+        case SWF::FILL_FOCAL_GRADIENT:
+            // Focal gradient.
+            im.reset(new ImageRGBA(64, 64));
+
+            for (size_t j = 0; j < im->height(); j++)
+            {
+                for (size_t i = 0; i < im->width(); i++)
+                {
+                    float radiusy = (im->height() - 1) / 2.0f;
+                    float radiusx = radiusy + std::abs(radiusy * m_focal_point);
+                    float y = (j - radiusy) / radiusy;
+                    float x = (i - radiusx) / radiusx;
+                    int ratio = static_cast<int>(std::floor(255.5f *
+                                std::sqrt(x*x + y*y)));
+                    
+                    if (ratio > 255) ratio = 255;
+
+                    rgba sample = sample_gradient(ratio);
+                    im->setPixel(i, j, sample.m_r, sample.m_g,
+                            sample.m_b, sample.m_a);
+                }
+            }
+            break;
     }
-        
-    bitmap_info* bi = render::createBitmapInfo(
-                    static_cast<std::auto_ptr<image::ImageBase> >(im));
+
+    const BitmapInfo* bi = renderer.createBitmapInfo(
+                    static_cast<std::auto_ptr<GnashImage> >(im));
 
     return bi;
 }
 
 
-gnash::bitmap_info*
-fill_style::need_gradient_bitmap() const 
+const BitmapInfo*
+fill_style::need_gradient_bitmap(Renderer& renderer) const 
 {
 
-  if (m_gradient_bitmap_info==NULL) {
+  if (!_bitmapInfo) {
     fill_style* this_non_const = const_cast<fill_style*>(this);
-    this_non_const->m_gradient_bitmap_info = create_gradient_bitmap();
+    this_non_const->_bitmapInfo = create_gradient_bitmap(renderer);
   }
   
-  return m_gradient_bitmap_info.get();
+  return _bitmapInfo.get();
 
 }
 
 
+// Sets this style to a blend of a and b.  t = [0,1]
 void
 fill_style::set_lerp(const fill_style& a, const fill_style& b, float t)
-    // Sets this style to a blend of a and b.  t = [0,1]
 {
     assert(t >= 0 && t <= 1);
 
@@ -516,35 +534,54 @@ fill_style::set_lerp(const fill_style& a, const fill_style& b, float t)
     m_type = a.get_type();
     assert(m_type == b.get_type());
 
-    // fill style color
+    // fill style color (TODO: only for solid fills ?)
     m_color.set_lerp(a.get_color(), b.get_color(), t);
 
-    // fill style gradient matrix
-    //
-    // @@ TODO morphed gradients don't come out exactly
-    // right; they shift around some.  Not sure where the
-    // problem is.
-    m_gradient_matrix.set_lerp(a.m_gradient_matrix, b.m_gradient_matrix, t);
+    bool usesMatrix = false;
 
-    // fill style gradients
-    assert(m_gradients.size() == a.m_gradients.size());
-    assert(m_gradients.size() == b.m_gradients.size());
-    for (size_t j=0, nj=m_gradients.size(); j<nj; ++j)
+    switch (m_type)
     {
-        m_gradients[j].m_ratio =
-            (boost::uint8_t) utility::frnd(
-                utility::flerp(a.m_gradients[j].m_ratio, b.m_gradients[j].m_ratio, t)
-                );
-        m_gradients[j].m_color.set_lerp(a.m_gradients[j].m_color, b.m_gradients[j].m_color, t);
+        case SWF::FILL_LINEAR_GRADIENT:
+        case SWF::FILL_RADIAL_GRADIENT:
+        case SWF::FILL_FOCAL_GRADIENT:
+        {
+            usesMatrix = true;
+
+            // fill style gradients
+            assert(m_gradients.size() == a.m_gradients.size());
+            assert(m_gradients.size() == b.m_gradients.size());
+            for (size_t j=0, nj=m_gradients.size(); j<nj; ++j)
+            {
+                m_gradients[j].m_ratio =
+                    (boost::uint8_t) frnd( flerp(a.m_gradients[j].m_ratio,
+                            b.m_gradients[j].m_ratio, t)
+                        );
+                m_gradients[j].m_color.set_lerp(a.m_gradients[j].m_color,
+                        b.m_gradients[j].m_color, t);
+            }
+            _bitmapInfo = NULL;
+            break;
+        }
+
+        case SWF::FILL_TILED_BITMAP:
+        case SWF::FILL_CLIPPED_BITMAP:
+        case SWF::FILL_TILED_BITMAP_HARD:
+        case SWF::FILL_CLIPPED_BITMAP_HARD:
+        {
+            usesMatrix = true;
+
+            // fill style bitmap ID
+            _bitmapInfo = a._bitmapInfo;
+            assert(_bitmapInfo == b._bitmapInfo);
+            break;
+        }
+
+        default:
+            break;
     }
-    m_gradient_bitmap_info = NULL;
 
-    // fill style bitmap ID
-    m_bitmap_character = a.m_bitmap_character;
-    assert(m_bitmap_character == b.m_bitmap_character);
-
-    // fill style bitmap matrix
-    m_bitmap_matrix.set_lerp(a.m_bitmap_matrix, b.m_bitmap_matrix, t);
+    // fill style bitmap or gradient SWFMatrix
+    if ( usesMatrix ) _matrix.set_lerp(a._matrix, b._matrix, t);
 }
 
 
@@ -560,11 +597,12 @@ fill_style::get_color_stop(int index) const
   return m_gradients[index];
 }
 
-fill_style::fill_style(bitmap_character_def* bitmap, const matrix& mat)
+fill_style::fill_style(const BitmapInfo* const bitmap, const SWFMatrix& mat)
     :
+    _matrix(mat),
+    _bitmapInfo(bitmap),
     m_type(SWF::FILL_CLIPPED_BITMAP),
-    m_bitmap_character(bitmap),
-    m_bitmap_matrix(mat)
+    _bitmapSmoothingPolicy(BITMAP_SMOOTHING_UNSPECIFIED)
 {
 }
 
@@ -576,21 +614,45 @@ fill_style::setSolid(const rgba& color)
 }
 
 void
-fill_style::setLinearGradient(const std::vector<gradient_record>& gradients, const matrix& mat)
+fill_style::setLinearGradient(const std::vector<gradient_record>& gradients,
+        const SWFMatrix& mat)
 {
     m_type = SWF::FILL_LINEAR_GRADIENT;
     m_gradients = gradients;
-    m_gradient_matrix = mat;
-    m_gradient_bitmap_info = 0;
+    _matrix = mat;
+    _bitmapInfo = 0;
 }
 
 void
-fill_style::setRadialGradient(const std::vector<gradient_record>& gradients, const matrix& mat)
+fill_style::setRadialGradient(const std::vector<gradient_record>& gradients,
+        const SWFMatrix& mat)
 {
     m_type = SWF::FILL_RADIAL_GRADIENT;
     m_gradients = gradients;
-    m_gradient_matrix = mat;
-    m_gradient_bitmap_info = 0;
+    _matrix = mat;
+    _bitmapInfo = 0;
+}
+
+std::ostream& operator << (std::ostream& os,
+        const fill_style::BitmapSmoothingPolicy& p)
+{
+    switch (p)
+    {
+        case fill_style::BITMAP_SMOOTHING_UNSPECIFIED:
+            os << "unspecified";
+            break;
+        case fill_style::BITMAP_SMOOTHING_ON:
+            os << "on";
+            break;
+        case fill_style::BITMAP_SMOOTHING_OFF:
+            os << "off";
+            break;
+        default:
+            // cast to int required to avoid infinite recursion
+            os << "unknown " << (int)p;
+            break;
+    }
+    return os;
 }
 
 
@@ -598,8 +660,7 @@ fill_style::setRadialGradient(const std::vector<gradient_record>& gradients, con
 void
 fill_style::markReachableResources() const
 {
-    if ( m_gradient_bitmap_info ) m_gradient_bitmap_info->setReachable();
-    if ( m_bitmap_character ) m_bitmap_character->setReachable();
+    if ( _bitmapInfo ) _bitmapInfo->setReachable();
 }
 #endif // GNASH_USE_GC
 
