@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -71,13 +71,15 @@
 
 #include "smart_ptr.h" // GNASH_USE_GC
 #include "dsodefs.h" // DSOEXPORT
-#include "mouse_button_state.h" // for composition
+#include "MouseButtonState.h" // for composition
 #include "drag_state.h" // for composition
-#include "movie_instance.h" // for inlines
-#include "asobj/Key.h"
+#include "flash/ui/Keyboard_as.h"
 #include "smart_ptr.h" // for memory management
 #include "URL.h" // for loadMovie
 #include "GnashKey.h" // key::code
+#include "Movie.h"
+#include "RunResources.h" // for initialization
+#include "gnash.h" // Quality
 
 #ifdef USE_SWFTREE
 # include "tree.hh"
@@ -101,10 +103,10 @@
 // Forward declarations
 namespace gnash {
     class ExecutableCode; // for ActionQueue
-    class Stage;
     class URL;
     class Timer;
-    class Gui;
+    class MovieClip;
+    class VirtualClock;
 }
 
 namespace gnash
@@ -112,7 +114,7 @@ namespace gnash
 
 struct DepthComparator
 {
-    typedef boost::intrusive_ptr<sprite_instance> LevelMovie;
+    typedef boost::intrusive_ptr<MovieClip> LevelMovie;
 
     bool operator() (const LevelMovie& d1, const LevelMovie& d2)
     {
@@ -120,13 +122,22 @@ struct DepthComparator
     }
 };
 
-/// The movie stage (absolute top level node in the characters hierarchy)
+/// This class represents the 'Stage' and top-level movie.
 //
-/// This is a wrapper around the set of loaded levels being played.
-///
-/// There is a *single* instance of this class for each run;
-/// loading external movies will *not* create a new instance of it.
-///
+/// It is a wrapper around the set of loaded levels being played. Each 
+/// 'run' of a SWF movie, including all further movies loaded during the
+/// run, has exactly one movie_root, which is kept for the entire run.
+/// Loading a new top-level movie does not create a new movie_root.
+//
+/// The 'Stage' part of movie_root is accessible through the ActionScript
+/// Stage object, implemented in Stage_as.cpp.
+//
+/// The movie_root class is responsible for accepting and passing on
+/// user events (mouse or key events), for maintaining the heart-beat
+/// mechanism, and for advancing all MovieClips on request from the
+/// hosting application.
+//
+/// The _root object is provided by getAsRoot().
 class DSOEXPORT movie_root : boost::noncopyable
 {
 
@@ -137,15 +148,11 @@ public:
     /// Make sure to call setRootMovie() 
     /// before using any of this class methods !
     ///
-    movie_root(VM& vm);
+    movie_root(const movie_definition& def, VirtualClock& clock,
+            const RunResources& runResources);
 
     ~movie_root();
 
-    void setGui(Gui* g)
-    {
-        _gui=g;
-    }
-
     /// Set the root movie, replacing the current one if any.
     //
     /// This is needed for the cases in which the top-level movie
@@ -158,7 +165,7 @@ public:
     ///
     /// Make sure to call this method before using the movie_root,
     /// as most operations are delegated to the associated/wrapped
-    /// movie_instance.
+    /// Movie.
     ///
     /// Note that the display viewport will be updated to match
     /// the size of given movie.
@@ -166,18 +173,18 @@ public:
     /// A call to this method is equivalent to a call to setLevel(0, movie).
     ///
     /// @param movie
-    /// The movie_instance to wrap.
+    /// The Movie to wrap.
     /// Will be stored in an intrusive_ptr.
     /// Must have a depth of 0.
     ///
-    void setRootMovie(movie_instance* movie);
+    void setRootMovie(Movie* movie);
 
     /// Return the movie at the given level (0 if unloaded level).
     //
     /// POST CONDITIONS:
-    /// - The returned character has a depth equal to 'num'
+    /// - The returned DisplayObject has a depth equal to 'num'
     ///
-    boost::intrusive_ptr<movie_instance> getLevel(unsigned int num) const;
+    boost::intrusive_ptr<Movie> getLevel(unsigned int num) const;
 
     /// Load movie at the specified URL in the given level 
     //
@@ -200,42 +207,28 @@ public:
     /// Character's depths are updated.
     ///
     /// @param sp
-    ///		The level to change depth/level of. A pointer to it is expected
-    ///		to be found in the _level# container, or an error will be printed
-    ///		and the call would result in a no-op.
+    ///    The level to change depth/level of. A pointer to it is expected
+    ///    to be found in the _level# container, or an error will be printed
+    ///    and the call would result in a no-op.
     ///
     /// @param depth
-    ///		New depth to assign to the character. If another level exists at 
-    ///		the target depth the latter is moved in place of the former, with
-    ///		its depth also updated.
+    ///    New depth to assign to the DisplayObject. If another level
+    ///    exists at the target depth the latter is moved in place of
+    ///    the former, with its depth also updated.
     ///
-    void swapLevels(boost::intrusive_ptr<sprite_instance> sp, int depth);
+    void swapLevels(boost::intrusive_ptr<MovieClip> sp, int depth);
 
     /// Drop level at given depth.
     //
     /// @param depth
-    ///   Depth of the level to drop. Note that this is -character::staticDepthOffset for
-    ///   the root movie. Must be >=0 and <= 1048575 or an assertion will fail.
-    ///   Note that if the depth evaluates to the original root movie nothing happens
-    ///   (not allowed to remove that). It is not tested if it's allowed to remove 
-    ///   _level0 after loading into it.
-    ///
+    ///   Depth of the level to drop. Note that this is 
+    ///   -DisplayObject::staticDepthOffset for the root movie. Must be >=0 and
+    ///   <= 1048575 or an assertion will fail. Note that if the depth
+    ///   evaluates to the original root movie nothing happens (not allowed
+    ///   to remove that). It is not tested if it's allowed to remove _level0
+    ///   after loading into it.
     void dropLevel(int depth);
 
-    /// @@ should this delegate to _level0?  probably !
-    void set_member(
-        const std::string& /*name*/,
-        const as_value& /*val*/)
-    {
-    }
-
-    /// @@ should this delegate to _level0?  probably !
-    bool get_member(const std::string& /*name*/,
-            as_value* /*val*/)
-    {
-        return false;
-    }
-
     /// Change display viewport coordinates
     //
     /// This currently also change the display scale
@@ -285,10 +278,6 @@ public:
     ///
     bool notify_mouse_clicked(bool mouse_pressed, int mask);
 
-    /// The host app can use this to tell the movie where the
-    /// user's mouse pointer is.
-    //void notify_mouse_state(int x, int y, int buttons);
-
     /// \brief
     /// The host app can use this to tell the movie when
     /// user pressed or released a key.
@@ -305,16 +294,34 @@ public:
     //
     /// Coordinates are in PIXELS, NOT TWIPS.
     ///
-    void get_mouse_state(int& x, int& y, int& buttons);
+    void get_mouse_state(boost::int32_t& x, boost::int32_t& y,
+            boost::int32_t& buttons);
 
     void get_drag_state(drag_state& st);
 
     void set_drag_state(const drag_state& st);
 
     /// @return the originating root movie (not necessarely _level0)
-    movie_instance* getRootMovie() const
+    const Movie& getRootMovie() const
     {
-	return _rootMovie.get();
+        return *_rootMovie;
+    }
+
+    /// Creating new MovieClips needs this for now.
+    //
+    /// TODO: create MovieClips without this and drop. It's deliberately
+    /// different from getRootMovie() so it doesn't end up getting used
+    /// in the same way.
+    Movie* topLevelMovie() const
+    {
+        return _rootMovie.get();
+    }
+
+    /// Return the current nominal frame rate for the Stage.
+    //
+    /// This is dependent on the Movie set as root movie.
+    float frameRate() const {
+        return _rootMovie->frameRate();
     }
 
     void stop_drag()
@@ -322,15 +329,6 @@ public:
         m_drag_state.reset();
     }
 
-    /// Return definition of originating root movie 
-    //
-    /// TODO: rename to getOriginatingDefinition ?
-    ///
-    movie_definition* get_movie_definition() const
-    {
-	return getRootMovie()->get_movie_definition();
-    }
-
     /// Add an interval timer
     //
     /// @param timer
@@ -343,7 +341,11 @@ public:
     ///         for subsequent call to clear_interval_timer.
     ///         It will NEVER be zero.
     ///
-    unsigned int add_interval_timer(std::auto_ptr<Timer> timer, bool internal=false);
+    unsigned int add_interval_timer(std::auto_ptr<Timer> timer);
+
+    void addAdvanceCallback(ActiveRelay* obj);
+
+    void removeAdvanceCallback(ActiveRelay* obj);
 
     /// Remove timer identified by given integer
     //
@@ -358,42 +360,51 @@ public:
     ///
     size_t get_current_frame() const
     {
-	return getRootMovie()->get_current_frame();
+        return _rootMovie->get_current_frame();
     }
 
     void set_background_color(const rgba& color);
 
     void set_background_alpha(float alpha);
 
-    float get_background_alpha() const
-    {
-        return m_background_color.m_a / 255.0f;
-    }
-
+    /// Return the VM used by this movie_root
+    VM& getVM() { return _vm; }
+ 
     /// Main and only callback from hosting application.
     /// Expected to be called at 10ms resolution.
-    void advance();
+    //
+    /// @return true if the heart-beat resulted in actual
+    ///         SWF playhead advancement (frame advancement)
+    ///
+    bool advance();
+
+    /// \brief
+    /// Return the number of milliseconds available before
+    /// it's time to advance the timeline again.
+    //
+    /// Return value can be negative if we're late...
+    ///
+    int timeToNextFrame() const;
 
     /// Entry point for movie advancement
     //
     /// This function does:
-    ///     - Execute all timers
-    ///     - Reset the next Random number
-    ///     - Advance all advanceable characters in reverse-placement order
-    ///     - Cleanup key listeners
-    ///     - Process all queued actions
-    ///     - Remove unloaded characters from the advanceable characters list.
-    ///     - Run the GC collector
-    ///
+    ///   - Execute all timers
+    ///   - Reset the next Random number
+    ///   - Advance all advanceable DisplayObjects in reverse-placement order
+    ///   - Cleanup key listeners
+    ///   - Process all queued actions
+    ///   - Remove unloaded DisplayObjects from the advanceable
+    ///     DisplayObjects list.
+    ///   - Run the GC collector
     void advanceMovie();
 
     /// 0-based!! delegates to originating root movie
     //
     /// TODO: drop this method. currently used by gprocessor.
-    ///
     void goto_frame(size_t target_frame_number)
     {
-	getRootMovie()->goto_frame(target_frame_number);
+        _rootMovie->goto_frame(target_frame_number);
     }
 
     void display();
@@ -401,62 +412,62 @@ public:
     /// Delegate to originating root movie
     //
     /// TODO: drop ?
-    ///
-    void set_play_state(sprite_instance::play_state s)
+    void set_play_state(MovieClip::PlayState s)
     {
-	getRootMovie()->set_play_state(s);
+        _rootMovie->setPlayState(s);
     }
 
-	/// Notify still loaded character listeners for key events
-	DSOEXPORT void notify_key_listeners(key::code k, bool down);
+    /// Notify still loaded DisplayObject listeners for key events
+    DSOEXPORT void notify_key_listeners(key::code k, bool down);
 
-	/// Push a new character listener for key events
-	void add_key_listener(character* listener)
+    /// Push a new DisplayObject listener for key events
+    void add_key_listener(DisplayObject* listener)
     {
         add_listener(m_key_listeners, listener);
     }
 
-	/// Remove a character listener for key events
-	void remove_key_listener(character* listener)
+    /// Remove a DisplayObject listener for key events
+    void remove_key_listener(DisplayObject* listener)
     {
         remove_listener(m_key_listeners, listener);
     }
 
-	/// Notify still loaded character listeners for mouse events
-	DSOEXPORT void notify_mouse_listeners(const event_id& event);
+    /// Notify still loaded DisplayObject listeners for mouse events
+    DSOEXPORT void notify_mouse_listeners(const event_id& event);
 
-	/// Push a new character listener for mouse events
-	void add_mouse_listener(character* listener)
+    /// Push a new DisplayObject listener for mouse events
+    void add_mouse_listener(DisplayObject* listener)
     {
         add_listener(m_mouse_listeners, listener);
     }
 
-	/// Remove a character listener for mouse events
-	void remove_mouse_listener(character* listener)
+    /// Remove a DisplayObject listener for mouse events
+    void remove_mouse_listener(DisplayObject* listener)
     {
         remove_listener(m_mouse_listeners, listener);
     }
 
-    /// Get the character having focus
+    /// Get the DisplayObject having focus
     //
-    /// The character having focus will receive mouse button
+    /// The DisplayObject having focus will receive mouse button
     /// and key presses/releases.
     ///
-    /// @return the character having focus or NULL of none.
+    /// @return the DisplayObject having focus or NULL of none.
     ///
-    character* getFocus();
+    boost::intrusive_ptr<DisplayObject> getFocus();
 
-    /// Set the character having focus
+    /// Set the DisplayObject having focus
     //
-    /// @param ch
-    /// The character having focus. NULL to kill focus.
-    ///
-    void setFocus(character* ch);
+    /// @param to
+    /// The DisplayObject to receive focus. NULL to kill focus.
+    /// @return true if the focus operation succeeded, false if the passed
+    /// DisplayObject cannot receive focus. setFocus(0) is a valid operation, so
+    /// returns true (always succeeds).
+    bool setFocus(boost::intrusive_ptr<DisplayObject> to);
     
-    DSOEXPORT void add_invalidated_bounds(InvalidatedRanges& ranges, bool force);
+    DSOEXPORT void add_invalidated_bounds(InvalidatedRanges& ranges,
+            bool force);
     
-    void dump_character_tree() const;
-
     /// Return the topmost active entity under the pointer
     //
     /// This method returns cached info, with cache updated
@@ -465,32 +476,30 @@ public:
     /// currently implmented).
     ///
     /// @return the topmost active entity under pointer or NULL if none.
-    ///
-    character* getActiveEntityUnderPointer() const;
+    DisplayObject* getActiveEntityUnderPointer() const;
 
     /// Return the topmost non-dragging entity under the pointer
     //
     /// This method triggers a displaylist scan
     ///
     /// @return the topmost non-dragging entity under pointer or NULL if none
-    ///
-    const character* getEntityUnderPointer() const;
+    const DisplayObject* getEntityUnderPointer() const;
 
-    /// Return the character currently being dragged, if any
-    character* getDraggingCharacter() const;
+    /// Return the DisplayObject currently being dragged, if any
+    DisplayObject* getDraggingCharacter() const;
 
     /// Return true if the mouse pointer is over an active entity
     bool isMouseOverActiveEntity() const;
 
     bool testInvariant() const;
 
-    /// enum for the values of Stage.displayState
+    /// The possible values of Stage.displayState
     enum DisplayState {
-		normal,
-		fullScreen
-	};
+        DISPLAYSTATE_NORMAL,
+        DISPLAYSTATE_FULLSCREEN
+    };
 
-    /// enum for the values of Stage.scaleMode
+    /// The possibile values of Stage.scaleMode
     enum ScaleMode {
         showAll,
         noScale,
@@ -498,20 +507,21 @@ public:
         noBorder
     };
 
-    /// enum for horizonal position of the Stage
+    /// The possible horizonal positions of the Stage
     enum StageHorizontalAlign {
         STAGE_H_ALIGN_C,
         STAGE_H_ALIGN_L,
         STAGE_H_ALIGN_R,
     };
 
-    /// enum for vertical position of the Stage
+    /// The possible vertical position of the Stage
     enum StageVerticalAlign {
         STAGE_V_ALIGN_C,
         STAGE_V_ALIGN_T,       
         STAGE_V_ALIGN_B
     };
 
+    /// The possible elements of a Stage.alignMode.
     enum AlignMode {
         STAGE_ALIGN_L,
         STAGE_ALIGN_T,
@@ -519,6 +529,12 @@ public:
         STAGE_ALIGN_B
     };
 
+    /// Set the current display quality of the entire SWF.
+    void setQuality(Quality q);
+
+    /// Get the current display quality.
+    Quality getQuality() const { return _quality; }
+
     /// Sets movie_root's horizontal and vertical alignment to one
     /// of the three possible positions for each dimension.
     void setStageAlignment(short s);
@@ -529,6 +545,14 @@ public:
     /// bottom/centre) as a std::pair
     StageAlign getStageAlignment() const;
 
+	/// Returns the current value of _showMenu which instructs the gui about
+	/// how much to display in the context menu
+	bool getShowMenuState() const;
+
+	/// Sets the value of _showMenu and calls the fscommand handler for the
+	/// current gui
+	void setShowMenuState( bool state );
+
     /// Sets the Stage object's align mode.
     void setStageScaleMode(ScaleMode sm);
     
@@ -568,10 +592,12 @@ public:
     void pushAction(std::auto_ptr<ExecutableCode> code, int lvl=apDOACTION);
 
     /// Push an executable code to the ActionQueue
-    void pushAction(const action_buffer& buf, boost::intrusive_ptr<character> target, int lvl=apDOACTION);
+    void pushAction(const action_buffer& buf,
+            boost::intrusive_ptr<DisplayObject> target, int lvl=apDOACTION);
 
     /// Push a function code to the ActionQueue
-    void pushAction(boost::intrusive_ptr<as_function> func, boost::intrusive_ptr<character> target, int lvl=apDOACTION);
+    void pushAction(boost::intrusive_ptr<as_function> func,
+            boost::intrusive_ptr<DisplayObject> target, int lvl=apDOACTION);
 
 #ifdef GNASH_USE_GC
     /// Mark all reachable resources (for GC)
@@ -587,26 +613,27 @@ public:
     /// - Mouse listeners (m_mouse_listeners)
     /// - global Key object (_keyobject)
     /// - global Mouse object (_mouseobject)
-    /// - Any character being dragged 
+    /// - Any DisplayObject being dragged 
     ///
     void markReachableResources() const;
 #endif // GNASH_USE_GC
 
     /// \brief
-    /// Register a newly born advanceable character to the
-    /// list of characters to be advanced on next ::advance call.
+    /// Register a newly born advanceable DisplayObject to the
+    /// list of DisplayObjects to be advanced on next ::advance call.
     //
-    /// The character will only be advanced if not unloaded when
+    /// The DisplayObject will only be advanced if not unloaded when
     /// its turn comes. Characters are advanced in reverse-placement
     /// order (first registered is advanced last)
     ///
-    void addLiveChar(boost::intrusive_ptr<character> ch)
+    void addLiveChar(boost::intrusive_ptr<DisplayObject> ch)
     {
-	// Don't register the object in the list twice 
+        // Don't register the object in the list twice 
 #if GNASH_PARANOIA_LEVEL > 1
-	assert(std::find(_liveChars.begin(), _liveChars.end(), ch) == _liveChars.end());
+        assert(std::find(_liveChars.begin(), _liveChars.end(), ch) ==
+            _liveChars.end());
 #endif
-        _liveChars.push_front(ch);
+        _liveChars.push_front(ch.get());
     }
 
     /// Cleanup all resources and run the GC collector
@@ -616,8 +643,7 @@ public:
     ///
     void clear();
 
-    /// Reset stage to it's initial state
-    //
+    /// Reset stage to its initial state
     void reset();
 
     /// Call this method for disabling run of actions
@@ -646,31 +672,40 @@ public:
     ///
     void flushHigherPriorityActionQueues();
 
-    character* findCharacterByTarget(const std::string& tgtstr) const;
-
-    /// URL access methods
-    enum LoadMethod {
-        NONE=0,
-        GET=1,
-        POST=2
-    };
+    DisplayObject* findCharacterByTarget(const std::string& tgtstr) const;
 
     /// Queue a request for loading a movie
     //
-    /// @param url
-    ///		The url to load.
-    ///
-    /// @param target
-    ///	    Target to load into.
-    ///
-    /// @param postdata
-    ///     If not null, the data to POST in an HTTP request.
-    ///     Tests show that if you queue a load request for a target which
-    ///     is unloaded at time of processing, you still get the original
-    ///     target variables posted, not the new ones !
-    ///	    See http://savannah.gnu.org/bugs/index.php?22257
-    ///
-    void loadMovie(const URL& url, const std::string& target, const std::string* postdata=NULL);
+    /// This function constructs the URL and, if required, the postdata
+    /// from the arguments. The variables to send should *not* be appended
+    /// to @param urlstr before calling this function.
+    //
+    /// @param urlstr   The url exactly as requested. This may already
+    ///                 contain a query string.
+    /// @param target   Target for request.
+    /// @param data     The variables data to send, URL encoded in
+    ///                 key/value pairs
+    /// @param method   The VariablesMethod to use for sending the data. If
+    ///                 MovieClip::METHOD_NONE, no data will be sent.
+    void loadMovie(const std::string& url, const std::string& target,
+            const std::string& data, MovieClip::VariablesMethod method);
+
+    /// Send a request to the hosting application (e.g. browser).
+    //
+    /// This function constructs the URL and, if required, the postdata
+    /// from the arguments. The variables to send should *not* be appended
+    /// to @param urlstr before calling this function.
+    //
+    /// @param urlstr   The url exactly as requested. This may already
+    ///                 contain a query string.
+    /// @param target   Target for request.
+    /// @param data     The variables data to send, URL encoded in
+    ///                 key/value pairs
+    /// @param method   The VariablesMethod to use for sending the data. If
+    ///                 MovieClip::METHOD_NONE, no data will be sent.
+    void getURL(const std::string& urlstr, const std::string& target,
+            const std::string& data, MovieClip::VariablesMethod method);
+
 
     /// Return true if the given string can be interpreted as a _level name
     //
@@ -679,8 +714,8 @@ public:
     ///   Will be considered case-insensitive if VM version is < 7.
     ///
     /// @param levelno
-    ///   Output parameter, will be set to the level number, if true is returned
-    ///
+    ///   Output parameter, will be set to the level number, if true is
+    ///   returned
     bool isLevelTarget(const std::string& name, unsigned int& levelno);
 
 
@@ -696,7 +731,7 @@ public:
     /// (for browser communication mostly)
     ///
     /// @return -1 if no filedescriptor is provided by host app.
-    int getHostFD()
+    int getHostFD() const
     {
         return _hostfd;
     }
@@ -714,22 +749,40 @@ public:
     /// a handler, which will be called when the embedded scripts
     /// call fscommand().
     ///
-    /// The handler gets the sprite_instance* that the script is
+    /// The handler gets the MovieClip* that the script is
     /// embedded in, and the two string arguments passed by the
     /// script to fscommand().
-    ///
     DSOEXPORT void registerFSCommandCallback(AbstractFsCallback* handler)
     {
         _fsCommandHandler = handler;
     }
 
     /// Call this to notify FS commands
-    DSOEXPORT void handleFsCommand(const std::string& cmd, const std::string& arg) const;
+    DSOEXPORT void handleFsCommand(const std::string& cmd,
+            const std::string& arg) const;
 
     /// Abstract base class for hosting app handler
-    class AbstractIfaceCallback {
+    class AbstractIfaceCallback
+    {
     public:
-        virtual std::string call(const std::string& cmd, const std::string& arg)=0;
+
+        /// Get Gui-related information for the core.
+        //
+        /// This should be used for occasional AS calls, such as for
+        /// Mouse.hide, System.capabilities etc. The return can be
+        /// various types, so it is passed as a string.
+        virtual std::string call(const std::string& cmd,
+                const std::string& arg = std::string()) = 0;
+
+        /// Ask the hosting application for a yes / no answer to
+        /// a question.
+        virtual bool yesNo(const std::string& cmd) = 0;
+
+        /// Send an error message to the hosting application.
+        //
+        /// This does not have to be implemented; the default is a no-op.
+        virtual void error(const std::string& /*msg*/) {}
+
         virtual ~AbstractIfaceCallback() {}
     };
 
@@ -739,7 +792,6 @@ public:
     /// information, for instance).
     ///
     /// See callInterface method
-    ///
     DSOEXPORT void registerEventCallback(AbstractIfaceCallback* handler)
     {
         _interfaceHandler = handler;
@@ -748,8 +800,16 @@ public:
     /// Call into the hosting application
     ///
     /// Will use callback set with registerEventCallback
-    ///
-    DSOEXPORT std::string callInterface(const std::string& cmd, const std::string& arg) const;
+    DSOEXPORT std::string callInterface(const std::string& cmd,
+            const std::string& arg = std::string()) const;
+
+    /// Send an error message to the hosting application.
+    //
+    /// @param msg  A message to send describing the error.
+    //
+    /// The hosting app decides what to do with the message, or whether it
+    /// wants to do anything at all. It may show a popup box.
+    DSOEXPORT void errorInterface(const std::string& msg) const;
 
     /// Called from the ScriptLimits tag parser to set the
     /// global script limits. It is expected behaviour that
@@ -785,10 +845,56 @@ public:
 #ifdef USE_SWFTREE
     typedef std::pair<std::string, std::string> StringPair;
     void getMovieInfo(tree<StringPair>& tr, tree<StringPair>::iterator it);
+    void getCharacterTree(tree<StringPair>& tr, tree<StringPair>::iterator it);
 #endif
 
+	/// Get URL of the SWF movie used to initialize this VM
+	//
+	/// This information will be used for security checks
+	///
+	const std::string& getOriginalURL() const { return _originalURL; }
+
+    const RunResources& runResources() const { return _runResources; }
+
+    /// Add a DisplayObject child on top depth
+    //
+    /// @param ch
+    ///     The child DisplayObject to add
+    void addChild(DisplayObject* ch);
+
+    /// Add a DisplayObject child at given depth
+    //
+    /// @param ch
+    ///     The child DisplayObject to add
+    ///
+    /// @param depth
+    ///     The depth to add the child to
+    ///
+    void addChildAt(DisplayObject* ch, int depth);
+	
+	int getXMouseLoc()
+	{
+		return m_mouse_x;
+	}
+	
+	int getYMouseLoc()
+	{
+		return m_mouse_y;
+	}
+	
+
 private:
 
+    const RunResources& _runResources; 
+
+    /// The URL of the original root movie.
+    //
+    /// This is a runtime constant because it must not change during a 
+    /// run.
+    const std::string _originalURL;
+
+    /// This initializes a SharedObjectLibrary, which requires 
+    /// _originalURL, so that must be initialized first.
     VM& _vm;
 
     /// Registered Interface command handler, if any
@@ -800,21 +906,22 @@ private:
     /// A load movie request
     class LoadMovieRequest {
     public:
-	/// @param postdata
-	///   If not null POST method will be used for HTTP.
-	///
-        LoadMovieRequest(const URL& u, const std::string& t, const std::string* postdata)
-            :
-            _target(t),
-            _url(u),
-	    _usePost(false)
+        /// @param postdata
+        ///   If not null POST method will be used for HTTP.
+        ///
+        LoadMovieRequest(const URL& u, const std::string& t,
+                const std::string* postdata)
+                :
+                _target(t),
+                _url(u),
+                _usePost(false)
         {
-		if ( postdata )
-		{
-			_postData = *postdata;
-			_usePost = true;
-		}
-	}
+            if ( postdata )
+            {
+                _postData = *postdata;
+                _usePost = true;
+            }
+        }
 
         const std::string& getTarget() const { return _target; }
         const URL& getURL() const { return _url; }
@@ -825,7 +932,7 @@ private:
         std::string _target;
         URL _url;
         bool _usePost;
-	std::string _postData;
+        std::string _postData;
     };
 
     /// Load movie requests
@@ -839,13 +946,12 @@ private:
     void processLoadMovieRequest(const LoadMovieRequest& r);
 
     /// Listeners container
-    typedef std::list< boost::intrusive_ptr<character> > CharacterList;
+    typedef std::list<DisplayObject*> CharacterList;
 
     /// key and mouse listeners container
     typedef CharacterList KeyListeners;
     typedef CharacterList MouseListeners;
 
-
     /// Take care of dragging, if needed
     void doMouseDrag();
 
@@ -855,26 +961,25 @@ private:
     /// Delete all elements on the timers list
     void clearIntervalTimers();
 
-    /// An element of the advanceable characters
-    typedef boost::intrusive_ptr<character> AdvanceableCharacter;
-
     /// A list of AdvanceableCharacters
     //
     /// This is a list (not a vector) as we want to allow
-    /// ::advance of each element to insert new characters before
+    /// ::advance of each element to insert new DisplayObjects before
     /// the start w/out invalidating iterators scanning the
     /// list forward for proper movie advancement
-    ///
-    typedef std::list<AdvanceableCharacter> LiveChars;
+    typedef std::list<DisplayObject*> LiveChars;
 
-    /// The list of advanceable character, in placement order
+    /// The list of advanceable DisplayObject, in placement order
     LiveChars _liveChars;
 
+    /// Execute expired timers
+    void executeAdvanceCallbacks();
+    
     /// Execute expired timers
     void executeTimers();
 
     /// Notify the global Key ActionScript object about a key status change
-    key_as_object * notify_global_key(key::code k, bool down);
+    Keyboard_as * notify_global_key(key::code k, bool down);
 
     /// Remove unloaded key and mouselisteners.
     void cleanupUnloadedListeners()
@@ -883,24 +988,29 @@ private:
         cleanupUnloadedListeners(m_mouse_listeners);
     }
 
-    /// Erase unloaded characters from the given listeners list
+    /// Erase unloaded DisplayObjects from the given listeners list
     static void cleanupUnloadedListeners(CharacterList& ll);
 
-    /// Cleanup references to unloaded characters and run the garbage collector.
+    /// Cleanup references to unloaded DisplayObjects and run the garbage collector.
     void cleanupAndCollect();
 
-    /// Push a character listener to the front of given container, if not already present
-    static void add_listener(CharacterList& ll, character* elem);
+    /// Push a DisplayObject listener to the front of given container, if not
+    /// already present
+    static void add_listener(CharacterList& ll, DisplayObject* elem);
 
     /// Remove a listener from the list
-    static void remove_listener(CharacterList& ll, character* elem);
+    static void remove_listener(CharacterList& ll, DisplayObject* elem);
 
     /// Return the current Stage object
     //
     /// Can return NULL if it's been deleted or not
     /// yet initialized.
-    ///
-    boost::intrusive_ptr<Stage> getStageObject();
+    as_object* getStageObject();
+
+    /// Return the singleton Selection object
+    //
+    /// Can return 0 if it's been deleted.
+    as_object* getSelectionObject() const;
 
     typedef std::list<ExecutableCode*> ActionQueue;
 
@@ -910,19 +1020,23 @@ private:
     void processActionQueue();
 
     // TODO: use Range2d<int> ?
-    int         m_viewport_x0, m_viewport_y0;
+    int m_viewport_x0, m_viewport_y0;
 
     /// Width and height of viewport, in pixels
-    int         m_viewport_width, m_viewport_height;
+    int m_viewport_width, m_viewport_height;
 
-    rgba        m_background_color;
-    bool	m_background_color_set;
+    rgba m_background_color;
+    bool m_background_color_set;
 
-    float       m_timer;
-    int         m_mouse_x, m_mouse_y, m_mouse_buttons;
+    float m_timer;
+    int m_mouse_x, m_mouse_y, m_mouse_buttons;
 
     MouseButtonState  m_mouse_button_state;
 
+    /// Objects requesting a callback on every movie_root::advance()
+    typedef std::set<ActiveRelay*> ObjectCallbacks;
+    ObjectCallbacks _objectCallbacks;
+
     typedef std::map<int, Timer*> TimerMap;
 
     TimerMap _intervalTimers;
@@ -931,41 +1045,39 @@ private:
     /// Characters for listening key events
     KeyListeners m_key_listeners;
 
-    boost::intrusive_ptr<key_as_object> _keyobject;
+    boost::intrusive_ptr<Keyboard_as> _keyobject;
 
     boost::intrusive_ptr<as_object> _mouseobject;
 
     /// Objects listening for mouse events (down,up,move)
     MouseListeners m_mouse_listeners;
 
-    character*              m_active_input_text;
-    float                   m_time_remainder;
+    /// The DisplayObject currently holding focus, or 0 if no focus.
+    boost::intrusive_ptr<DisplayObject> _currentFocus;
 
-    /// @@ fold this into m_mouse_button_state?
+    float m_time_remainder;
+
+    /// @todo fold this into m_mouse_button_state?
     drag_state m_drag_state;
 
+    typedef boost::intrusive_ptr<MovieClip> LevelMovie;
+    typedef std::map<int, LevelMovie> Levels;
+
     /// The movie instance wrapped by this movie_root
     //
-    /// We keep a pointer to the base sprite_instance class
+    /// We keep a pointer to the base MovieClip class
     /// to avoid having to replicate all of the base class
-    /// interface to the movie_instance class definition
-    ///
-    /// TODO: use a different container, to allow for _level0 and _level100
-    ///       to exist while just taking 2 elements in the container.
-    ///       Appropriate container could be list, set or map (order is important)
-    ///
-    typedef boost::intrusive_ptr<sprite_instance> LevelMovie;
-    typedef std::map<int, LevelMovie> Levels;
+    /// interface to the Movie class definition
     Levels _movies;
 
     /// The root movie. This is initially the same as getLevel(0) but might
-    /// change during the run. It will be used to setup and retrive initial stage size
-    boost::intrusive_ptr<movie_instance> _rootMovie;
+    /// change during the run. It will be used to setup and retrive initial
+    /// stage size
+    boost::intrusive_ptr<Movie> _rootMovie;
 
     /// This function should return TRUE iff any action triggered
     /// by the event requires redraw, see \ref events_handling for
     /// more info.
-    ///
     bool fire_mouse_event();
 
     bool generate_mouse_button_events();
@@ -985,18 +1097,19 @@ private:
     /// @param y
     ///     Y ordinate of the pointer, in world coordiante space (twips).
     ///
-    character* getTopmostMouseEntity(boost::int32_t x, boost::int32_t y);
+    InteractiveObject* getTopmostMouseEntity(boost::int32_t x,
+            boost::int32_t y) const;
 
-    /// Delete characters removed from the stage
+    /// Delete DisplayObjects removed from the stage
     /// from the display lists
     void cleanupDisplayList();
 
-    /// Advance a live character
+    /// Advance a live DisplayObject
     //
     /// @param ch
-    ///     The character to advance, will NOT be advanced if unloaded
+    ///     The DisplayObject to advance, will NOT be advanced if unloaded
     ///
-    static void advanceLiveChar(boost::intrusive_ptr<character> ch);
+    static void advanceLiveChar(boost::intrusive_ptr<DisplayObject> ch);
 
     /// Advance all non-unloaded live chars
     void advanceLiveChars();
@@ -1004,35 +1117,34 @@ private:
     /// Put the given movie at the given level 
     //
     /// @param movie
-    /// The movie_instance to store at the given level.
+    /// The Movie to store at the given level.
     /// Will be stored in an intrusive_ptr.
-    /// It's depth will be set to <num>+character::staticDepthOffset and it's name to
-    /// _level<num>
-    ///
-    void setLevel(unsigned int num, boost::intrusive_ptr<movie_instance> movie);
+    /// Its depth will be set to <num>+DisplayObject::staticDepthOffset and
+    /// its name to _level<num>
+    void setLevel(unsigned int num, boost::intrusive_ptr<Movie> movie);
 
     /// Return the global Key object 
     //
     /// @@ might be worth making public
     ///
-    boost::intrusive_ptr<key_as_object> getKeyObject();
+    boost::intrusive_ptr<Keyboard_as> getKeyObject();
 
     /// Return the global Mouse object 
     //
-    /// TODO: expose the mouse_as_object directly for faster calls ?
+    /// TODO: expose the Mouse as_object directly for faster calls ?
     ///
     boost::intrusive_ptr<as_object> getMouseObject();
 
     /// Boundaries of the Stage are always world boundaries
     /// and are only invalidated by changes in the background
     /// color.
-    void setInvalidated() { _invalidated=true; }
+    void setInvalidated() { _invalidated = true; }
 
     /// Every ::display call clears the invalidated flag
     //
     /// See setInvalidated();
     ///
-    void clearInvalidated() { _invalidated=false; }
+    void clearInvalidated() { _invalidated = false; }
 
     /// An invalidated stage will trigger complete redraw
     //
@@ -1069,15 +1181,23 @@ private:
         return (_processingActionLevel < apSIZE);
     }
 
-    const character* findDropTarget(boost::int32_t x, boost::int32_t y, character* dragging) const;
+    const DisplayObject* findDropTarget(boost::int32_t x, boost::int32_t y,
+            DisplayObject* dragging) const;
 
     /// filedescriptor to write to for host application requests
     //
     /// -1 if none
-    ///
     int _hostfd;
-    
-    std::bitset<4> _alignMode;
+
+    /// The display quality of the entire movie.
+    //
+    /// This is here, not just in the Renderer, so that AS compatibility
+    /// does not rely on the presence of a renderer.
+    Quality _quality;
+
+    std::bitset<4u> _alignMode;
+
+	bool _showMenu;
     
     ScaleMode _scaleMode;
     
@@ -1091,9 +1211,6 @@ private:
     // ScriptLimits tag.    
     boost::uint16_t _timeoutLimit;
 
-    /// Hosting app gui, externally owned
-    Gui* _gui;
-
     void handleActionLimitHit(const std::string& ref);
 
     // delay between movie advancement, in milliseconds
@@ -1103,7 +1220,6 @@ private:
     unsigned int _lastMovieAdvancement;
 };
 
-
 } // namespace gnash
 
 #endif // GNASH_MOVIE_ROOT_H
