@@ -34,6 +34,7 @@ static string_table::svt preload_names[] =
 	string_table::svt( "a", NSV::PROP_A ),
 	string_table::svt( "addListener", NSV::PROP_ADD_LISTENER ),
 	string_table::svt( "align", NSV::PROP_ALIGN ),
+	string_table::svt( "ASnative", NSV::PROP_AS_NATIVE ),
 	string_table::svt( "_alpha", NSV::PROP_uALPHA ),
 	string_table::svt( "b", NSV::PROP_B ),
 	string_table::svt( "blockIndent", NSV::PROP_BLOCK_INDENT ),
@@ -54,6 +55,7 @@ static string_table::svt preload_names[] =
 	string_table::svt( "data", NSV::PROP_DATA ),
 	string_table::svt( "decode", NSV::PROP_DECODE ),
 	string_table::svt( "e", NSV::PROP_E ),	
+	string_table::svt( "escape", NSV::PROP_ESCAPE ),	
 	string_table::svt( "_droptarget", NSV::PROP_uDROPTARGET ),
 	string_table::svt( "enabled", NSV::PROP_ENABLED ),
 	string_table::svt( "useHandCursor", NSV::PROP_USEHANDCURSOR ),
@@ -61,8 +63,8 @@ static string_table::svt preload_names[] =
 	string_table::svt( "_focusrect", NSV::PROP_uFOCUSRECT ),
 	string_table::svt( "_framesloaded", NSV::PROP_uFRAMESLOADED ),
 	string_table::svt( "_height", NSV::PROP_uHEIGHT ),
-    string_table::svt( "g", NSV::PROP_G ),
-    string_table::svt( "h", NSV::PROP_H ),
+        string_table::svt( "g", NSV::PROP_G ),
+        string_table::svt( "h", NSV::PROP_H ),
 	string_table::svt( "height", NSV::PROP_HEIGHT ),
 	string_table::svt( "_highquality", NSV::PROP_uHIGHQUALITY ),
 	string_table::svt( "_quality", NSV::PROP_uQUALITY ),
@@ -83,7 +85,7 @@ static string_table::svt preload_names[] =
 	string_table::svt( "onLoadError", NSV::PROP_ON_LOAD_ERROR ),
 	string_table::svt( "onLoadProgress", NSV::PROP_ON_LOAD_PROGRESS ),
 	string_table::svt( "onLoadInit", NSV::PROP_ON_LOAD_INIT ),
-    string_table::svt( "onSoundComplete", NSV::PROP_ON_SOUND_COMPLETE ),
+        string_table::svt( "onSoundComplete", NSV::PROP_ON_SOUND_COMPLETE ),
 	string_table::svt( "onUnload", NSV::PROP_ON_UNLOAD ),
 	string_table::svt( "onEnterFrame", NSV::PROP_ON_ENTER_FRAME ),
 	string_table::svt( "onConstruct", NSV::PROP_ON_CONSTRUCT ),
@@ -111,7 +113,7 @@ static string_table::svt preload_names[] =
 	string_table::svt( "onResult", NSV::PROP_ON_RESULT ),
 	string_table::svt( "onMetaData", NSV::PROP_ON_META_DATA ),
 	string_table::svt( "onConnect", NSV::PROP_ON_CONNECT ),
-    string_table::svt( "onXML", NSV::PROP_ON_XML ),
+        string_table::svt( "onXML", NSV::PROP_ON_XML ),
 	string_table::svt( "parseXML", NSV::PROP_PARSE_XML ),
 	string_table::svt( "onTimer", NSV::PROP_ON_TIMER ),
 	string_table::svt( "_parent", NSV::PROP_uPARENT ),
@@ -132,6 +134,7 @@ static string_table::svt preload_names[] =
 	string_table::svt( "Stage", NSV::PROP_iSTAGE ),
 	string_table::svt( "status", NSV::PROP_STATUS ),
 	string_table::svt( "super", NSV::PROP_SUPER ),
+	string_table::svt( "target", NSV::PROP_TARGET ),
 	string_table::svt( "_target", NSV::PROP_uTARGET ),
 	string_table::svt( "text", NSV::PROP_TEXT ),
 	string_table::svt( "textColor", NSV::PROP_TEXT_COLOR ),
@@ -158,19 +161,33 @@ static string_table::svt preload_names[] =
 	string_table::svt( "_ymouse", NSV::PROP_uYMOUSE ),
 	string_table::svt( "_yscale", NSV::PROP_uYSCALE ),
 	string_table::svt( "System", NSV::CLASS_SYSTEM ),
+	string_table::svt( "AntiAliasType", NSV::CLASS_ANTIALIASTYPE ),
+	string_table::svt( "CSMTextSettings", NSV::CLASS_CSMTEXTSETTINGS ),
+	string_table::svt( "Font", NSV::CLASS_FONT),
+	string_table::svt( "FontStyle", NSV::CLASS_FONTSTYLE),
+	string_table::svt( "GridFitType", NSV::CLASS_GRIDFITTYPE),
+	string_table::svt( "StaticText", NSV::CLASS_STATICTEXT),
+	string_table::svt( "StyleSheet", NSV::CLASS_STYLESHEET),
+	string_table::svt( "TextColorType", NSV::CLASS_TEXTCOLORTYPE),
+	string_table::svt( "TextDisplayMode", NSV::CLASS_TEXTDISPLAYMODE),
+	string_table::svt( "TextFieldType", NSV::CLASS_TEXTFIELDTYPE),
+	string_table::svt( "TextFormatAlign", NSV::CLASS_TEXTFORMATALIGN),
+	string_table::svt( "TextLineMetrics", NSV::CLASS_TEXTLINEMETRICS),
+	string_table::svt( "TextRenderer", NSV::CLASS_TEXTRENDERER),
 //	string_table::svt( "Stage", NSV::CLASS_STAGE ), // Identical to PROP_iSTAGE
 	string_table::svt( "MovieClip", NSV::CLASS_MOVIE_CLIP ),
 	string_table::svt( "TextField", NSV::CLASS_TEXT_FIELD ),
 	string_table::svt( "Button", NSV::CLASS_BUTTON ),
 	string_table::svt( "Math", NSV::CLASS_MATH ),
 	string_table::svt( "Boolean", NSV::CLASS_BOOLEAN ),
+	string_table::svt( "Bitmap", NSV::CLASS_BITMAP ),
 	string_table::svt( "Color", NSV::CLASS_COLOR ),
 	string_table::svt( "Selection", NSV::CLASS_SELECTION ),
 	string_table::svt( "Sound", NSV::CLASS_SOUND ),
-	string_table::svt( "XMLSocket", NSV::CLASS_X_M_L_SOCKET ),
+	string_table::svt( "XMLSocket", NSV::CLASS_XMLSOCKET ),
 	string_table::svt( "Date", NSV::CLASS_DATE ),
-	string_table::svt( "XML", NSV::CLASS_X_M_L ),
-	string_table::svt( "XMLNode", NSV::CLASS_X_M_L_NODE ),
+	string_table::svt( "XML", NSV::CLASS_XML ),
+	string_table::svt( "XMLNode", NSV::CLASS_XMLNODE ),
 	string_table::svt( "Mouse", NSV::CLASS_MOUSE ),
 	string_table::svt( "Object", NSV::CLASS_OBJECT ),
 	string_table::svt( "String", NSV::CLASS_STRING ),
@@ -187,13 +204,25 @@ static string_table::svt preload_names[] =
 	string_table::svt( "Microphone", NSV::CLASS_MICROPHONE ),
 	string_table::svt( "SharedObject", NSV::CLASS_SHARED_OBJECT ),
 	string_table::svt( "LoadVars", NSV::CLASS_LOAD_VARS ),
-	string_table::svt( "LocalConnection", NSV::CLASS_LOCAL_CONNECTION ),
+	string_table::svt( "LocalConnection", NSV::CLASS_LOCALCONNECTION ),
 	string_table::svt( "CustomActions", NSV::CLASS_CUSTOM_ACTIONS ),
+	string_table::svt( "QName", NSV::CLASS_QNAME ),
+	string_table::svt( "Namespace", NSV::CLASS_NAMESPACE ),
 	string_table::svt( "NetConnection", NSV::CLASS_NET_CONNECTION ),
 	string_table::svt( "NetStream", NSV::CLASS_NET_STREAM ),
-	string_table::svt( "ContextMenu", NSV::CLASS_CONTEXT_MENU ),
+	string_table::svt( "ContextMenu", NSV::CLASS_CONTEXTMENU ),
+	string_table::svt( "ContextMenuItem", NSV::CLASS_CONTEXTMENUITEM ),
 	string_table::svt( "MovieClipLoader", NSV::CLASS_MOVIE_CLIP_LOADER ),
 	string_table::svt( "Error", NSV::CLASS_ERROR ),
+	string_table::svt( "Event", NSV::CLASS_EVENT),
+	string_table::svt( "EventDispatcher", NSV::CLASS_EVENTDISPATCHER),
+	string_table::svt( "DisplayObject", NSV::CLASS_DISPLAYOBJECT),
+	string_table::svt( "InteractiveObject", NSV::CLASS_INTERACTIVEOBJECT ),
+	string_table::svt( "DisplayObjectContainer", NSV::CLASS_DISPLAYOBJECTCONTAINER ),
+	string_table::svt( "Sprite", NSV::CLASS_SPRITE ),
+	string_table::svt( "int", NSV::CLASS_INT ),
+	string_table::svt( "TextFieldAutoSize", NSV::CLASS_TEXTFIELDAUTOSIZE),
+	string_table::svt( "onSync", NSV::PROP_ON_SYNC ),
 	string_table::svt( "flash.display", NSV::NS_FLASH_DISPLAY ),
 	string_table::svt( "flash.text", NSV::NS_FLASH_TEXT ),
 	string_table::svt( "flash.geom", NSV::NS_FLASH_GEOM ),
@@ -201,6 +230,7 @@ static string_table::svt preload_names[] =
 	string_table::svt( "flash.system", NSV::NS_FLASH_SYSTEM ),
 	string_table::svt( "flash.utils", NSV::NS_FLASH_UTILS ),
 	string_table::svt( "flash.events", NSV::NS_FLASH_EVENTS ),
+	string_table::svt( "flash.errors", NSV::NS_FLASH_ERRORS ),
 	string_table::svt( "flash.accessibility", NSV::NS_FLASH_ACCESSIBILITY ),
 	string_table::svt( "flash.media", NSV::NS_FLASH_MEDIA ),
 	string_table::svt( "flash.xml", NSV::NS_FLASH_XML ),
