@@ -1,6 +1,6 @@
 // action_buffer.cpp:  holds actions for later execution, for Gnash.
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -25,7 +25,7 @@
 #include "action_buffer.h"
 #include "log.h"
 #include "SWFStream.h"
-#include "swf.h"
+#include "SWF.h"
 #include "ASHandlers.h"
 #include "as_environment.h"
 #include "movie_definition.h"
@@ -42,7 +42,6 @@ namespace gnash {
 
 // Forward declarations
 static float convert_float_little(const void *p);
-static double convert_double_wacky(const void *p);
 
 action_buffer::action_buffer(const movie_definition& md)
     :
@@ -174,8 +173,8 @@ disasm_instruction(const unsigned char* instruction_data, size_t maxBufferLength
 
     assert (maxBufferLength > 0);
 
-    as_arg_t fmt = ARG_HEX;
-    action_type action_id = static_cast<action_type>(instruction_data[0]);
+    ArgumentType fmt = ARG_HEX;
+    ActionType action_id = static_cast<ActionType>(instruction_data[0]);
 
     std::stringstream ss;
 
@@ -531,7 +530,7 @@ convert_float_little(const void *p)
 // Read a 64-bit double from memory, stored in word-swapped little-endian
 // format and return it as a host-endian double.
 // "Wacky format" is 45670123.
-static double
+double
 convert_double_wacky(const void *p)
 {
     const boost::uint8_t *cp = (const boost::uint8_t *)p;    // Handy uchar version
@@ -622,6 +621,12 @@ action_buffer::getDefinitionURL() const
     return _src.get_url();
 }
 
+int
+action_buffer::getDefinitionVersion() const
+{
+    return _src.get_version();
+}
+
 }
 
 // Local Variables:
