@@ -27,13 +27,10 @@
 #include "action.h"
 #include "action_buffer.h"
 #include "Font.h"
-#include "fontlib.h"
 #include "log.h"
-#include "morph2_character_def.h"
-#include "shape.h"
+#include "Geometry.h"
 #include "SWFStream.h"
 #include "styles.h"
-#include "timers.h"
 #include "GnashImage.h"
 #include "zlib_adapter.h"
 #include "sprite_definition.h"
@@ -41,22 +38,18 @@
 #include "swf_function.h"
 #include "as_function.h"
 #include "SWFMovieDefinition.h"
-#include "swf.h"
+#include "SWF.h"
 #include "swf/TagLoadersTable.h"
 #include "URL.h"
 #include "GnashException.h"
 #include "swf/DefineVideoStreamTag.h"
 #include "sound_definition.h"
-#include "abc_block.h"
 #include "SoundInfo.h"
-#include "gnash.h" // FileType enum
 #include "MediaHandler.h"
 #include "SimpleBuffer.h"
 #include "sound_handler.h"
 #include "ExportableResource.h"
-
-// TODO: pass the render handler with RunInfo and use that.
-#include "render.h"
+#include "MovieFactory.h"
 
 #ifdef HAVE_ZLIB_H
 #include <zlib.h>
@@ -70,7 +63,6 @@
 namespace gnash {
 
 namespace SWF {
-namespace tag_loaders {
 
 
 /// Anonymous namespace
@@ -172,14 +164,14 @@ public:
 
 // Silently ignore the contents of this tag.
 void null_loader(SWFStream& /*in*/, TagType /*tag*/, movie_definition& /*m*/,
-        const RunInfo& /*r*/)
+        const RunResources& /*r*/)
 {
 }
 
 // Label the current frame of m with the name from the SWFStream.
 void
 frame_label_loader(SWFStream& in, TagType tag, movie_definition& m,
-		const RunInfo& /*r*/)
+		const RunResources& /*r*/)
 {
     assert(tag == SWF::FRAMELABEL); // 43
 
@@ -223,7 +215,7 @@ frame_label_loader(SWFStream& in, TagType tag, movie_definition& m,
 // images further along in the SWFStream.
 void
 jpeg_tables_loader(SWFStream& in, TagType tag, movie_definition& m,
-		const RunInfo& /*r*/)
+		const RunResources& /*r*/)
 {
     //GNASH_REPORT_FUNCTION;
     assert(tag == SWF::JPEGTABLES);
@@ -281,18 +273,18 @@ jpeg_tables_loader(SWFStream& in, TagType tag, movie_definition& m,
 // existing JpegImageInput object stored in the movie.
 void
 define_bits_jpeg_loader(SWFStream& in, TagType tag, movie_definition& m,
-		const RunInfo& /*r*/)
+		const RunResources& r)
 {
     assert(tag == SWF::DEFINEBITS); // 6
 
     in.ensureBytes(2);
-    boost::uint16_t character_id = in.read_u16();
+    boost::uint16_t id = in.read_u16();
 
-    if (m.getBitmap(character_id))
+    if (m.getBitmap(id))
     {
         IF_VERBOSE_MALFORMED_SWF(
-        log_swferror(_("DEFINEBITS: Duplicate id (%d) for bitmap character "
-                "- discarding it"), character_id);
+        log_swferror(_("DEFINEBITS: Duplicate id (%d) for bitmap DisplayObject "
+                "- discarding it"), id);
         );
         return;
     }
@@ -303,7 +295,7 @@ define_bits_jpeg_loader(SWFStream& in, TagType tag, movie_definition& m,
     {
         IF_VERBOSE_MALFORMED_SWF(
         log_swferror(_("DEFINEBITS: No jpeg loader registered in movie "
-                "definition - discarding bitmap character %d"), character_id);
+                "definition - discarding bitmap DisplayObject %d"), id);
         );
         return;
     }
@@ -318,56 +310,83 @@ define_bits_jpeg_loader(SWFStream& in, TagType tag, movie_definition& m,
     catch (std::exception& e)
     {
         IF_VERBOSE_MALFORMED_SWF(
-        log_swferror("Error reading jpeg2 with headers for character "
-            "id %d: %s", character_id, e.what());
+        log_swferror("Error reading jpeg2 with headers for DisplayObject "
+            "id %d: %s", id, e.what());
         );
         return;
     }
     
-    boost::intrusive_ptr<BitmapInfo> bi = render::createBitmapInfo(im);
+    Renderer* renderer = r.renderer();
+    if (!renderer) {
+        IF_VERBOSE_PARSE(log_parse(_("No renderer, not adding bitmap")));
+        return;
+    }    
+    boost::intrusive_ptr<BitmapInfo> bi = renderer->createBitmapInfo(im);
 
-    // add bitmap to movie under character id.
-    m.addBitmap(character_id, bi);
+    // add bitmap to movie under DisplayObject id.
+    m.addBitmap(id, bi);
 }
 
 
 void
 define_bits_jpeg2_loader(SWFStream& in, TagType tag, movie_definition& m,
-		const RunInfo& /*r*/)
+		const RunResources& r)
 {
     assert(tag == SWF::DEFINEBITSJPEG2); // 21
 
     in.ensureBytes(2);
-    boost::uint16_t character_id = in.read_u16();
+    boost::uint16_t id = in.read_u16();
 
     IF_VERBOSE_PARSE
     (
     log_parse(_("  define_bits_jpeg2_loader: charid = %d pos = %ld"),
-          character_id, in.tell());
+          id, in.tell());
     );
 
     
-    if ( m.getBitmap(character_id) )
+    if ( m.getBitmap(id) )
     {
         IF_VERBOSE_MALFORMED_SWF(
         log_swferror(_("DEFINEBITSJPEG2: Duplicate id (%d) for bitmap "
-                "character - discarding it"), character_id);
+                "DisplayObject - discarding it"), id);
         );
         return;
     }
 
-    // Read the image data.else
+    char buf[3];
+    if (in.read(buf, 3) < 3) {
+        log_swferror(_("DEFINEBITS data too short to read type header"));
+        return;
+    }
+    in.seek(in.tell() - 3);
 
-    boost::shared_ptr<IOChannel> ad( StreamAdapter::getFile(in,
+    FileType ft = GNASH_FILETYPE_JPEG;  
+
+    // Check the data type. The pp version 9,0,115,0 supports PNG and GIF
+    // in DefineBits tags, though it is not documented. The version makes
+    // no difference.
+    if (std::equal(buf, buf + 3, "\x89PN")) {
+        ft = GNASH_FILETYPE_PNG;
+    }
+    else if (std::equal(buf, buf + 3, "GIF")) {
+        ft = GNASH_FILETYPE_GIF;
+    }
+
+    // Read the image data.
+    boost::shared_ptr<IOChannel> ad(StreamAdapter::getFile(in,
                 in.get_tag_end_position()).release() );
 
-    std::auto_ptr<GnashImage> im (ImageInput::readImageData(ad,
-                GNASH_FILETYPE_JPEG));
+    std::auto_ptr<GnashImage> im (ImageInput::readImageData(ad, ft));
 
-    boost::intrusive_ptr<BitmapInfo> bi = render::createBitmapInfo(im);
+    Renderer* renderer = r.renderer();
+    if (!renderer) {
+        IF_VERBOSE_PARSE(log_parse(_("No renderer, not adding bitmap")));
+        return;
+    }    
+    boost::intrusive_ptr<BitmapInfo> bi = renderer->createBitmapInfo(im);
 
-    // add bitmap to movie under character id.
-    m.addBitmap(character_id, bi);
+    // add bitmap to movie under DisplayObject id.
+    m.addBitmap(id, bi);
 
 }
 
@@ -464,17 +483,17 @@ void inflate_wrapper(SWFStream& in, void* buffer, int buffer_bytes)
 // channel using zlib compression.
 void
 define_bits_jpeg3_loader(SWFStream& in, TagType tag, movie_definition& m,
-		const RunInfo& /*r*/)
+		const RunResources& r)
 {
     assert(tag == SWF::DEFINEBITSJPEG3); // 35
 
     in.ensureBytes(2);
-    boost::uint16_t character_id = in.read_u16();
+    boost::uint16_t id = in.read_u16();
 
     IF_VERBOSE_PARSE
     (
     log_parse(_("  define_bits_jpeg3_loader: charid = %d pos = %lx"),
-          character_id, in.tell());
+          id, in.tell());
     );
 
     in.ensureBytes(4);
@@ -513,25 +532,31 @@ define_bits_jpeg3_loader(SWFStream& in, TagType tag, movie_definition& m,
     //  ea8bbad50ccbc52dd734dfc93a7f06a7  6964trev3c.swf
     im->mergeAlpha(buffer.get(), bufferLength);
 
-    boost::intrusive_ptr<BitmapInfo> bi =
-        render::createBitmapInfo(static_cast<std::auto_ptr<GnashImage> >(im));
 
-    // add bitmap to movie under character id.
-    m.addBitmap(character_id, bi);
+    Renderer* renderer = r.renderer();
+    if (!renderer) {
+        IF_VERBOSE_PARSE(log_parse(_("No renderer, not adding bitmap")));
+        return;
+    }    
+    boost::intrusive_ptr<BitmapInfo> bi =
+        renderer->createBitmapInfo(static_cast<std::auto_ptr<GnashImage> >(im));
+
+    // add bitmap to movie under DisplayObject id.
+    m.addBitmap(id, bi);
 #endif
 }
 
 
 void
 define_bits_lossless_2_loader(SWFStream& in, TagType tag, movie_definition& m,
-		const RunInfo& /*r*/)
+		const RunResources& r)
 {
     // tags 20 || 36
     assert(tag == SWF::DEFINELOSSLESS || tag == SWF::DEFINELOSSLESS2);
 
     in.ensureBytes(2+2+2+1); // the initial header 
 
-    boost::uint16_t character_id = in.read_u16();
+    boost::uint16_t id = in.read_u16();
 
     // 3 == 8 bit, 4 == 16 bit, 5 == 32 bit
     boost::uint8_t bitmap_format = in.read_u8();
@@ -541,25 +566,25 @@ define_bits_lossless_2_loader(SWFStream& in, TagType tag, movie_definition& m,
     IF_VERBOSE_PARSE(
         log_parse(_("  defbitslossless2: tag = %d, id = %d, "
             "fmt = %d, w = %d, h = %d"),
-            tag, character_id, bitmap_format, width, height);
+            tag, id, bitmap_format, width, height);
     );
 
     if (!width || !height) {
          IF_VERBOSE_MALFORMED_SWF(
-            log_swferror(_("Bitmap character %d has a height or width of 0"),
-                character_id);
+            log_swferror(_("Bitmap DisplayObject %d has a height or width of 0"),
+                id);
         );   
         return;  
     }
 
     // No need to parse any further if it already exists, as we aren't going
     // to add it.
-    if (m.getBitmap(character_id))
+    if (m.getBitmap(id))
     {
         IF_VERBOSE_MALFORMED_SWF(
             log_swferror(_("DEFINEBITSLOSSLESS: Duplicate id (%d) "
-                           "for bitmap character - discarding it"),
-                character_id);
+                           "for bitmap DisplayObject - discarding it"),
+                id);
         );
     }
 
@@ -701,10 +726,15 @@ define_bits_lossless_2_loader(SWFStream& in, TagType tag, movie_definition& m,
 
     }
 
-    boost::intrusive_ptr<BitmapInfo> bi = render::createBitmapInfo(image);
+    Renderer* renderer = r.renderer();
+    if (!renderer) {
+        IF_VERBOSE_PARSE(log_parse(_("No renderer, not adding bitmap")));
+        return;
+    }    
+    boost::intrusive_ptr<BitmapInfo> bi = renderer->createBitmapInfo(image);
 
-    // add bitmap to movie under character id.
-    m.addBitmap(character_id, bi);
+    // add bitmap to movie under DisplayObject id.
+    m.addBitmap(id, bi);
 #endif // HAVE_ZLIB_H
 
 }
@@ -712,7 +742,7 @@ define_bits_lossless_2_loader(SWFStream& in, TagType tag, movie_definition& m,
 // This is like null_loader except it prints a message to nag us to fix it.
 void
 fixme_loader(SWFStream& /*in*/, TagType tag, movie_definition& /*m*/,
-		const RunInfo& /*r*/)
+		const RunResources& /*r*/)
 {
     static std::set<TagType> warned;
     if (warned.insert(tag).second) {
@@ -720,58 +750,19 @@ fixme_loader(SWFStream& /*in*/, TagType tag, movie_definition& /*m*/,
     }
 }
 
-void define_shape_loader(SWFStream& in, TagType tag, movie_definition& m,
-		const RunInfo& /*r*/)
-{
-    assert(tag == SWF::DEFINESHAPE
-       || tag == SWF::DEFINESHAPE2
-       || tag == SWF::DEFINESHAPE3
-       || tag == SWF::DEFINESHAPE4 || tag == SWF::DEFINESHAPE4_);
-
-    in.ensureBytes(2);
-    boost::uint16_t    character_id = in.read_u16();
-    IF_VERBOSE_PARSE(
-        log_parse(_("  shape_loader: id = %d"), character_id);
-    );
-
-    shape_character_def* ch = new shape_character_def;
-    ch->read(in, tag, true, m);
-
-    m.add_character(character_id, ch);
-}
-
-void define_shape_morph_loader(SWFStream& in, TagType tag, movie_definition& m,
-		const RunInfo& /*r*/)
-{
-    assert(tag == SWF::DEFINEMORPHSHAPE
-        || tag == SWF::DEFINEMORPHSHAPE2
-        || tag == SWF::DEFINEMORPHSHAPE2_); 
-
-    in.ensureBytes(2);
-    boost::uint16_t character_id = in.read_u16();
-
-    IF_VERBOSE_PARSE(
-        log_parse(_("  shape_morph_loader: id = %d"), character_id);
-    );
-
-    morph2_character_def* morph = new morph2_character_def;
-    morph->read(in, tag, m);
-    m.add_character(character_id, morph);
-}
-
 // Create and initialize a sprite, and add it to the movie.
 void
 sprite_loader(SWFStream& in, TagType tag, movie_definition& m,
-		const RunInfo& r)
+		const RunResources& r)
 {
     assert(tag == SWF::DEFINESPRITE); // 39 - DefineSprite
 
     in.ensureBytes(2);
-    int    character_id = in.read_u16();
+    int    id = in.read_u16();
 
     IF_VERBOSE_PARSE
     (
-    log_parse(_("  sprite:  char id = %d"), character_id);
+    log_parse(_("  sprite:  char id = %d"), id);
     );
 
     // A DEFINESPRITE tag as part of a DEFINESPRITE
@@ -783,7 +774,7 @@ sprite_loader(SWFStream& in, TagType tag, movie_definition& m,
         }
         catch (std::bad_cast& e) {
             log_swferror(_("Nested DEFINESPRITE tags. Will add to "
-                           "top-level characters dictionary."));
+                           "top-level DisplayObjects dictionary."));
         }
     );
 
@@ -792,12 +783,12 @@ sprite_loader(SWFStream& in, TagType tag, movie_definition& m,
 
     IF_VERBOSE_MALFORMED_SWF(
         if (!ch->get_frame_count()) {
-            log_swferror(_("Sprite %d advertise no frames"), character_id);
+            log_swferror(_("Sprite %d advertise no frames"), id);
         }
     );
 
 
-    m.add_character(character_id, ch);
+    m.addDisplayObject(id, ch);
 }
 
 
@@ -807,7 +798,7 @@ sprite_loader(SWFStream& in, TagType tag, movie_definition& m,
 
 
 void export_loader(SWFStream& in, TagType tag, movie_definition& m,
-		const RunInfo& /*r*/)
+		const RunResources& /*r*/)
     // Load an export tag (for exposing internal resources of m)
 {
     assert(tag == SWF::EXPORTASSETS); // 56
@@ -845,10 +836,10 @@ void export_loader(SWFStream& in, TagType tag, movie_definition& m,
             log_parse(_("  export: id = %d, name = %s"), id, symbolName);
         );
 
-        // Fonts, characters and sounds can be exported.
+        // Fonts, DisplayObjects and sounds can be exported.
         ExportableResource* f;
         if ((f = m.get_font(id)) ||
-            (f = m.get_character_def(id)) ||
+            (f = m.getDefinitionTag(id)) ||
             (f = m.get_sound_sample(id))) {
             
             m.export_resource(symbolName, f);
@@ -870,7 +861,7 @@ void export_loader(SWFStream& in, TagType tag, movie_definition& m,
 
 
 void import_loader(SWFStream& in, TagType tag, movie_definition& m,
-		const RunInfo& r)
+		const RunResources& r)
 {
     assert(tag == SWF::IMPORTASSETS || tag == SWF::IMPORTASSETS2);
 
@@ -904,7 +895,7 @@ void import_loader(SWFStream& in, TagType tag, movie_definition& m,
     boost::intrusive_ptr<movie_definition> source_movie;
 
     try {
-        source_movie = create_library_movie(abs_url, r);
+        source_movie = MovieFactory::makeMovie(abs_url, r);
     }
     catch (gnash::GnashException& e) {
         log_error(_("Exception: %s"), e.what());
@@ -969,15 +960,15 @@ static unsigned int s_sample_rate_table_len = 4;
 // Load a DefineSound tag.
 void
 define_sound_loader(SWFStream& in, TagType tag, movie_definition& m,
-		const RunInfo& r)
+		const RunResources& r)
 {
     assert(tag == SWF::DEFINESOUND); // 14
 
     sound::sound_handler* handler = r.soundHandler();
 
-    in.ensureBytes(2+4+1+4); // character id + flags + sample count
+    in.ensureBytes(2+4+1+4); // DisplayObject id + flags + sample count
 
-    boost::uint16_t    character_id = in.read_u16();
+    boost::uint16_t    id = in.read_u16();
 
     media::audioCodecType format = static_cast<media::audioCodecType>(
             in.read_uint(4));
@@ -999,34 +990,19 @@ define_sound_loader(SWFStream& in, TagType tag, movie_definition& m,
 
     unsigned int    sample_count = in.read_u32();
 
+    boost::int16_t delaySeek = 0;
+
     if (format == media::AUDIO_CODEC_MP3) {
         in.ensureBytes(2);
-        boost::int16_t    delay_seek = in.read_s16();    // FIXME - not implemented/used
-        //
-        // The DelaySeek field has the following meaning:
-        // * If this value is positive, the player seeks this number of
-        //   samples into the sound block before the sound is played.
-        //   However, the seek is only performed if the player reached
-        //   the current frame via a GotoFrame action, otherwise no
-        //   seek is performed.
-        //
-        // * If this value is negative the player plays this number of
-        //   silent samples before playing the sound block. The player
-        //   behaves this way, regardless of how the current frame was
-        //   reached.
-        //
-        // quoted from
-        // http://www-lehre.informatik.uni-osnabrueck.de/~fbstark/diplom/docs/swf/Sounds.htm
-        //
-        if ( delay_seek ) { LOG_ONCE( log_unimpl("MP3 delay seek") ); }
+        delaySeek = in.read_s16();
     }
 
     IF_VERBOSE_PARSE
     (
         log_parse(_("define sound: ch=%d, format=%s, "
-            "rate=%d, 16=%d, stereo=%d, ct=%d"),
-              character_id, format, sample_rate,
-              int(sample_16bit), int(stereo), sample_count);
+            "rate=%d, 16=%d, stereo=%d, ct=%d, delay=%d"),
+              id, format, sample_rate,
+              int(sample_16bit), int(stereo), sample_count, delaySeek);
     );
 
     // If we have a sound_handler, ask it to init this sound.
@@ -1059,7 +1035,7 @@ define_sound_loader(SWFStream& in, TagType tag, movie_definition& m,
         // Store all the data in a SoundInfo object
         std::auto_ptr<media::SoundInfo> sinfo;
         sinfo.reset(new media::SoundInfo(format, stereo, sample_rate,
-                    sample_count, sample_16bit));
+                    sample_count, sample_16bit, delaySeek));
 
         // Stores the sounddata in the soundhandler, and the ID returned
         // can be used to starting, stopping and deleting that sound
@@ -1068,7 +1044,7 @@ define_sound_loader(SWFStream& in, TagType tag, movie_definition& m,
         if (handler_id >= 0)
         {
         sound_sample* sam = new sound_sample(handler_id, r);
-        m.add_sound_sample(character_id, sam);
+        m.add_sound_sample(id, sam);
         }
 
     }
@@ -1076,16 +1052,16 @@ define_sound_loader(SWFStream& in, TagType tag, movie_definition& m,
     {
         // is this nice to do?
         log_error(_("There is no sound handler currently active, "
-            "so character with id %d will NOT be added to "
+            "so DisplayObject with id %d will NOT be added to "
             "the dictionary"),
-              character_id);
+              id);
     }
 }
 
 // Load a SoundStreamHead(2) tag.
 void
 sound_stream_head_loader(SWFStream& in, TagType tag, movie_definition& m,
-		const RunInfo& r)
+		const RunResources& r)
 {
 
     // 18 || 45
@@ -1097,7 +1073,7 @@ sound_stream_head_loader(SWFStream& in, TagType tag, movie_definition& m,
     if (!handler) return;
 
     // FIXME:
-    // no character id for soundstreams... so we make one up...
+    // no DisplayObject id for soundstreams... so we make one up...
     // This only works if there is only one SWFStream in the movie...
     // The right way to do it is to make seperate structures for streams
     // in SWFMovieDefinition.
@@ -1185,8 +1161,7 @@ sound_stream_head_loader(SWFStream& in, TagType tag, movie_definition& m,
         try
         {
             in.ensureBytes(2);
-            latency = in.read_s16(); // UNUSED !!
-            LOG_ONCE ( if ( latency ) log_unimpl("MP3 stream latency seek") );
+            latency = in.read_s16(); 
         }
         catch (ParserException& ex)
         {
@@ -1212,7 +1187,7 @@ sound_stream_head_loader(SWFStream& in, TagType tag, movie_definition& m,
 
     // Store all the data in a SoundInfo object
     std::auto_ptr<media::SoundInfo> sinfo;
-    sinfo.reset(new media::SoundInfo(format, streamSoundStereo, streamSoundRate, sampleCount, streamSound16bit));
+    sinfo.reset(new media::SoundInfo(format, streamSoundStereo, streamSoundRate, sampleCount, streamSound16bit, latency));
 
     // Stores the sounddata in the soundhandler, and the ID returned
     // can be used to starting, stopping and deleting that sound
@@ -1223,16 +1198,17 @@ sound_stream_head_loader(SWFStream& in, TagType tag, movie_definition& m,
 
 
 void
-file_attributes_loader(SWFStream& in, TagType tag, movie_definition& /*m*/,
-        const RunInfo& /*r*/)
+file_attributes_loader(SWFStream& in, TagType tag, movie_definition& m,
+        const RunResources& /*r*/)
 {
     assert(tag == SWF::FILEATTRIBUTES); // 69
 
     struct file_attrs_flags {
         unsigned reserved1;
-        unsigned has_metadata;
+        bool metadata;
+        bool as3;
         unsigned reserved2;
-        unsigned use_network;
+        bool network;
         unsigned reserved3;
     };
 
@@ -1240,47 +1216,59 @@ file_attributes_loader(SWFStream& in, TagType tag, movie_definition& /*m*/,
 
     in.ensureBytes(1 + 3);
     flags.reserved1 = in.read_uint(3);
-    flags.has_metadata = in.read_bit(); 
-    flags.reserved2 = in.read_uint(3);
-    flags.use_network = in.read_bit(); 
+    flags.metadata = in.read_bit(); 
+    flags.as3 = in.read_bit();
+    flags.reserved2 = in.read_uint(2);
+    flags.network = in.read_bit(); 
     flags.reserved3 = in.read_uint(24);
 
     IF_VERBOSE_PARSE
     (
-    log_parse(_("  file attributes: has_metadata=%s use_network=%s"),
-          flags.has_metadata ? _("true") : _("false"),
-          flags.use_network ? _("true") : _("false"))
+        log_parse(_("File attributes: metadata=%s network=%s"),
+              flags.metadata ? _("true") : _("false"),
+              flags.network ? _("true") : _("false"))
     );
 
-    if ( ! flags.use_network )
-    {
-    log_unimpl(_("FileAttributes tag in the SWF requests that "
-            "network access is not granted to this movie "
-            "(or application?) when loaded from the filesystem. "
-                "Anyway Gnash won't care; "
-            "use white/black listing in your .gnashrc instead"));
+    if (!flags.network) {
+        log_unimpl(_("FileAttributes tag in the SWF requests that "
+                "network access is not granted to this movie "
+                "(or application?) when loaded from the filesystem. "
+                    "Anyway Gnash won't care; "
+                "use white/black listing in your .gnashrc instead"));
     }
 
-    // TODO:
-    //     - attach info to movie_definition.
-    //     - don't allow later FileAttributes tags in the same movie
-    //       to override the first one used.
-    //     - only use if it is the *first* tag in the SWFStream.
+    if (flags.as3) {
+        log_debug("This SWF uses AVM2");
+#ifndef ENABLE_AVM2
+        /// Log an error if this build can't interpret AS3.
+        log_error(_("This SWF file requires AVM2, which was not enabled at "
+                    "compile time."));
+#endif
+    }
+    else log_debug("This SWF uses AVM1");
+
+    // TODO: - don't allow later FileAttributes tags in the same movie
+    //         to override the first one used.
+    //       - only use if it is the *first* tag in the SWFStream.
+
+    if (flags.as3) m.setAS3();
+
 }
 
 
 void
 metadata_loader(SWFStream& in, TagType tag, movie_definition& m,
-		const RunInfo& /*r*/)
+		const RunResources& /*r*/)
 {
-    assert(tag == SWF::METADATA); // 77
+    assert(tag == SWF::METADATA); 
 
     // this is supposed to be an XML string
     std::string metadata;
     in.read_string(metadata);
 
     IF_VERBOSE_PARSE (
-    log_parse(_("  RDF metadata (information only): [[\n%s\n]]"), metadata);
+        log_parse(_("  RDF metadata (information only): [[\n%s\n]]"),
+            metadata);
     );
 
     // The metadata tag exists exclusively for external
@@ -1312,7 +1300,7 @@ metadata_loader(SWFStream& in, TagType tag, movie_definition& m,
 
 void
 serialnumber_loader(SWFStream& in, TagType tag, movie_definition& /*m*/, 
-        const RunInfo& /*r*/)
+        const RunResources& /*r*/)
 {
     assert(tag == SWF::SERIALNUMBER); // 41
 
@@ -1346,7 +1334,7 @@ serialnumber_loader(SWFStream& in, TagType tag, movie_definition& /*m*/,
 
 void
 reflex_loader(SWFStream& in, TagType tag, movie_definition& /*m*/,
-        const RunInfo& /*r*/)
+        const RunResources& /*r*/)
 {
     assert(tag == SWF::REFLEX); // 777
 
@@ -1364,43 +1352,6 @@ reflex_loader(SWFStream& in, TagType tag, movie_definition& /*m*/,
 
 }
 
-void
-abc_loader(SWFStream& in, TagType tag, movie_definition& /*m*/,
-        const RunInfo& /*r*/)
-{
-    assert(tag == SWF::DOABC
-        || tag == SWF::DOABCDEFINE); // 72 or 82
-
-    abc_block a;
-
-    if (tag == SWF::DOABCDEFINE)
-    {
-
-        // Skip the 'flags' until they are actually used.
-        in.ensureBytes(4);
-        static_cast<void> (in.read_u32());
-        std::string name;
-        in.read_string(name);
-    }
-
-    //TODO: Move this to execution time so that as_object can be used.
-    // bool success = a.read(in);
-
-    log_unimpl(_("%s tag parsed but not yet used"),
-            tag == SWF::DOABC ? "DOABC" : "DOABCDEFINE");
-}
-
-void
-define_scene_frame_label_loader(SWFStream& /*in*/, TagType tag,
-        movie_definition& /*m*/, const RunInfo& /*r*/)
-{
-    assert(tag == SWF::DEFINESCENEANDFRAMELABELDATA); //86
-
-    log_unimpl(_("%s tag parsed but not yet used"), 
-            "DEFINESCENEANDFRAMELABELDATA");
-}
-
-} // namespace gnash::SWF::tag_loaders
 } // namespace gnash::SWF
 
 // Local Variables:
