@@ -28,12 +28,12 @@
 #include "log.h"
 #include "VM.h"
 #include "GnashException.h"
-
-#include "swf.h"
+#include "DisplayObject.h"
+#include "movie_root.h"
+#include "SWF.h"
 #include "ASHandlers.h"
 #include "as_environment.h"
 #include "debugger.h"
-//#include "WallClockTimer.h" // will probably be used for scriptTimeout
 
 #include <sstream>
 #include <string>
@@ -150,7 +150,7 @@ ActionExec::ActionExec(const action_buffer& abuf, as_environment& newEnv, bool a
 void
 ActionExec::operator() ()
 {
-    VM& vm = env.getVM();
+    VM& vm = getVM(env);
 
     // Do not execute if scripts are disabled
     if ( vm.getRoot().scriptsDisabled() ) return;
@@ -270,7 +270,7 @@ ActionExec::operator() ()
                       stop_pc);
                     );
                     
-                    //throw ActionException(ss.str());;
+                    //throw ActionException(ss.str());
                     // no way to recover from this actually...
                     // Give this action handler a chance anyway.
                     // Maybe it will be able to do something about
@@ -291,14 +291,14 @@ ActionExec::operator() ()
 
 #if 0
             // curveball.swf and feed.swf break with this
-            character* guardedChar = env.get_original_target(); // watch out : _originalTarget is not necessarely the same
+            DisplayObject* guardedChar = env.get_original_target(); // watch out : _originalTarget is not necessarely the same
 #else
             // curveball.swf and feed.swf suggest that it is the *current* target,
             // not the *original* one that matters.
-            character* guardedChar = env.get_target();
+            DisplayObject* guardedChar = env.get_target();
 #endif
 
-            if ( _abortOnUnload && guardedChar && guardedChar->isUnloaded() )
+            if ( _abortOnUnload && guardedChar && guardedChar->unloaded() )
             // action_execution_order_test8.c shows that the opcode guard is not SWF version based (TODO: automate it!)
             {
                 std::stringstream ss;
@@ -598,7 +598,7 @@ ActionExec::processExceptions(TryBlock& t)
 void
 ActionExec::cleanupAfterRun(bool /*expectInconsistencies*/) // TODO: drop argument...
 {
-    VM& vm = env.getVM();
+    VM& vm = getVM(env);
 
     //assert(_originalTarget); // this execution context might have been started while target had a null target
     env.set_target(_originalTarget);
@@ -622,7 +622,7 @@ ActionExec::cleanupAfterRun(bool /*expectInconsistencies*/) // TODO: drop argume
     );
 
     // Have movie_root flush any newly pushed actions in higher priority queues
-    env.getVM().getRoot().flushHigherPriorityActionQueues();
+    getRoot(env).flushHigherPriorityActionQueues();
 
 //    log_debug("After cleanup of ActionExec %p, env %p has "
 //        "stack size of %d and callStackDepth of %d",
@@ -702,7 +702,7 @@ ActionExec::delVariable(const std::string& name)
 bool
 ActionExec::delObjectMember(as_object& obj, const std::string& name)
 {
-    string_table& st = env.getVM().getStringTable();
+    string_table& st = getStringTable(env);
     std::pair<bool,bool> ret = obj.delProperty(st.find(PROPNAME(name)));
     return ret.second;
 }
@@ -716,7 +716,7 @@ ActionExec::setVariable(const std::string& name, const as_value& val)
 as_value
 ActionExec::getVariable(const std::string& name)
 {
-    return env.get_variable(PROPNAME(name), getScopeStack());
+    return env.get_variable(name, getScopeStack());
 }
 
 as_value
@@ -739,17 +739,19 @@ ActionExec::setLocalVariable(const std::string& name, const as_value& val)
 }
 
 void
-ActionExec::setObjectMember(as_object& obj, const std::string& var, const as_value& val)
+ActionExec::setObjectMember(as_object& obj, const std::string& var,
+        const as_value& val)
 {
-    string_table& st = env.getVM().getStringTable();
-    obj.set_member(st.find(PROPNAME(var)), val);
+    string_table& st = getStringTable(env);
+    obj.set_member(st.find(var), val);
 }
 
 bool
-ActionExec::getObjectMember(as_object& obj, const std::string& var, as_value& val)
+ActionExec::getObjectMember(as_object& obj, const std::string& var,
+        as_value& val)
 {
-    string_table& st = env.getVM().getStringTable();
-    return obj.get_member(st.find(PROPNAME(var)), &val);
+    string_table& st = getStringTable(env);
+    return obj.get_member(st.find(var), &val);
 }
 
 /*private*/
