@@ -126,7 +126,14 @@ Network::createServer(void)
 {
 //    GNASH_REPORT_FUNCTION;
 
-    return createServer(DEFAULTPORT);
+    short port;
+
+    if (_port) {
+	port = _port;
+    } else {
+	port = DEFAULTPORT;
+    }
+    return createServer(port);
 }
 
 // FIXME: Should also support IPv6 (AF_INET6)
@@ -139,15 +146,7 @@ Network::createServer(short port)
     struct sockaddr_in sock_in;
     int             on, type;
     int             retries = 0;
-    in_addr_t       nodeaddr;
 
-#if 0
-    if (port < 1024) {
-	log_error(_("Can't connect to privileged port #%d"), port);
-	return -1;
-    }
-#endif
-    
     if (_listenfd >= 2) {
 	log_debug("already connected to port %hd", port);
 	return _listenfd;
@@ -201,7 +200,8 @@ Network::createServer(short port)
 
     retries = 0;
 
-    nodeaddr = inet_lnaof(*thisaddr);
+//     in_addr_t       nodeaddr;
+//     nodeaddr = inet_lnaof(*thisaddr);
     while (retries < 5) {
         if (bind(_listenfd, reinterpret_cast<struct sockaddr *>(&sock_in),
                  sizeof(sock_in)) == -1) {
@@ -277,7 +277,7 @@ Network::newConnection(bool block, int fd)
         return -1;
     }
     if (_debug) {
-	log_debug(_("Trying to accept net traffic on fd #%d for port %d"), fd, _port);
+	log_debug(_("Waiting to accept net traffic on fd #%d for port %d"), fd, _port);
     }
 
 #ifdef HAVE_PSELECT
@@ -355,6 +355,7 @@ Network::newConnection(bool block, int fd)
         if (ret == 0) {
             if (_debug) {
                 log_debug(_("The accept() socket for fd #%d timed out waiting to write"), fd);
+		return 0;
             }
         }
 	
@@ -377,7 +378,7 @@ Network::newConnection(bool block, int fd)
     return _sockfd;
 }
 
-#ifdef _WIN32
+#if defined(_WIN32) || defined(__amigaos4__)
 /* from sys/socket.h */
 typedef unsigned short      sa_family_t;
 
@@ -489,7 +490,14 @@ Network::createClient(void)
 {
 //    GNASH_REPORT_FUNCTION;
 
-    return createClient("localhost", RTMP_PORT);
+    short port;
+
+    if (_port) {
+	port = _port;
+    } else {
+	port = RTMP_PORT;
+    }
+    return createClient("localhost", port);
 }
 bool
 Network::createClient(short /* port */)
@@ -504,7 +512,14 @@ Network::createClient(const string &hostname)
 {
 //    GNASH_REPORT_FUNCTION;
 
-    return createClient(hostname, RTMP_PORT);
+    short port;
+
+    if (_port) {
+	port = _port;
+    } else {
+	port = RTMP_PORT;
+    }
+    return createClient(hostname, port);
 }
 
 bool
@@ -520,12 +535,9 @@ Network::createClient(const string &hostname, short port)
     char                thishostname[MAXHOSTNAMELEN];
     struct protoent     *proto;
 
-    assert( ! connected() );
-
-    if (port < 1024) {
-        log_error(_("Can't connect to privileged port %hd"), port);
-        _connected = false;
-        return false;
+//    assert( ! connected() );
+    if (connected()) {
+        return true;
     }
 
     _port = port;    
@@ -541,6 +553,7 @@ Network::createClient(const string &hostname, short port)
             return false;
         }
     }
+
     const struct hostent *hent = ::gethostbyname(hostname.c_str());
     if (hent > 0) {
         ::memcpy(&sock_in.sin_addr, hent->h_addr, hent->h_length);
@@ -557,12 +570,11 @@ Network::createClient(const string &hostname, short port)
     proto = ::getprotobyname("TCP");
 
     _sockfd = ::socket(PF_INET, SOCK_STREAM, proto->p_proto);
-    if (_sockfd < 0)
-        {
-            log_error(_("unable to create socket: %s"), strerror(errno));
-            _sockfd = -1;
-            return false;
-        }
+    if (_sockfd < 0) {
+        log_error(_("unable to create socket: %s"), strerror(errno));
+        _sockfd = -1;
+        return false;
+    }
 
     retries = 2;
     while (retries-- > 0) {
@@ -579,37 +591,46 @@ Network::createClient(const string &hostname, short port)
         ret = ::select(_sockfd+1, &fdset, NULL, NULL, &tval);
 
         // If interupted by a system call, try again
-        if (ret == -1 && errno == EINTR)
-            {
-                log_debug(_("The connect() socket for fd %d was interupted by a system call"),
-                        _sockfd);
-                continue;
-            }
+        if (ret == -1 && errno == EINTR) {
+            log_debug(_("The connect() socket for fd %d was interrupted "
+                        "by a system call"), _sockfd);
+            continue;
+        }
 
-        if (ret == -1)
-            {
-                log_debug(_("The connect() socket for fd %d never was available for writing"),
-                        _sockfd);
+        if (ret == -1) {
+            log_debug(_("The connect() socket for fd %d never was "
+                        "available for writing"), _sockfd);
 #ifdef HAVE_WINSOCK_H
-                ::shutdown(_sockfd, 0); // FIXME: was SHUT_BOTH
+            ::shutdown(_sockfd, 0); // FIXME: was SHUT_BOTH
 #else
-                ::shutdown(_sockfd, SHUT_RDWR);
+            ::shutdown(_sockfd, SHUT_RDWR);
 #endif
-                _sockfd = -1;
-                return false;
-            }
+	    ::close(_sockfd);
+            _sockfd = -1;
+            return false;
+        }
+
         if (ret == 0) {
-            log_error(_("The connect() socket for fd %d timed out waiting to write"),
-                      _sockfd);
+#ifdef HAVE_WINSOCK_H
+            ::shutdown(_sockfd, 0); // FIXME: was SHUT_BOTH
+#else
+            ::shutdown(_sockfd, SHUT_RDWR);
+#endif
+            log_error(_("The connect() socket for fd %d timed out waiting "
+                        "to write"), _sockfd);
+	    ::close(_sockfd);
             continue;
         }
 
         if (ret > 0) {
-            ret = ::connect(_sockfd, reinterpret_cast<struct sockaddr *>(&sock_in), sizeof(sock_in));
+            ret = ::connect(_sockfd, 
+                    reinterpret_cast<struct sockaddr *>(&sock_in),
+                    sizeof(sock_in));
+
             if (ret == 0) {
-		char *ascip = ::inet_ntoa(sock_in.sin_addr);
-// 		char ascip[INET_ADDRSTRLEN];
-// 		inet_ntop(sock_in.sin_family, &sock_in.sin_addr.s_addr, ascip, INET_ADDRSTRLEN);
+                char *ascip = ::inet_ntoa(sock_in.sin_addr);
+        // 		char ascip[INET_ADDRSTRLEN];
+        // 		inet_ntop(sock_in.sin_family, &sock_in.sin_addr.s_addr, ascip, INET_ADDRSTRLEN);
                 log_debug(_("\tport %d at IP %s for fd %d"), port,
                         ascip, _sockfd);
                 _connected = true;
@@ -617,8 +638,14 @@ Network::createClient(const string &hostname, short port)
                 return true;
             }
             if (ret == -1) {
-                log_error(_("The connect() socket for fd %d never was available for writing"),
-                        _sockfd);
+                log_error(_("The connect() socket for fd %d never was "
+                            "available for writing"), _sockfd);
+#ifdef HAVE_WINSOCK_H
+		::shutdown(_sockfd, 0); // FIXME: was SHUT_BOTH
+#else
+		::shutdown(_sockfd, SHUT_RDWR);
+#endif
+		::close(_sockfd);
                 _sockfd = -1;
                 assert(!_connected);
                 return false;
@@ -751,6 +778,7 @@ Network::readNet()
 int
 Network::readNet(int fd, amf::Buffer &buffer)
 {
+//    GNASH_REPORT_FUNCTION;
     int ret = readNet(fd, buffer.reference(), buffer.size(), _timeout);
     if (ret > 0) {
 	buffer.resize(ret);
@@ -783,7 +811,7 @@ Network::readNet(amf::Buffer &buffer, int timeout)
 int
 Network::readNet(int fd, amf::Buffer &buffer, int timeout)
 {
-    GNASH_REPORT_FUNCTION;
+    // GNASH_REPORT_FUNCTION;
     int ret = readNet(fd, buffer.reference(), buffer.size(), timeout);
     buffer.setSeekPointer(ret);
 #if 0
@@ -819,11 +847,13 @@ Network::readNet(int fd, byte_t *data, int nbytes)
 int
 Network::readNet(int fd, byte_t *buffer, int nbytes, int timeout)
 {
-//    GNASH_REPORT_FUNCTION;
+//     GNASH_REPORT_FUNCTION;
 
     fd_set              fdset;
     int                 ret = -1;
 
+//     boost::mutex::scoped_lock lock(_net_mutex);
+
     if (_debug) {
 	log_debug (_("Trying to read %d bytes from fd #%d"), nbytes, fd);
     }
@@ -904,7 +934,16 @@ Network::readNet(int fd, byte_t *buffer, int nbytes, int timeout)
             return 0;
         }
 
-        ret = read(fd, buffer, nbytes);
+#ifdef USE_SSL
+	if (_ssl) {
+	    ret = _ssl->sslRead(buffer, nbytes);
+	} else {
+	    ret = read(fd, buffer, nbytes);
+	}
+#else
+	ret = read(fd, buffer, nbytes);
+#endif
+
 	// If we read zero bytes, the network may be closed, as we returned from the select()
         if (ret == -1) {
             log_error (_("The socket for fd #%d was never available for reading data"), fd);
@@ -937,14 +976,14 @@ Network::readNet(int fd, byte_t *buffer, int nbytes, int timeout)
 int
 Network::writeNet(amf::Buffer *buffer)
 {
-//    GNASH_REPORT_FUNCTION;
+//     GNASH_REPORT_FUNCTION;
     return writeNet(buffer->reference(), buffer->allocated());
 };
 
 int
 Network::writeNet(int fd, amf::Buffer *buffer)
 {
-//    GNASH_REPORT_FUNCTION;
+//     GNASH_REPORT_FUNCTION;
     return writeNet(fd, buffer->reference(), buffer->allocated());
 };
 
@@ -952,7 +991,7 @@ Network::writeNet(int fd, amf::Buffer *buffer)
 int
 Network::writeNet(amf::Buffer &buffer)
 {
-//    GNASH_REPORT_FUNCTION;
+//     GNASH_REPORT_FUNCTION;
     return writeNet(buffer.reference(), buffer.allocated());
 };
 
@@ -960,20 +999,21 @@ Network::writeNet(amf::Buffer &buffer)
 int
 Network::writeNet(int fd, amf::Buffer &buffer)
 {
-//    GNASH_REPORT_FUNCTION;
+//     GNASH_REPORT_FUNCTION;
     return writeNet(fd, buffer.reference(), buffer.allocated());
 };
 
 int
 Network::writeNet(const std::string& data)
 {
-//    GNASH_REPORT_FUNCTION;
+//     GNASH_REPORT_FUNCTION;
     return writeNet(reinterpret_cast<const byte_t *>(data.c_str()), data.size());
 }
 
 int
 Network::writeNet(const byte_t *data, int nbytes)
 {
+//     GNASH_REPORT_FUNCTION;
     return writeNet(_sockfd, data, nbytes, _timeout);
 }
 
@@ -992,15 +1032,20 @@ Network::writeNet(const byte_t *data, int nbytes)
 int
 Network::writeNet(int fd, const byte_t *data, int nbytes)
 {
+//     GNASH_REPORT_FUNCTION;
     return writeNet(fd, data, nbytes, _timeout);
 }
 
 int
 Network::writeNet(int fd, const byte_t *buffer, int nbytes, int timeout)
 {
+//     GNASH_REPORT_FUNCTION;
+
     fd_set              fdset;
     int                 ret = -1;
 
+    boost::mutex::scoped_lock lock(_net_mutex);
+    
     // We need a writable, and not const point for byte arithmetic.
     byte_t *bufptr = const_cast<byte_t *>(buffer);
 
@@ -1060,8 +1105,15 @@ Network::writeNet(int fd, const byte_t *buffer, int nbytes, int timeout)
 	    return 0;
         }
 
-        ret = write(fd, bufptr, nbytes);
-
+#ifdef USE_SSL
+	if (_ssl) {
+	    ret = _ssl->sslWrite(buffer, nbytes);
+	} else {
+	    ret = write(fd, bufptr, nbytes);
+	}
+#else
+	ret = write(fd, bufptr, nbytes);
+#endif
         if (ret == 0) {
             log_error (_("Wrote zero out of %d bytes to fd #%d: %s"), 
 		nbytes, fd, strerror(errno));
@@ -1209,6 +1261,10 @@ Network::waitForNetData(int limit, struct pollfd *fds)
 	return hits;
     }
     
+    int timeout = _timeout;
+    if (timeout <= 0) {
+	timeout = 5;
+    }
 #ifdef HAVE_PPOLL
 	struct timespec tval;
 	sigset_t pending, blockset;
@@ -1227,7 +1283,15 @@ Network::waitForNetData(int limit, struct pollfd *fds)
 	    sigwait(&blockset, &sig);
 	}
 #else
+#ifdef HAVE_POLL_H
 	int ret = poll(fds, limit, _timeout);
+#else
+	fd_set fdset;
+	struct timeval        tval;
+	tval.tv_sec = timeout;
+	tval.tv_usec = 0;
+	int ret = select(limit+1, &fdset, NULL, NULL, &tval);
+#endif
 #endif
 
     log_debug("Poll returned: %d, timeout is: %d", ret, _timeout);
@@ -1300,27 +1364,32 @@ Network::waitForNetData(int limit, struct pollfd *fds)
 fd_set
 Network::waitForNetData(vector<int> &data)
 {
-    GNASH_REPORT_FUNCTION;
+    // GNASH_REPORT_FUNCTION;
 
+    int max = 0;
+    
     fd_set fdset;
     FD_ZERO(&fdset);
     
     for (size_t i = 0; i<data.size(); i++) {
 	FD_SET(data[i], &fdset);
+	if (data[i] > max) {
+	    max = data[i];
+	}
     }
 
-    return waitForNetData(data.size(), fdset);
+    return waitForNetData(max+1, fdset);
 }
 
 fd_set
 Network::waitForNetData(int limit, fd_set files)
 {
-    GNASH_REPORT_FUNCTION;
+    // GNASH_REPORT_FUNCTION;
 
-    // select modifies this the set of file descriptors, and we don't
+    // select modifies the set of file descriptors, and we don't
     // want to modify the one passed as an argument, so we make a copy.
     fd_set fdset = files;
-    
+
     // Reset the timeout value, since select modifies it on return
     int timeout = _timeout;
     if (timeout <= 0) {
@@ -1331,8 +1400,8 @@ Network::waitForNetData(int limit, fd_set files)
     sigset_t pending, sigmask;
     sigprocmask(SIG_BLOCK, &sigmask, NULL);
 
-    tval.tv_sec = timeout;
-    tval.tv_nsec = 0;
+    tval.tv_sec = 0;
+    tval.tv_nsec = timeout * 1000;
     int ret = pselect(limit+1, &fdset, NULL, NULL, &tval, &sigmask);
     sigpending(&pending);
     if (sigismember(&pending, SIGINT)) {
@@ -1346,25 +1415,30 @@ Network::waitForNetData(int limit, fd_set files)
 	sigwait(&sigmask, &sig);
     }
 #else
-    struct timeval        tval;
-    tval.tv_sec = timeout;
-    tval.tv_usec = 0;
+    struct timeval tval;
+    tval.tv_sec = 0;
+    tval.tv_usec = timeout;
     int ret = select(limit+1, &fdset, NULL, NULL, &tval);
+    FD_ZERO(&fdset);
 #endif
     // If interupted by a system call, try again
     if (ret == -1 && errno == EINTR) {
-	log_error (_("Waiting for data for fdset 0x%x was interupted by a system call"), reinterpret_cast<long>(fdset.fds_bits));
+	log_error (_("Waiting for data was interupted by a system call"));
     }
     
     if (ret == -1) {
-	log_error (_("Waiting for data for fdset  0x%x was never available for reading"), reinterpret_cast<long>(fdset.fds_bits));
+	log_error (_("Waiting for data for fdset, was never available for reading"));
     }
     
     if (ret == 0) {
-	log_debug (_("Waiting for data for fdset  0x%x timed out waiting for data"), reinterpret_cast<long>(fdset.fds_bits));
+	// log_debug (_("Waiting for data for fdset, timed out waiting for data"));
 	FD_ZERO(&fdset);
     }
 
+    if (ret) {
+	log_network("select() saw activity on %d file descriptors.", ret);
+    }
+
     return fdset;
 }
 
@@ -1400,6 +1474,92 @@ Network::toggleDebug(bool val)
     // toggleDebug(true);
 }
 
+#ifdef USE_SSL
+bool
+Network::initSSL(std::string &hostname)
+{
+    GNASH_REPORT_FUNCTION;
+    string nothing;
+    initSSL(hostname, nothing);
+}
+
+bool
+Network::initSSL(std::string &hostname, std::string &password)
+{
+    GNASH_REPORT_FUNCTION;
+    string nothing;
+
+    initSSL(hostname, password, nothing, nothing, nothing, true);
+}
+
+bool
+Network::initSSL(std::string &hostname, std::string &password, bool auth)
+{
+    GNASH_REPORT_FUNCTION;
+
+    string nothing;
+    initSSL(hostname, password, nothing, nothing, nothing, auth);
+}
+
+bool
+Network::initSSL(std::string &hostname, std::string &passwd, 
+		 std::string &keyfile, std::string &calist,
+		 std::string &rootpath, bool auth)
+{
+    GNASH_REPORT_FUNCTION;
+
+    // FIXME: make sure we have a connection
+
+    if (_sockfd == 0) {
+	if ((_sockfd = createClient(hostname, SSL_PORT) == false)) {
+	    log_error("Can't connect to server %s", hostname);
+	    return false;
+	}
+    }
+
+    if (!_ssl) {
+	_ssl.reset(new SSLClient);
+    }
+
+    if (!hostname.empty()) {
+	_ssl->setHostname(hostname);
+    } else {
+	log_debug("Using default hostname: \"%s\"", _host);
+    }
+    if (!keyfile.empty()) {
+	_ssl->setKeyfile(keyfile);
+    } else {
+	log_debug("Using default keyfile: \"%s\"", _ssl->getKeyfile());
+    }
+    if (!calist.empty()) {
+	_ssl->setCAlist(calist);
+    } else {
+	log_debug("Using default CA List: \"%s\"", _ssl->getCAlist());
+    }
+
+    if (!passwd.empty()) {
+	_ssl->setPassword(passwd);
+    } else {
+	log_debug("Using default Password: \"%s\"", _ssl->getPassword());
+    }
+    if (!rootpath.empty()) {
+	_ssl->setRootPath(rootpath);
+    } else {
+	log_debug("Using default Root Path to PEM files: \"%s\"",
+		  _ssl->getRootPath());
+    }
+
+    if (_ssl->sslConnect(_sockfd)) {
+        log_debug("Connected to SSL server");
+    } else {
+        log_error("Couldn't connect to SSL server");
+	return false;
+    }
+
+    // If we got this far, everthing worked
+    return true;
+}
+#endif
 
 // Trap Control-C so we can cleanly exit
 static void
