@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -15,9 +15,6 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-// 
-//
-
 // Written by Koos Vriezen <koos ! vriezen ? xs4all ! nl>
 
 #ifndef KLASH_PART_H
@@ -147,6 +144,7 @@ protected slots:
 protected:
     virtual bool openFile();
 private:
+    QStringList m_args;
     KlashBrowserExtension * m_browserextension;
     KlashLiveConnectExtension * m_liveconnectextension;
     KProcess * m_process;
