@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -15,13 +15,12 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-/* $Id: plugin.h,v 1.34 2007/07/23 04:08:08 strk Exp $ */
 
-#ifndef __PLUGIN_H__
-#define __PLUGIN_H__
+#ifndef GNASH_PLUGIN_H
+#define GNASH_PLUGIN_H
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include "gnashconfig.h"
 #endif
 
 #ifndef HAVE_FUNCTION
@@ -52,20 +51,11 @@
 #include "prcvar.h"
 #include "prthread.h"
 
-/* ascii codes for various special keys */
-#define ESCAPE 27
-#define PAGE_UP 73
-#define PAGE_DOWN 81
-#define UP_ARROW 72
-#define DOWN_ARROW 80
-#define LEFT_ARROW 75
-#define RIGHT_ARROW 77
-
 extern NPBool      plugInitialized;
 extern PRLock      *playerMutex;
 extern PRCondVar   *playerCond;
 
-class DSOLOCAL nsPluginInstance : public nsPluginInstanceBase
+class nsPluginInstance : public nsPluginInstanceBase
 {
 public:
     nsPluginInstance(nsPluginCreateData* );
@@ -73,7 +63,7 @@ public:
 
     // We are required to implement these three methods.
     NPBool init(NPWindow *aWindow);
-    NPBool isInitialized() {return plugInitialized;}
+    NPBool isInitialized() { return plugInitialized; }
     void shut();
 
     NPError GetValue(NPPVariable variable, void *value);
@@ -93,10 +83,33 @@ public:
 
 private:
 
+    static bool handlePlayerRequestsWrapper(GIOChannel* iochan, GIOCondition cond, nsPluginInstance* plugin);
+
+    bool handlePlayerRequests(GIOChannel* iochan, GIOCondition cond);
+
+    /// Process a null-terminated request line
+    //
+    /// @param buf
+    ///	  The single request.
+    ///   Caller is responsible for memory management, but give us
+    ///   permission to modify the string.
+    ///
+    /// @param len
+    ///	  Lenght of buffer.
+    ///
+    /// @return true if the request was processed, false otherwise (bogus request..)
+    ///
+    bool processPlayerRequest(gchar* buf, gsize len);
+
     // EMBED or OBJECT attributes / parameters
     // @@ this should likely replace the _options element below
     std::map<std::string, std::string> _params;
 
+    /// Dump current session cookies to a file,
+    /// setting _cookieFile to its name (or clear
+    /// it on failure).
+    void dumpCookies();
+
     NPP                                _instance;
     Window                             _window;
     std::string                        _swf_url;
@@ -105,9 +118,17 @@ private:
     unsigned int                       _height;
     std::map<std::string, std::string> _options;
     int                                _streamfd;
+    GIOChannel*                        _ichan;
+    int                                _ichanWatchId;
     pid_t                              _childpid;
     int                                _filefd;
 
+    /// Name of the plugin instance element in the dom 
+    std::string                        _name;
+
+    /// Cookie file
+    std::string                        _cookieFile;
+
     const char* getCurrentPageURL() const;
 };
 
