@@ -1,6 +1,6 @@
 // plugin.cpp:  Windows "win32" flash player Mozilla plugin, for Gnash.
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -21,6 +21,8 @@
 #include "gnashconfig.h"
 #endif
 
+#include <cstdlib>
+
 #define FLASH_MAJOR_VERSION "9"
 #define FLASH_MINOR_VERSION "0"
 #define FLASH_REV_NUMBER "82"
@@ -48,7 +50,7 @@
     FLASH_MINOR_VERSION" r"FLASH_REV_NUMBER"."
 
 #define PLUGIN_DESCRIPTION \
-  "Shockwave Flash "FLASH_VERSION" Gnash "VERSION", the GNU Flash Player. \
+  "Shockwave Flash "FLASH_VERSION" Gnash "VERSION", the GNU SWF Player. \
   Copyright &copy; 2006, 2007, 2008 \
   <a href=\"http://www.fsf.org\">Free Software Foundation</a>, Inc.<br> \
   Gnash comes with NO WARRANTY, to the extent permitted by law. \
@@ -64,8 +66,8 @@
 #include <windowsx.h>
 #include <wingdi.h>
 
-#include <stdarg.h>
-#include <stdint.h>
+#include <cstdarg>
+#include <boost/cstdint.hpp>
 #include <fstream>
 
 #include "plugin.h"
@@ -332,7 +334,7 @@ nsPluginInstance::shut(void)
 
 NPError
 nsPluginInstance::NewStream(NPMIMEType type, NPStream *stream,
-        NPBool seekable, uint16_t *stype)
+        NPBool seekable, boost::uint16_t *stype)
 {
     DBG("nsPluginInstance::NewStream\n");
     DBG("stream->url: %s\n", stream->url);
@@ -392,22 +394,20 @@ nsPluginInstance::threadMain(void)
 
     // Initialize Gnash core library.
     gnash::gnashInit();
-    gnash::set_use_cache_files(false);
-    gnash::registerFSCommandCallback(FSCommand_callback);
     DBG("Gnash core initialized.\n");
  
     // Init logfile.
     gnash::RcInitFile& rcinit = gnash::RcInitFile::getDefaultInstance();
-    std::string logfilename = std::string(getenv("TEMP")) +
+    std::string logfilename = std::string(std::getenv("TEMP")) +
         std::string("\\npgnash.log");
     rcinit.setDebugLog(logfilename);
     gnash::LogFile& dbglogfile = gnash::LogFile::getDefaultInstance();
     dbglogfile.setWriteDisk(true);
-    dbglogfile.setVerbosity(DEBUGLEVEL);
+    dbglogfile.setVerbosity(GNASH_DEBUG_LEVEL);
     DBG("Gnash logging initialized: %s\n", logfilename.c_str());
 
     // Init sound.
-    _sound_handler.reset(gnash::media::create_sound_handler_sdl());
+    _sound_handler.reset(gnash::sound::create_sound_handler_sdl());
     gnash::set_sound_handler(_sound_handler.get());
     DBG("Gnash sound initialized.\n");
 
@@ -433,7 +433,7 @@ nsPluginInstance::threadMain(void)
 
     gnash::movie_definition* md = NULL;
     try {
-        md = gnash::create_library_movie(url, _url.c_str(), false);
+        md = gnash::createMovie(url, _url.c_str(), false);
     } catch (const gnash::GnashException& err) {
         md = NULL;
     }
@@ -463,18 +463,29 @@ nsPluginInstance::threadMain(void)
             movie_width, movie_height, movie_fps);
 
     gnash::SystemClock clock; // use system clock here...
-    gnash::movie_root& root = gnash::VM::init(*md, clock).getRoot(); 
+    gnash::movie_root& root = gnash::VM::init(*md, clock).getRoot();
     DBG("Gnash VM initialized.\n");
+    
+    // Register this plugin as listener for FsCommands from the core
+    // (movie_root)
+#if 0
+    /* Commenting out for now as registerFSCommandCallback() has changed. */
+    root.registerFSCommandCallback(FSCommand_callback);
+#endif
+    
+    // Register a static function to handle ActionScript events such
+    // as Mouse.hide, Stage.align etc.
+    // root.registerEventCallback(&staticEventHandlingFunction);
 
     md->completeLoad();
     DBG("Movie loaded.\n");
 
-    std::auto_ptr<gnash::movie_instance> mr(md->create_movie_instance());
+    std::auto_ptr<gnash::Movie> mr(md->createMovie());
     mr->setVariables(_flashVars);
     root.setRootMovie(mr.release());
     root.set_display_viewport(0, 0, _width, _height);
     root.set_background_alpha(1.0f);
-    gnash::movie_instance* mi = root.getRootMovie();
+    gnash::Movie* mi = root.getRootMovie();
     DBG("Movie instance created.\n");
 
     ShowWindow(_window, SW_SHOW);
@@ -499,7 +510,7 @@ nsPluginInstance::threadMain(void)
         // DBG("Ensuring frame is loaded.\n");
         root.get_movie_definition()->ensure_frame_loaded(tot_frames);
         // DBG("Setting play state to PLAY.\n");
-        root.set_play_state(gnash::sprite_instance::PLAY);
+        root.set_play_state(gnash::MovieClip::PLAY);
 
         if (old_mouse_x != mouse_x || old_mouse_y != mouse_y) {
             old_mouse_x = mouse_x;
@@ -567,7 +578,7 @@ nsPluginInstance::getVersion()
 }
 
 void
-nsPluginInstance::FSCommand_callback(gnash::sprite_instance* movie, const std::string& command, const std::string& args)
+nsPluginInstance::FSCommand_callback(gnash::MovieClip* movie, const std::string& command, const std::string& args)
 // For handling notification callbacks from ActionScript.
 {
     gnash::log_debug(_("FSCommand_callback(%p): %s %s"), (void*) movie, command, args);
