@@ -1,5 +1,5 @@
 /* 
- *   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+ *   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
 
 #include "Range2d.h"
 #include "GnashKey.h" // for namespace key
-#include "types.h" // for rgba class
+#include "RGBA.h" // for rgba class (composition)
 
 #include <iostream> 
 
