@@ -22,7 +22,7 @@
 
 #include "MovieTester.h"
 #include "MovieClip.h"
-#include "character.h"
+#include "DisplayObject.h"
 #include "DisplayList.h"
 #include "log.h"
 #include "VM.h"
@@ -46,7 +46,7 @@ main(int /*argc*/, char** /*argv*/)
 	MovieClip* root = tester.getRootMovie();
 	assert(root);
 
-    VM& vm = root->getVM();
+    VM& vm = getVM(*root);
     string_table& st = vm.getStringTable();
 
     const size_t framecount = root->get_frame_count();
