@@ -1,5 +1,5 @@
 /* 
- *   Copyright (C) 2007, 2008 Free Software Foundation, Inc.
+ *   Copyright (C) 2007, 2008, 2009 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -38,6 +38,7 @@
 const char* mediadir=".";
 char filename[256];
 char filename2[256];
+char filename3[256];
 
 
 int
@@ -49,10 +50,13 @@ main(int argc, char** argv)
   SWFDisplayItem item;
   SWFAction a;
   SWFAction b;
-  char buffer_a[1028];
-  char buffer_b[2048];
+  char buffer_a[2024];
+  char buffer_b[2024];
+  char buffer_c[2024];
 
-  int video_width = 128;
+  // This is different from the real video width to make sure that
+  // Video.width returns the actual width (128).
+  int video_width = 130;
   int video_height = 96;
 
   if ( argc>1 ) mediadir=argv[1];
@@ -64,6 +68,7 @@ main(int argc, char** argv)
 	
   sprintf(filename, "%s/square.flv", mediadir);
   sprintf(filename2, "%s/square.ogg", mediadir);
+  sprintf(filename3, "%s/audio_timewarp.flv", mediadir);
 
   // Some online examples...
   //
@@ -84,7 +89,7 @@ main(int argc, char** argv)
   	"nc=new NetConnection();"
 	"check(!nc.isConnected, 'newly created NetConnection is not connected');"
 	"nc.connect(null);"
-	"xcheck(nc.isConnected, 'NetConnection is connected after .connect(null)');"
+	"check(nc.isConnected, 'NetConnection is connected after .connect(null)');"
 	"check(!NetStream.prototype.hasOwnProperty('currentFPS'));" // version 7 here
 	"xcheck(!NetStream.prototype.hasOwnProperty('currentFps'));"
 	"stream = new NetStream();"
@@ -95,6 +100,7 @@ main(int argc, char** argv)
 	"stream.bytesTotal = 'string';"
 	"check_equals ( typeof(stream.bytesTotal), 'number' );"
 	"stream2 = new NetStream(nc);"
+	"stream3 = new NetStream(nc);"
 	, OUTPUT_VERSION);
 
   sprintf(buffer_b,
@@ -124,7 +130,9 @@ main(int argc, char** argv)
 	"	pc.onEnterFrame = function() {"
 	"		pcp._xscale = 100*(this.stream.bytesLoaded/this.stream.bytesTotal);"
 	"	};"
-	"};"
+	"};");
+
+    sprintf(buffer_c,
 	"MovieClip.prototype.addBufferLoadedProgress = function(v, s) {"
 	"	var nam = 'blprogress_'+v;"
 	"	var dep = this.getNextHighestDepth();"
@@ -161,13 +169,14 @@ main(int argc, char** argv)
 	"};"
 	"stream.play('%s');"
 	"stream2.play('%s');"
+	"stream3.play('%s');"
 	"stream.pause(true);" 
 	"stream.paused=true;"
 	"_root.metadataNotified=0;"
 	"_root.startNotified=0;"
 	"_root.stopNotified=0;"
 	"stop();",
-	filename, filename2);
+	filename, filename2, filename3);
 
   Ming_init();
   Ming_useSWFVersion (OUTPUT_VERSION);
@@ -199,6 +208,11 @@ main(int argc, char** argv)
   SWFDisplayItem_moveTo(item, 400, 0);
   SWFDisplayItem_setName(item, "video2");
 
+  stream = newSWFVideoStream();
+  SWFVideoStream_setDimension(stream, video_width, video_height);
+  item = SWFMovie_add(mo, (SWFBlock)stream);
+  SWFDisplayItem_setName(item, "video3");
+
   a = newSWFAction(buffer_a);
   if(a == NULL) return -1;
   SWFMovie_add(mo, (SWFBlock)a);
@@ -222,22 +236,22 @@ main(int argc, char** argv)
   // create a movieclip to use as sound controller
   add_actions(mo, "createEmptyMovieClip('dv1', 1);");
 
-  // attach Sound s1 to character dv1
+  // attach Sound s1 to DisplayObject dv1
   add_actions(mo, "s1 = new Sound(dv1);");
   // Sound.getVolume fetches volume from dv1 (see s2 below)
   check_equals(mo, "s1.getVolume()", "100");
 
-  // Change volume of Sound s1 (will change volume of attached character, see below)
+  // Change volume of Sound s1 (will change volume of attached DisplayObject, see below)
   add_actions(mo, "s1.setVolume(1000);");
   check_equals(mo, "s1.getVolume()", "1000"); 
 
-  // attach Sound s2 to character dv2
+  // attach Sound s2 to DisplayObject dv2
   add_actions(mo, "s2 = new Sound(dv1);");
   // Sound s2 finds volume of dv1 being 1000
   check_equals(mo, "s2.getVolume()", "1000");
 
   // This shows that setVolume/getVolume make callbacks
-  // to attached character as changing volume of one Sound
+  // to attached DisplayObject as changing volume of one Sound
   // influence the other attached sound...
   add_actions(mo, "s2.setVolume(5);");
   check_equals(mo, "s2.getVolume()", "5"); 
@@ -250,7 +264,7 @@ main(int argc, char** argv)
   check_equals(mo, "s2.getVolume()", "-20"); 
   check_equals(mo, "s1.getVolume()", "-20");
 
-  // If the attached-to character gets unloaded,
+  // If the attached-to DisplayObject gets unloaded,
   // getVolume returns undefined...
   add_actions(mo, "dv1.removeMovieClip();");
   check_equals(mo, "typeof(s2.getVolume())", "'undefined'"); 
@@ -267,7 +281,7 @@ main(int argc, char** argv)
   check_equals(mo, "s2.getVolume()", "80"); 
   check_equals(mo, "s1.getVolume()", "80");
 
-  // And you can attach a Sound to any character, not
+  // And you can attach a Sound to any DisplayObject, not
   // just MovieClip ones.
   // This is against a video instance
   add_actions(mo, "s1 = new Sound(video); s2 = new Sound(video);");
@@ -294,12 +308,28 @@ main(int argc, char** argv)
                 );
 
   //------------------------------------------
-  // Now attach video to the video characters
+  // Now attach video to the video DisplayObjects
   //------------------------------------------
-  
+  check(mo, "Video.prototype.hasOwnProperty('attachVideo')");
+  check(mo, "Video.prototype.hasOwnProperty('smoothing')");
+  check(mo, "Video.prototype.hasOwnProperty('deblocking')");
+  check(mo, "Video.prototype.hasOwnProperty('clear')");
+  check(mo, "Video.prototype.hasOwnProperty('height')");
+  check(mo, "Video.prototype.hasOwnProperty('width')");
+  check_equals(mo, "video.height", "0");
+  check_equals(mo, "video.width", "0");
+  check_equals(mo, "video2.height", "0");
+  check_equals(mo, "video2.width", "0");
+
   add_actions(mo, "video.attachVideo(stream);"); 
   add_actions(mo, "video2.attachVideo(stream2);"); 
+  add_actions(mo, "video3.attachVideo(stream3);"); 
   
+  check_equals(mo, "video.height", "0");
+  check_equals(mo, "video.width", "0");
+  check_equals(mo, "video2.height", "0");
+  check_equals(mo, "video2.width", "0");
+
   // currentFps (read-only)
   check_equals (mo, "typeof(stream.currentFps)", "'number'" );
   add_actions(mo, "stream.currentFps = 'string';");
@@ -352,6 +382,9 @@ main(int argc, char** argv)
   b = newSWFAction(buffer_b);
   if(b == NULL) return -1;
   SWFMovie_add(mo, (SWFBlock)b);
+  b = newSWFAction(buffer_c);
+  if(b == NULL) return -1;
+  SWFMovie_add(mo, (SWFBlock)b);
  
   check_equals (mo, "stream.currentFps", "0" );
  
@@ -376,6 +409,32 @@ main(int argc, char** argv)
   check_equals(mo, "video._yscale", "100");
   check_equals(mo, "video._rotation", "0");
   check_equals(mo, "video._target", "'/video'");
+  check_equals(mo, "video.height", "0");
+  check_equals(mo, "video.width", "0");
+  check_equals(mo, "video2.height", "0");
+  check_equals(mo, "video2.width", "0");
+
+  check(mo, "Video.prototype.hasOwnProperty('attachVideo')");
+  check(mo, "Video.prototype.hasOwnProperty('smoothing')");
+  check(mo, "Video.prototype.hasOwnProperty('deblocking')");
+  check(mo, "Video.prototype.hasOwnProperty('clear')");
+  check(mo, "Video.prototype.hasOwnProperty('height')");
+  check(mo, "Video.prototype.hasOwnProperty('width')");
+
+  check(mo, "!Video.prototype.hasOwnProperty('_alpha')");
+  check(mo, "!Video.prototype.hasOwnProperty('_height')");
+  check(mo, "!Video.prototype.hasOwnProperty('_name')");
+  check(mo, "!Video.prototype.hasOwnProperty('_parent')");
+  check(mo, "!Video.prototype.hasOwnProperty('_rotation')");
+  check(mo, "!Video.prototype.hasOwnProperty('_visible')");
+  check(mo, "!Video.prototype.hasOwnProperty('_width')");
+  check(mo, "!Video.prototype.hasOwnProperty('_x')");
+  check(mo, "!Video.prototype.hasOwnProperty('_xmouse')");
+  check(mo, "!Video.prototype.hasOwnProperty('_xscale')");
+  check(mo, "!Video.prototype.hasOwnProperty('_y')");
+  check(mo, "!Video.prototype.hasOwnProperty('_ymouse')");
+  check(mo, "!Video.prototype.hasOwnProperty('_yscale')");
+  check(mo, "!Video.prototype.hasOwnProperty('_xmouse')");
 
   add_actions(mo,
 		"video._x = 100;"
@@ -399,7 +458,8 @@ main(int argc, char** argv)
   check_equals(mo, "typeof(video.getBounds)", "'undefined'");
 
   SWFMovie_add(mo, (SWFBlock)newSWFAction(
-
+        "note('The video on the right is an OGG theora stream. It will appear "
+        "in Gnash, but not the Adobe player');"
 		"_root.onKeyDown = function() {"
 		" 	_root.note(' bufferLength:'+stream.bufferLength+"
 		" 		' bytesLoaded:'+stream.bytesLoaded+"
@@ -423,6 +483,8 @@ main(int argc, char** argv)
 		"};"
 		"Key.addListener(_root);"
 
+		"\n"
+
 		"stream.onStatus = function(info) {"
 
 		"  if ( ! _root.enumerableStatusInfoChecked ) {"
@@ -447,12 +509,16 @@ main(int argc, char** argv)
 		"    info.level = backup;"
 		"  }"
 
+		"\n"
+
 		// Ignore Buffer.Flush for now
 		"  if ( info.code == 'NetStream.Buffer.Flush' ) return; "
 
 		// Print some info
 		" _root.note('onStatus('+info.code+') called'); "
 
+		"\n"
+
 		" if ( info.code == 'NetStream.Play.Start' )"
 		" {"
 		"	check(!_root.startNotified, 'No duplicated Play.Start notification');"
@@ -509,7 +575,7 @@ main(int argc, char** argv)
 		" check(_root.startNotified, 'onMetaData should be notified after Play.Start');"
 		" check_equals(arguments.length, 1, 'single argument');"
 		" check(info instanceof Array, 'onMetaData argument sent from square.flv should be instanceof Array');"
-		" check_equals(info.length, 11);" // it's an array so we can check length. gnash fails by discarding the count element of ECMA_ARRAY AMF type
+		" check_equals(info.length, 11);" 
 
 		// Test enumeration
 		" var enu = new Array;"
@@ -604,18 +670,33 @@ main(int argc, char** argv)
 		" check_equals(info.duration, 'changed');" // can be overridden
 		" delete info.duration;"
 		" check(!info.hasOwnProperty('duration'), 'metadata duration can be deleted');"
-
 		" _root.note('1. Press \"p\" key');"
-
 		"};"
 		));
 
   SWFMovie_nextFrame(mo);
 
+  check_equals(mo, "video.height", "96");
+  check_equals(mo, "video.width", "128");
+
+
+  // See https://savannah.gnu.org/bugs/?26687
+  SWFMovie_add(mo, (SWFBlock)newSWFAction(
+  //"note('VM.time:'+getTimer()+' stream3.time:'+stream3.time);"
+  "check(stream3.time > 4150, 'stream3.time '+stream3.time+' > 4150');"
+  ));
+
+
+  //add_actions(mo, "note('This is an OGG stream, so the Adobe player "
+  //    "will fail the next two tests.');");
+  // gstreamer also fails, so let's not test this.
+  //check_equals(mo, "video2.height", "96");
+  //check_equals(mo, "video2.width", "128");
+
   check_equals(mo, "metadataNotified", "1");
   check_equals(mo, "stopNotified", "2");
   check_equals(mo, "startNotified", "1");
-  SWFMovie_add(mo, (SWFBlock)newSWFAction("totals(142); stop(); end_of_test=true;"));
+  SWFMovie_add(mo, (SWFBlock)newSWFAction("totals(183); stop(); end_of_test=true;"));
 
   SWFMovie_nextFrame(mo);
 
