@@ -1,5 +1,5 @@
 /* 
- *   Copyright (C) 2007, 2008 Free Software Foundation, Inc.
+ *   Copyright (C) 2007, 2008, 2009 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,15 +23,16 @@
 #include "MovieTester.h"
 #include "VM.h"
 #include "string_table.h"
-#include "sprite_instance.h"
-#include "character.h"
+#include "MovieClip.h"
+#include "DisplayObject.h"
 #include "DisplayList.h"
 #include "log.h"
+#include "GnashSleep.h"
 
 #include "check.h"
 #include <string>
 #include <cassert>
-#include <unistd.h>
+#include "GnashSystemIOHeaders.h"
 
 using namespace gnash;
 using namespace std;
@@ -48,7 +49,7 @@ main(int /*argc*/, char** /*argv*/)
 	gnash::LogFile& dbglogfile = gnash::LogFile::getDefaultInstance();
 	dbglogfile.setVerbosity(1);
 
-	sprite_instance* root = tester.getRootMovie();
+	MovieClip* root = tester.getRootMovie();
 	assert(root);
 
 	if ( ! tester.canTestVideo() )
@@ -75,7 +76,8 @@ main(int /*argc*/, char** /*argv*/)
         // data and trigger notifications
 		// needs more analisys to find a good way for doing this..
         // TODO: sleep less ! GnashSleep !
-		sleep(1);
+        gnashSleep(10000); // 10 milliseconds should be enough for loading
+		//sleep(1);
 	}
 
 	cout << "Pressing space" << endl;
@@ -89,7 +91,8 @@ main(int /*argc*/, char** /*argv*/)
 		// sleep to give the NetStream a chance to load data and trigger notifications
 		// needs more analisys to find a good way for doing this..
         // TODO: sleep less !!
-		sleep(1);
+        gnashSleep(10000); // 10 milliseconds should be enough for loading
+		//sleep(1);
 	}
 
 	// Consistency check 
