@@ -1,5 +1,5 @@
 /* 
- *   Copyright (C) 2007, 2008 Free Software Foundation, Inc.
+ *   Copyright (C) 2007, 2008, 2009 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,13 +21,13 @@
 #define INPUT_FILENAME "PrototypeEventListeners.swf"
 
 #include "MovieTester.h"
-#include "sprite_instance.h"
-#include "character.h"
-#include "dlist.h"
+#include "MovieClip.h"
+#include "DisplayObject.h"
+#include "DisplayList.h"
 #include "log.h"
 #include "VM.h"
 #include "string_table.h"
-#include "gnash.h" // gnash::key::code
+#include "GnashKey.h" // gnash::key::code
 
 #include "check.h"
 #include <string>
@@ -47,11 +47,11 @@ main(int /*argc*/, char** /*argv*/)
 	gnash::LogFile& dbglogfile = gnash::LogFile::getDefaultInstance();
 	dbglogfile.setVerbosity(1);
 
-	sprite_instance* root = tester.getRootMovie();
+	MovieClip* root = tester.getRootMovie();
 	assert(root);
 
 	// for variables lookup (consistency checking)
-	string_table& st = root->getVM().getStringTable();
+	string_table& st = getStringTable(*root);
 
 	check_equals(root->get_frame_count(), 2);
 	check_equals(root->get_current_frame(), 0);
@@ -77,7 +77,7 @@ main(int /*argc*/, char** /*argv*/)
 
 	if ( endOfTestFound )
 	{
-		cerr << eot.to_debug_string() << endl;
+		cerr << eot.toDebugString() << endl;
 		check_equals(eot.to_bool(), true);
 	}
 	else
