@@ -22,8 +22,9 @@
 
 #include "MovieTester.h"
 #include "MovieClip.h"
-#include "character.h"
+#include "DisplayObject.h"
 #include "DisplayList.h"
+#include "TextField.h"
 #include "log.h"
 
 #include "check.h"
@@ -34,7 +35,7 @@ using namespace gnash;
 using namespace std;
 
 void
-test_mouse_activity(MovieTester& tester, const character* text, const character* text2, bool covered, bool enabled)
+test_mouse_activity(MovieTester& tester, const TextField* text, const TextField* text2, bool covered, bool enabled)
 {
 	rgba red(255,0,0,255);
 	rgba covered_red(127,126,0,255); // red, covered by 50% black
@@ -182,7 +183,7 @@ test_mouse_activity(MovieTester& tester, const character* text, const character*
 
 	tester.movePointerTo(39, 60);
 
-	// The following might be correct, as the character still catches releaseOutside events
+	// The following might be correct, as the DisplayObject still catches releaseOutside events
 	//check(tester.isMouseOverMouseEntity());
 	tester.depressMouseButton();
 
@@ -210,21 +211,24 @@ main(int /*argc*/, char** /*argv*/)
 	check_equals(root->get_frame_count(), 5);
 	check_equals(root->get_current_frame(), 0);
 
-	const character* text = tester.findDisplayItemByName(*root, "textfield");
+	const TextField* text = dynamic_cast<const TextField*>(
+		tester.findDisplayItemByName(*root, "textfield"));
 	check(text);
 
-	const character* text2 = tester.findDisplayItemByName(*root, "textfield2");
+	const TextField* text2 = dynamic_cast<const TextField*>(
+		tester.findDisplayItemByName(*root, "textfield2"));
 	check(text2);
 
-	const character* text3 = tester.findDisplayItemByName(*root, "textfield3");
+	const TextField* text3 = dynamic_cast<const TextField*>(
+		tester.findDisplayItemByName(*root, "textfield3"));
 	check(text3);
 
 	tester.advance();
 	check_equals(root->get_current_frame(), 1);
 
-	const character* mc1 = tester.findDisplayItemByName(*root, "square1");
+	const DisplayObject* mc1 = tester.findDisplayItemByName(*root, "square1");
 	check(mc1);
-	check_equals(mc1->get_depth(), 2+character::staticDepthOffset);
+	check_equals(mc1->get_depth(), 2+DisplayObject::staticDepthOffset);
 
 
 	check_equals(string(text->get_text_value()), idleString);
@@ -237,8 +241,8 @@ main(int /*argc*/, char** /*argv*/)
 
 	for (size_t fno=1; fno<root->get_frame_count(); fno++)
 	{
-		const character* square_back = tester.findDisplayItemByDepth(*root, 1+character::staticDepthOffset);
-		const character* square_front = tester.findDisplayItemByDepth(*root, 3+character::staticDepthOffset);
+		const DisplayObject* square_back = tester.findDisplayItemByDepth(*root, 1+DisplayObject::staticDepthOffset);
+		const DisplayObject* square_front = tester.findDisplayItemByDepth(*root, 3+DisplayObject::staticDepthOffset);
 
 		switch (fno)
 		{
@@ -259,7 +263,8 @@ main(int /*argc*/, char** /*argv*/)
 		check_equals(root->get_current_frame(), fno);
 
 		info (("testing mouse activity in frame %d", root->get_current_frame()));
-		test_mouse_activity(tester, text, text2, square_front!=NULL, fno != root->get_frame_count()-1);
+		test_mouse_activity(tester, text, text2,
+				square_front!=NULL, fno != root->get_frame_count()-1);
 
 		// TODO: test key presses !
 		//       They seem NOT to trigger immediate redraw
@@ -269,7 +274,7 @@ main(int /*argc*/, char** /*argv*/)
 	}
 
 	// last advance should not restart the loop (it's in STOP mode)
-        check_equals(root->get_play_state(), MovieClip::STOP);
+        check_equals(root->getPlayState(), MovieClip::PLAYSTATE_STOP);
 	check_equals(root->get_current_frame(), 4);
 
 }
