@@ -1,6 +1,6 @@
 /***********************************************************************
  *
- *   Copyright (C) 2007 Free Software Foundation, Inc.
+ *   Copyright (C) 2007, 2009 Free Software Foundation, Inc.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -36,6 +36,7 @@
 
 #include <stdlib.h>
 #include <stdio.h>
+#include <string.h>
 #include <ming.h>
 
 // In version 7 loadMovie doesn't work !!
@@ -78,12 +79,18 @@ add_clip(SWFMovie mo, char* file, char* name,
 	SWFDisplayItem_setName(it, name);
 	SWFDisplayItem_moveTo(it, x, y);
 
+	// last portion of the url (before the slash)
+	char* fname = strrchr(url, '/');
+	if ( fname ) fname++;
+	else fname = url;
+
 	/* "Click" handler */
-	sprintf(action, " \
-%s.onPress = function () { \
-	coverart.loadMovie('%s'); \
-}; \
-", name, url);
+	snprintf(action,  1023,
+		"%s.onPress = function () { "
+		"	coverart.loadMovie('%s');"
+		"	_level0.expectLoaded = '%s';" 
+		"};"
+		,name, url, fname);
 
 	ac = compileSWFActionCode(action);
 
@@ -97,6 +104,8 @@ add_coverart(SWFMovie mo, int x, int y)
 	SWFFillStyle fstyle;
 	SWFMovieClip mc_coverart;
 	SWFDisplayItem it;
+#define BUFSIZE 1024
+	char buf[BUFSIZE];
 
 	sh_coverart = newSWFShape();
 	fstyle = SWFShape_addSolidFillStyle(sh_coverart, 0,0,0,255);
@@ -109,10 +118,53 @@ add_coverart(SWFMovie mo, int x, int y)
 
 	mc_coverart = newSWFMovieClip();
 	SWFMovieClip_add(mc_coverart, (SWFBlock)sh_coverart);
+	SWFMovieClip_add(mc_coverart, (SWFBlock)newSWFAction(
+		"_root.xcheck(getBytesLoaded() < _root.getBytesLoaded());"
+		"_root.xcheck(getBytesTotal() < _root.getBytesTotal());"
+	));
 	SWFMovieClip_nextFrame(mc_coverart); /* showFrame */
+
 	it = SWFMovie_add(mo, (SWFBlock)mc_coverart);
 	SWFDisplayItem_setName(it, "coverart"); 
 	SWFDisplayItem_moveTo(it, x, y);
+
+	snprintf(buf, BUFSIZE,
+		//"_root.note(this+'.on(RollOver) ... ');"
+		//"_level0.coverart.onUnload = function() {"
+		//"	_root.note(this+'.onUnload called');"
+		//"};"
+		"_level0.coverart.onMouseDown = function() {"
+        "t = this.createEmptyMovieClip('tc', 8);"
+        "_root.check(tc instanceof MovieClip);"
+        "_root.check(t instanceof MovieClip);"
+		//"  _root.note('_url is '+this._url);"
+		"  var lastUrlComponent = this._url.substring(this._url.lastIndexOf('/')+1);"
+		//"  _root.note('last component of _url is '+lastUrlComponent);"
+		"  _root.check_equals(lastUrlComponent, _level0.expectLoaded, '%s:%d');"
+		"  _root.check_equals(this.getDepth(), -16376);"
+		"  _root.check_equals(this.getBytesLoaded(), this.getBytesTotal());"
+		"  _root.check(this.getBytesLoaded() > 0);" /* assuming something was loaded here */
+		"  _root.note('bytesLoaded: '+this.getBytesLoaded());"
+		"  if ( Key.isDown(Key.SHIFT) ) { "
+		"	trace('SHIFT-click on coverart...');"
+		//"	_root.note('SHIFT-click on coverart...');"
+		"	_root.totals(26, '"__FILE__"');"
+		"	_root.END_OF_TEST = true;"
+		" }"
+		"  else _root.note('2 tests run');"
+		"};"
+		, __FILE__, __LINE__);
+
+	/*printf("%s", buf);*/
+
+	SWFDisplayItem_addAction(it, compileSWFActionCode(buf),
+		SWFACTION_ROLLOVER);
+
+	SWFDisplayItem_addAction(it, compileSWFActionCode(
+		"delete _level0.coverart.onMouseDown;"
+		),
+		SWFACTION_ROLLOUT);
+
 }
 
 SWFTextField
@@ -146,6 +198,8 @@ main(int argc, char** argv)
 {
 	SWFMovie mo;
 	SWFMovieClip dejagnuclip;
+	SWFDisplayItem it;
+
 	char file_lynch[256];
 	char file_green[256];
 	char file_offspring[256];
@@ -198,9 +252,9 @@ main(int argc, char** argv)
 	 *****************************************************/
 
 	dejagnuclip = get_dejagnu_clip((SWFBlock)get_default_font(mediadir), 10, 0, 0, 800, 600);
-	//it = SWFMovie_add(mo, (SWFBlock)dejagnuclip);
-	//SWFDisplayItem_moveTo(it, 0, 200);
-	//SWFMovie_nextFrame(mo); 
+	it = SWFMovie_add(mo, (SWFBlock)dejagnuclip);
+	SWFDisplayItem_moveTo(it, 0, 270);
+	SWFMovie_nextFrame(mo); 
 
 	/*****************************************************
 	 *
@@ -248,6 +302,13 @@ main(int argc, char** argv)
 				" art.clear(); "
 				));
 
+	add_button(mo, 50, 240, "Unload", newSWFAction(
+		   		"_root.coverart.onUnload = function() {"
+				"  _root.note(this+'.onUnload called');"
+				"};"	
+				" _root.coverart.unloadMovie();"
+				));
+
 	/*****************************************************
 	 *
 	 * Add the coverart clip
@@ -256,9 +317,17 @@ main(int argc, char** argv)
 
 	puts("Adding coverart");
 
-	add_coverart(mo, 500, 200);
+	add_coverart(mo, 600, 100);
 
-	add_actions(mo, "stop();");
+	add_actions(mo,
+		"note('Click on each image to load it into the container on the right.');"
+		"note('After each load, click on the container.');"
+		"note('Finally, shift-click on the container to get results printed.');"
+		"_level0.expectLoaded = 'loadMovieTest.swf';" 
+		// TODO: add self-contained tests after each load
+		//       like for the DragAndDropTest.as movie
+		"stop();"
+	);
 	SWFMovie_nextFrame(mo); /* showFrame */
 
 	/*****************************************************
